
#include <boost/python/numpy.hpp>
#include <boost/python.hpp>
#include <iostream>
#include <vector>


#include "Output/ExportMacros.h"
#include "Output/PythonIO.h"
typedef IO_<PythonIO> IO;
#define FromIO(a) IO:: a
Redeclare2Types(FromIO,Msg,WarnMsg)

#include "DispatchAndRun.h"
  
struct HFMIO {
    Redeclare3Types(FromIO,ScalarType,KeyCRef,ndarray);
    IO io;
    void Run(){::Run(io);}
    
    // Input output
    std::string GetString(KeyCRef key) const {return io.GetString(key);}
    void SetString(KeyCRef key, const std::string & val){io.SetString(key,val);}
    ScalarType GetScalar(KeyCRef key) const {return io.Get<ScalarType>(key);}
    void SetScalar(KeyCRef key, ScalarType val){io.Set<ScalarType>(key,val);}
    ndarray GetArray(KeyCRef key) const {return io.PyGetArray(key);}
    void SetArray(KeyCRef key, ndarray val) {io.PySetArray(key,val);}
    
    // Field inspection
    std::string GetComputedKeys() {return io.GetComputedKeys();}
    bool HasField(KeyCRef key) const {return io.HasField(key);}
    bool EraseField(KeyCRef key) {return io.EraseField(key);}
};


void translator(JMMCppException const & e) {
    PyErr_SetString(PyExc_UserWarning, (std::string("HFMIO Exception caught ! ")+e.what()).c_str());}

BOOST_PYTHON_MODULE(PythonModuleName)
{
    boost::python::numpy::initialize(); // Required !
    boost::python::register_exception_translator<JMMCppException>(translator);

    boost::python::class_<HFMIO, boost::noncopyable>("HFMIO")
    .def("Run",&HFMIO::Run)
    .def("GetString",&HFMIO::GetString)
    .def("SetString",&HFMIO::SetString)
    .def("GetScalar",&HFMIO::GetScalar)
    .def("SetScalar",&HFMIO::SetScalar)
    .def("GetArray", &HFMIO::GetArray)
    .def("SetArray", &HFMIO::SetArray)
    .def("GetComputedKeys", &HFMIO::GetComputedKeys)
    .def("HasField", &HFMIO::HasField)
    .def("EraseField",&HFMIO::EraseField)
    ;
}
