(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     23804,        614]
NotebookOptionsPosition[     23399,        595]
NotebookOutlinePosition[     23758,        611]
CellTagsIndexPosition[     23715,        608]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Common definitions for Hamilton Fast Marching", "Title",
 CellChangeTimes->{{3.696873305660097*^9, 3.696873310883669*^9}, {
  3.699380210329989*^9, 3.699380270276122*^9}}],

Cell["\<\
The following definitions are required for the mathematica example files
Important: Please correctly setup the directory containing the FileHFM \
executable !\
\>", "Text",
 CellChangeTimes->{
  3.696873289898459*^9, {3.699380315019144*^9, 3.699380385776102*^9}, {
   3.6993805332465754`*^9, 3.6993805473324213`*^9}, {3.699380580213957*^9, 
   3.699380688131352*^9}, {3.6993807281014233`*^9, 3.699380750658681*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"!!", " ", 
    RowBox[{"To", " ", 
     RowBox[{"user", ":", " ", 
      RowBox[{"Change", " ", 
       RowBox[{"this", " ", "!!"}]}]}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
   "binDirectoryHFM", " ", "=", " ", 
    "\"\</Users/mirebeau/bin/2018/FileHFM/Release\>\""}], ";"}]}]], "Input",
 CellChangeTimes->{{3.699386983914554*^9, 3.699386986480248*^9}, {
  3.7017725659010487`*^9, 3.701772568283402*^9}, {3.729539345011047*^9, 
  3.7295393462929087`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"execPrefix", "=", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "\[Equal]", "\"\<Windows\>\""}], ",", 
      "\"\<\>\"", ",", "\"\<./\>\""}], "]"}]}], ";"}], "\[IndentingNewLine]", 
  
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"dynP", " ", "is", " ", "Dynamic", " ", "partition"}], ",", " ", 
    RowBox[{
     RowBox[{"from", " ", 
      RowBox[{"http", ":"}]}], "//", 
     RowBox[{
      RowBox[{
       RowBox[{"mathematica", ".", "stackexchange", ".", "com"}], "/", "a"}], 
      "/", "7512."}]}]}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"dynP", "[", 
    RowBox[{"l_", ",", "p_"}], "]"}], ":=", 
   RowBox[{"MapThread", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"l", "[", 
       RowBox[{"[", 
        RowBox[{"#", ";;", "#2"}], "]"}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"{", "0", "}"}], "~", "Join", "~", 
           RowBox[{"Most", "@", "#"}]}], "+", "1"}], ",", "#"}], "}"}], "&"}],
       "@", 
      RowBox[{"Accumulate", "@", "p"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MergeRules", "[", 
    RowBox[{"provided_", ",", "default_"}], "]"}], ":=", 
   RowBox[{"With", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"l", "=", 
       RowBox[{"First", "/@", "provided"}]}], "}"}], ",", 
     RowBox[{"Join", "[", 
      RowBox[{"provided", ",", 
       RowBox[{"Select", "[", 
        RowBox[{"default", ",", 
         RowBox[{
          RowBox[{"Not", "@", 
           RowBox[{"MemberQ", "[", 
            RowBox[{"l", ",", 
             RowBox[{"First", "@", "#"}]}], "]"}]}], "&"}]}], "]"}]}], 
      "]"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"TablePlot", "[", 
     RowBox[{"t_", ",", 
      RowBox[{"opts_:", 
       RowBox[{"{", "}"}]}]}], "]"}], ":=", 
    RowBox[{"ArrayPlot", "@@", 
     RowBox[{"Prepend", "[", 
      RowBox[{"opts", ",", 
       RowBox[{"Reverse", "@", 
        RowBox[{"Transpose", "@", "t"}]}]}], "]"}]}]}], ";"}], " ", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RunExec", "[", 
     RowBox[{"exec_", ",", "params_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"refDir", "=", 
         RowBox[{"Directory", "[", "]"}]}], ",", "out", ",", "geo"}], "}"}], 
      ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"SetDirectory", "[", "binDirectoryHFM", "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"RulesToRaw", "[", 
          RowBox[{"params", ",", "\"\<input\>\""}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Print", "[", "\"\<Failed to export params\>\"", "]"}], ";", 
          RowBox[{"Return", "[", "]"}], ";"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Run", "[", 
        RowBox[{"execPrefix", "<>", "exec", "<>", "\"\< > log.txt\>\""}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"out", "=", 
        RowBox[{"RawToRules", "[", "\"\<output\>\"", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"AppendTo", "[", 
        RowBox[{"out", ",", 
         RowBox[{"\"\<log\>\"", "\[Rule]", 
          RowBox[{"Import", "[", "\"\<log.txt\>\"", "]"}]}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"SetDirectory", "[", "refDir", "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"MemberQ", "[", 
          RowBox[{
           RowBox[{"First", "/@", "out"}], ",", "\"\<geodesicPoints\>\""}], 
          "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"AppendTo", "[", 
          RowBox[{"out", ",", 
           RowBox[{"\"\<geodesics\>\"", "\[Rule]", 
            RowBox[{"dynP", "[", 
             RowBox[{
              RowBox[{"\"\<geodesicPoints\>\"", "/.", "out"}], ",", 
              RowBox[{"Round", "[", 
               RowBox[{"\"\<geodesicLengths\>\"", "/.", "out"}], "]"}]}], 
             "]"}]}]}], "]"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"Return", "[", "out", "]"}], ";"}]}], "\[IndentingNewLine]", 
     "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"--", 
      RowBox[{"--", 
       RowBox[{"--", 
        RowBox[{"--", " ", "Copied"}]}]}]}], " ", "from", " ", 
     RowBox[{"FileIO", ".", "nb"}]}], ",", " ", 
    RowBox[{"in", " ", 
     RowBox[{"JMMCPPLibs", "/", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Output", " ", "--"}], "--"}], "--"}], "--"}]}]}]}], "*)"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RulesToRaw", "[", 
     RowBox[{"l_List", ",", "fileName_String"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
       "lStr", " ", ",", " ", "lNum", ",", " ", "lArr", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"str", "=", "\"\<\>\""}], ",", 
        RowBox[{"data", "=", 
         RowBox[{"{", "}"}]}]}], "\[IndentingNewLine]", "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"bad", "=", 
           RowBox[{"Select", "[", 
            RowBox[{"l", ",", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "@", "#"}], "!=", "2"}], "&"}]}], "]"}]}], 
          "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"bad", "\[NotEqual]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<SaveToFile Error : Elements of l are not pairs\\n\>\"", ",", 
              RowBox[{"Short", "[", 
               RowBox[{"bad", ",", "2"}], "]"}]}], "]"}], ";", 
            "\[IndentingNewLine]", " ", 
            RowBox[{"Return", "[", "True", "]"}], ";"}]}], "]"}]}], "]"}], 
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"bad", "=", 
           RowBox[{"Select", "[", 
            RowBox[{"l", ",", 
             RowBox[{
              RowBox[{"!", 
               RowBox[{"StringQ", "@", 
                RowBox[{"First", "@", "#"}]}]}], "&"}]}], "]"}]}], "}"}], ",",
          "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"bad", "\[NotEqual]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<SaveToFile Error : Keys are not strings\\n\>\"", ",", 
              RowBox[{"Short", "[", 
               RowBox[{"bad", ",", "2"}], "]"}]}], "]"}], ";", 
            "\[IndentingNewLine]", " ", 
            RowBox[{"Return", "[", "True", "]"}], ";"}]}], "]"}]}], "]"}], 
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"With", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"bad", "=", 
           RowBox[{"Select", "[", 
            RowBox[{"l", ",", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"!", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"StringQ", "@", "#"}], "||", 
                   RowBox[{"NumericQ", "@", "#"}], "||", 
                   RowBox[{"ArrayQ", "[", 
                    RowBox[{"#", ",", "_", ",", "NumericQ"}], "]"}], "||", 
                   RowBox[{"#", "===", "Null"}]}], ")"}]}], "&"}], "@", 
               RowBox[{"Last", "[", "#", "]"}]}], "&"}]}], "]"}]}], "}"}], 
         ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"bad", "\[NotEqual]", 
            RowBox[{"{", "}"}]}], ",", 
           RowBox[{
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<SaveToFile Error : data format should be \
string|Numeric|Array\\n\>\"", ",", 
              RowBox[{"Short", "[", 
               RowBox[{"bad", ",", "2"}], "]"}]}], "]"}], ";", 
            "\[IndentingNewLine]", " ", 
            RowBox[{"Return", "[", "True", "]"}], ";"}]}], "]"}]}], "]"}], 
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"lStr", "=", 
        RowBox[{"Select", "[", 
         RowBox[{"l", ",", 
          RowBox[{
           RowBox[{"StringQ", "@", 
            RowBox[{"Last", "@", "#"}]}], "&"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"lNum", "=", 
        RowBox[{"Select", "[", 
         RowBox[{"l", ",", 
          RowBox[{
           RowBox[{"NumericQ", "@", 
            RowBox[{"Last", "@", "#"}]}], "&"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"lArr", "=", 
        RowBox[{"Select", "[", 
         RowBox[{"l", ",", 
          RowBox[{
           RowBox[{"ArrayQ", "@", 
            RowBox[{"Last", "@", "#"}]}], "&"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"str", "=", 
           RowBox[{"str", "<>", 
            RowBox[{"(", 
             RowBox[{"First", "@", "eStr"}], ")"}], "<>", "\"\<\\n-1\\n\>\"", 
            "<>", 
            RowBox[{"(", 
             RowBox[{"Last", "@", "eStr"}], ")"}], "<>", "\"\<\\n\\n\>\""}]}],
           ";"}], "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"eStr", ",", "lStr"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"str", "=", 
           RowBox[{"str", "<>", 
            RowBox[{"(", 
             RowBox[{"First", "@", "eNum"}], ")"}], "<>", 
            "\"\<\\n0\\n\\n\>\""}]}], ";", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"data", ",", 
            RowBox[{"N", "@", 
             RowBox[{"Last", "@", "eNum"}]}]}], "]"}], ";"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"eNum", ",", "lNum"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"dims", "=", 
              RowBox[{"Dimensions", "@", 
               RowBox[{"Last", "@", "eArr"}]}]}], "}"}], ",", " ", 
            RowBox[{"(*", 
             RowBox[{"Row", " ", "major"}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"str", "=", 
              RowBox[{"str", "<>", 
               RowBox[{"(", 
                RowBox[{"First", "@", "eArr"}], ")"}], "<>", "\"\<\\n\>\"", 
               "\[IndentingNewLine]", "<>", 
               RowBox[{"ToString", "[", 
                RowBox[{"Length", "@", "dims"}], "]"}], "<>", 
               "\"\<\\n\>\""}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"Do", "[", 
              RowBox[{
               RowBox[{"str", "=", 
                RowBox[{"str", "<>", 
                 RowBox[{"ToString", "[", "d", "]"}], "<>", "\"\<\\n\>\""}]}],
                ",", 
               RowBox[{"{", 
                RowBox[{"d", ",", "dims"}], "}"}]}], "]"}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"str", "=", 
              RowBox[{"str", "<>", "\"\<\\n\>\""}]}], ";"}]}], 
           "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"data", "=", 
           RowBox[{"Join", "[", 
            RowBox[{"data", ",", 
             RowBox[{"N", "@", 
              RowBox[{"Flatten", "@", 
               RowBox[{"Last", "@", "eArr"}]}]}]}], "]"}]}], ";"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"eArr", ",", "lArr"}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"StringLength", "[", "fileName", "]"}], ">", "0"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"With", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"formatFile", "=", 
              RowBox[{"fileName", "<>", "\"\<_Format.txt\>\""}]}], ",", 
             RowBox[{"dataFile", "=", 
              RowBox[{"fileName", "<>", "\"\<_Data.dat\>\""}]}]}], "}"}], ",",
            "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"Export", "[", 
             RowBox[{"formatFile", ",", "str"}], "]"}], ";", " ", 
            RowBox[{"(*", 
             RowBox[{
              RowBox[{"Mathematica", " ", "adds", " ", "one", " ", "\\n"}], 
              ",", " ", 
              RowBox[{
               RowBox[{
               "so", " ", "we", " ", "must", " ", "remove", " ", "one"}], 
               "..."}]}], "*)"}], "\[IndentingNewLine]", 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Length", "@", "data"}], ">", "0"}], ",", 
              RowBox[{
               RowBox[{"BinaryWrite", "[", 
                RowBox[{"dataFile", ",", "data", ",", "\"\<Real64\>\""}], 
                "]"}], ";", " ", 
               RowBox[{"Close", "[", "dataFile", "]"}], ";"}]}], "]"}], ";", 
            "\[IndentingNewLine]", 
            RowBox[{"Return", "[", "False", "]"}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Return", "[", 
           RowBox[{"{", 
            RowBox[{"str", ",", "data"}], "}"}], "]"}], ";"}]}], 
        "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}],
    ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"RawToRules", "[", "fileName_String", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"formatFile", "=", 
        RowBox[{"fileName", "<>", "\"\<_Format.txt\>\""}]}], ",", 
       RowBox[{"dataFile", "=", 
        RowBox[{"fileName", "<>", "\"\<_Data.dat\>\""}]}], ",", "data"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"RawToRules", "[", 
      RowBox[{
       RowBox[{"Import", "[", 
        RowBox[{"formatFile", ",", "\"\<String\>\""}], "]"}], ",", 
       RowBox[{"BinaryReadList", "[", 
        RowBox[{"dataFile", ",", "\"\<Real64\>\""}], "]"}]}], "]"}]}], 
    "\[IndentingNewLine]", "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"RawToRules", "[", 
   RowBox[{"str_String", ",", "data_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"l", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"pos", "=", "0"}], ",", 
      RowBox[{"newlineCharacter", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"$OperatingSystem", "\[Equal]", "\"\<Windows\>\""}], ",", 
         "\"\<\\r\\n\>\"", ",", "\"\<\\n\>\""}], "]"}]}]}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Do", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Module", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"format", " ", "=", " ", 
            RowBox[{"StringSplit", "[", 
             RowBox[{"s", ",", "newlineCharacter"}], "]"}]}], ",", "dim", ",",
            "dims", ",", 
           RowBox[{"oldPos", "=", "pos"}], ",", 
           RowBox[{"oldLen", "=", 
            RowBox[{"Length", "@", "l"}]}]}], "}"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"Length", "@", "format"}], "\[GreaterEqual]", "2"}], ",",
             "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"dim", "=", 
              RowBox[{"ToExpression", "[", 
               RowBox[{"format", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"IntegerQ", "[", "dim", "]"}], " ", "&&", " ", 
                RowBox[{"dim", "\[GreaterEqual]", 
                 RowBox[{"-", "1"}]}]}], ",", "\[IndentingNewLine]", 
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{"dim", "\[Equal]", 
                  RowBox[{"-", "1"}]}], ",", "\[IndentingNewLine]", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Length", "@", "format"}], "\[Equal]", "3"}], ",", 
                   RowBox[{"AppendTo", "[", 
                    RowBox[{"l", ",", 
                    RowBox[{
                    RowBox[{"format", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "\[Rule]", 
                    RowBox[{"format", "[", 
                    RowBox[{"[", "3", "]"}], "]"}]}]}], " ", "]"}]}], " ", 
                  "]"}], ",", "\[IndentingNewLine]", 
                 RowBox[{"If", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Length", "@", "format"}], "\[Equal]", 
                    RowBox[{"dim", "+", "2"}]}], ",", "\[IndentingNewLine]", 
                   RowBox[{
                    RowBox[{"dims", "=", 
                    RowBox[{"Table", "[", 
                    RowBox[{
                    RowBox[{"ToExpression", "@", 
                    RowBox[{"format", "[", 
                    RowBox[{"[", 
                    RowBox[{"i", "+", "2"}], "]"}], "]"}]}], ",", 
                    RowBox[{"{", 
                    RowBox[{"i", ",", "dim"}], "}"}]}], "]"}]}], ";", " ", 
                    RowBox[{"(*", 
                    RowBox[{"row", " ", "major"}], "*)"}], 
                    "\[IndentingNewLine]", 
                    RowBox[{"pos", "=", 
                    RowBox[{"pos", "+", 
                    RowBox[{"Times", "@@", "dims"}]}]}], ";", 
                    "\[IndentingNewLine]", 
                    RowBox[{"If", "[", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"Length", "@", "data"}], "\[GreaterEqual]", 
                    "pos"}], ",", 
                    RowBox[{"AppendTo", "[", 
                    RowBox[{"l", ",", 
                    RowBox[{
                    RowBox[{"format", "[", 
                    RowBox[{"[", "1", "]"}], "]"}], "\[Rule]", " ", 
                    RowBox[{"ArrayReshape", "[", 
                    RowBox[{
                    RowBox[{"data", "[", 
                    RowBox[{"[", 
                    RowBox[{
                    RowBox[{"oldPos", "+", "1"}], ";;", "pos"}], "]"}], "]"}],
                     ",", "dims"}], "]"}]}]}], " ", "]"}]}], "]"}]}]}], 
                  "\[IndentingNewLine]", "]"}]}], " ", "]"}]}], " ", 
              "]"}]}]}], " ", "]"}], ";", "\[IndentingNewLine]", 
          RowBox[{"(*", 
           RowBox[{
           "Allow", " ", "for", " ", "a", " ", "final", " ", "blank", " ", 
            "line"}], "*)"}], "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Length", "@", "l"}], "\[Equal]", "oldLen"}], " ", "&&",
              " ", 
             RowBox[{"s", "\[NotEqual]", "newlineCharacter"}]}], ",", 
            RowBox[{"Print", "[", 
             RowBox[{
             "\"\<RawToRules error : could not import \>\"", "<>", "s"}], 
             "]"}]}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], 
       "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"s", ",", 
         RowBox[{"StringSplit", "[", 
          RowBox[{"str", ",", 
           RowBox[{"newlineCharacter", "<>", "newlineCharacter"}]}], "]"}]}], 
        "}"}]}], "]"}], ";", "\[IndentingNewLine]", "l"}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 CellChangeTimes->{{3.696873107708523*^9, 3.696873280372493*^9}, {
   3.6968735175848227`*^9, 3.696873544038447*^9}, {3.696873883092235*^9, 
   3.6968738866873617`*^9}, {3.696932632863077*^9, 3.696932683915284*^9}, {
   3.696943072071027*^9, 3.696943090021679*^9}, {3.696943371703699*^9, 
   3.6969433796839247`*^9}, {3.697743684182283*^9, 3.697743700937011*^9}, {
   3.699379911773041*^9, 3.699379915829468*^9}, {3.699379954174136*^9, 
   3.699379957148058*^9}, {3.699379996216611*^9, 3.699379998377261*^9}, {
   3.699380091805086*^9, 3.699380121229937*^9}, {3.699380157431526*^9, 
   3.699380189663466*^9}, {3.699380289139727*^9, 3.699380310301757*^9}, {
   3.699380469782707*^9, 3.699380572745825*^9}, 3.699380604092313*^9, {
   3.69938068985175*^9, 3.699380771922076*^9}, {3.699380818902176*^9, 
   3.699380839220717*^9}, {3.6993869743422413`*^9, 3.6993869795148697`*^9}, {
   3.7002035804277782`*^9, 3.700203744865365*^9}, {3.700203775619684*^9, 
   3.7002037773743668`*^9}, {3.700203827416172*^9, 3.700203861373456*^9}, {
   3.7002038979905367`*^9, 3.7002039686560163`*^9}, {3.700204065457471*^9, 
   3.700204129348976*^9}, 3.7004562988277903`*^9, {3.701774367564468*^9, 
   3.7017743997212267`*^9}, 3.7017745837881813`*^9, {3.720184242947942*^9, 
   3.720184279651361*^9}, {3.720184623142488*^9, 3.720184624857038*^9}, {
   3.720184660800836*^9, 3.720184661590372*^9}}]
}, Open  ]]
},
WindowSize->{906, 855},
WindowMargins->{{Automatic, 235}, {Automatic, 0}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September \
21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 177, 2, 148, "Title"],
Cell[759, 26, 425, 8, 49, "Text"],
Cell[1187, 36, 531, 14, 54, "Input"],
Cell[1721, 52, 21662, 540, 2170, "Input"]
}, Open  ]]
}
]
*)

