(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     25840,        635]
NotebookOptionsPosition[     25393,        615]
NotebookOutlinePosition[     25752,        631]
CellTagsIndexPosition[     25709,        628]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Fast Marching methods for curvature penalized shortest paths", "Title",
 CellChangeTimes->{{3.696671384756301*^9, 3.696671430795917*^9}, {
  3.696671461195407*^9, 3.696671461763341*^9}, {3.6967599927427*^9, 
  3.696760007329444*^9}}],

Cell["An interface for defining walls, seeds and tips.", "Subtitle",
 CellChangeTimes->{{3.696671435623104*^9, 3.696671456549898*^9}, {
  3.69674449088338*^9, 3.6967444911818132`*^9}, {3.6967445879208384`*^9, 
  3.696744588737105*^9}, {3.696750974984147*^9, 3.6967509867724247`*^9}, {
  3.6967600152327013`*^9, 3.696760044584044*^9}}],

Cell["\<\
You need to evaluate the notebook Common.nb first, as well as the code below. \
Then, in another notebook, evaluate:

myOptions={\[OpenCurlyDoubleQuote]domain\[CloseCurlyDoubleQuote]\[Rule] \
{{0,0},{2,1}},\[CloseCurlyDoubleQuote]imageSize\[CloseCurlyDoubleQuote]\[Rule]\
300};
myOptions=Join[myOptions,RangeAndBox2[myOptions]];
ObstaclesInterface[myOptions]
\
\>", "Text",
 CellChangeTimes->{{3.696744688029573*^9, 3.696744699254168*^9}, {
  3.697223069169018*^9, 3.697223101663372*^9}, {3.69996654554652*^9, 
  3.6999667434615107`*^9}, {3.699967116872004*^9, 3.6999672023811274`*^9}, {
  3.699967243653112*^9, 3.699967311785471*^9}}],

Cell["\<\
Add as many walls, seeds and tips as you like, using Alt + Click. Note that \
two points are required to define each wall, seed, tip. Once this is done, \
evaluate:
     
myOptions = MergeRules[{\"obstacles\" -> manObstacles, \"nRaster\" -> 100, \
\[OpenCurlyDoubleQuote]modelName\[CloseCurlyDoubleQuote] -> \
\[OpenCurlyDoubleQuote]Elastica2\[CloseCurlyDoubleQuote]}, myOptions];
myParams = {\"tips\" -> manTips, \"seeds\" -> manSeeds, \"xi\" -> 0.2};
ShowShortestPaths[myOptions, myParams];\
\>", "Text",
 CellChangeTimes->{{3.699967269386609*^9, 3.699967300403363*^9}, 
   3.699983737811639*^9, {3.729539906719499*^9, 3.729539912285626*^9}, 
   3.7295400942085943`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RangeAndBox2", "[", "options_", "]"}], ":=", 
    RowBox[{"With", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"dom", "=", 
        RowBox[{"\"\<domain\>\"", "/.", "options"}]}], "}"}], ",", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"\"\<range\>\"", "\[Rule]", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"#1", "-", "0.1"}], ",", 
              RowBox[{"#2", "+", "0.1"}]}], "}"}], "&"}], "@@@", 
           RowBox[{"Transpose", "@", "dom"}]}], ")"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<box\>\"", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"Transparent", ",", 
           RowBox[{"EdgeForm", "[", "Black", "]"}], ",", 
           RowBox[{"Rectangle", "@@", "dom"}]}], "}"}]}]}], 
       "\[IndentingNewLine]", "}"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RasterObstacles", "[", "options_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "g", ",", "rast", ",", "arr", ",", "pos", ",", "xMin", ",", "xMax", 
        ",", "yMin", ",", "yMax", ",", "opts", ",", "n", ",", 
        "\[IndentingNewLine]", 
        RowBox[{"box", "=", 
         RowBox[{"\"\<box\>\"", "/.", "options"}]}], ",", 
        RowBox[{"range", "=", 
         RowBox[{"\"\<range\>\"", "/.", "options"}]}], ",", 
        RowBox[{"obstacles", "=", 
         RowBox[{"\"\<obstacles\>\"", "/.", "options"}]}]}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"opts", "=", 
        RowBox[{"MergeRules", "[", 
         RowBox[{"options", ",", 
          RowBox[{"{", 
           RowBox[{"\"\<nRaster\>\"", "\[Rule]", "100"}], "}"}]}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"n", "=", 
        RowBox[{"\"\<nRaster\>\"", "/.", "opts"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"g", "=", 
        RowBox[{"Graphics", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"box", ",", 
            RowBox[{"Line", "/@", "obstacles"}]}], "}"}], ",", 
          RowBox[{"PlotRange", "\[Rule]", "range"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Print", "[", "g", "]"}], ";"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"rast", "=", 
        RowBox[{"Rasterize", "[", 
         RowBox[{"g", ",", 
          RowBox[{"RasterSize", "\[Rule]", 
           RowBox[{"n", "+", "2"}]}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"arr", "=", 
        RowBox[{"First", "@", 
         RowBox[{"First", "@", "rast"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"arr", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "&"}], ",", "arr", ",", 
          RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"pos", "=", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"Min", "/@", "arr"}], ",", "0"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"xMin", "=", 
        RowBox[{"First", "@", 
         RowBox[{"First", "@", "pos"}]}]}], ";", 
       RowBox[{"xMax", "=", 
        RowBox[{"First", "@", 
         RowBox[{"Last", "@", "pos"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"pos", "=", 
        RowBox[{"Position", "[", 
         RowBox[{
          RowBox[{"Min", "/@", 
           RowBox[{"Transpose", "@", "arr"}]}], ",", "0"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"yMin", "=", 
        RowBox[{"First", "@", 
         RowBox[{"First", "@", "pos"}]}]}], ";", 
       RowBox[{"yMax", "=", 
        RowBox[{"First", "@", 
         RowBox[{"Last", "@", "pos"}]}]}], ";", "\[IndentingNewLine]", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Print", "[", 
          RowBox[{
          "xMin", ",", "\"\< \>\"", ",", "xMax", ",", "\"\< \>\"", ",", 
           "yMin", ",", "\"\< \>\"", ",", "yMax"}], "]"}], ";"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{"arr", "=", 
        RowBox[{"arr", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{
            RowBox[{"xMin", "+", "1"}], ";;", 
            RowBox[{"xMax", "-", "1"}]}], ",", 
           RowBox[{
            RowBox[{"yMin", "+", "1"}], ";;", 
            RowBox[{"yMax", "-", "1"}]}]}], "]"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"arr", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"#", "\[Equal]", "255"}], ",", "0", ",", "1"}], "]"}], 
           "&"}], ",", "arr", ",", 
          RowBox[{"{", "2", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"arr", "=", 
        RowBox[{"Transpose", "@", "arr"}]}], ";", "\[IndentingNewLine]", 
       "arr"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"RunPathPlanning2", "[", 
     RowBox[{"options_", ",", "params_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "opts", ",", "prms", ",", "gridScale", ",", "refDir", ",", "out", ",", 
        "\[IndentingNewLine]", "walls", ",", 
        RowBox[{"dom", "=", 
         RowBox[{"\"\<domain\>\"", "/.", "options"}]}], ",", "executable"}], 
       "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"opts", "=", 
        RowBox[{"MergeRules", "[", 
         RowBox[{"options", ",", 
          RowBox[{"{", 
           RowBox[{"\"\<nDir\>\"", "\[Rule]", "60"}], "}"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"walls", "=", 
        RowBox[{"RasterObstacles", "[", "options", "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"gridScale", "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"dom", "[", 
            RowBox[{"[", 
             RowBox[{"2", ",", "1"}], "]"}], "]"}], "-", 
           RowBox[{"dom", "[", 
            RowBox[{"[", 
             RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ")"}], "/", 
         RowBox[{
          RowBox[{"Dimensions", "[", "walls", "]"}], "[", 
          RowBox[{"[", "1", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"prms", "=", 
        RowBox[{"MergeRules", "[", 
         RowBox[{"params", ",", 
          RowBox[{"{", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"\"\<dims\>\"", "\[Rule]", 
             RowBox[{"Append", "[", 
              RowBox[{
               RowBox[{"Dimensions", "[", "walls", "]"}], ",", 
               RowBox[{"\"\<nDir\>\"", "/.", "opts"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"\"\<gridScale\>\"", "\[Rule]", "gridScale"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"\"\<speed\>\"", "\[Rule]", "1"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"\"\<progressReportLandmarks\>\"", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"\"\<eps\>\"", "\[Rule]", "0.1"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"\"\<exportValues\>\"", "\[Rule]", "0"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{"\"\<walls\>\"", "\[Rule]", "walls"}]}], 
           "\[IndentingNewLine]", "}"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"executable", "=", 
        RowBox[{"\"\<FileHFM_\>\"", "<>", 
         RowBox[{"(", 
          RowBox[{"\"\<modelName\>\"", "/.", "opts"}], ")"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"out", "=", 
        RowBox[{"RunExec", "[", 
         RowBox[{"executable", ",", "prms"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"prms", ",", "out"}], "}"}]}]}], "\[IndentingNewLine]", 
     "]"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ShowShortestPaths", "[", 
     RowBox[{"options_", ",", "params_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "prms", ",", "out", ",", "geo", ",", "gridScale", ",", 
        "\[IndentingNewLine]", "imageSize", ",", "walls"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"prms", ",", "out"}], "}"}], "=", 
        RowBox[{"RunPathPlanning2", "[", 
         RowBox[{"options", ",", "params"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"gridScale", "=", 
        RowBox[{"\"\<gridScale\>\"", "/.", "prms"}]}], ";", 
       RowBox[{"imageSize", "=", 
        RowBox[{"\"\<imageSize\>\"", "/.", "options"}]}], ";", 
       RowBox[{"walls", "=", 
        RowBox[{"\"\<walls\>\"", "/.", "prms"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"geo", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Drop", "[", 
            RowBox[{"#", ",", 
             RowBox[{"-", "1"}]}], "]"}], "&"}], ",", 
          RowBox[{"\"\<geodesics\>\"", "/.", "out"}], ",", "2"}], "]"}]}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<log\>\"", "/.", "out"}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{
         RowBox[{"TablePlot", "[", 
          RowBox[{"walls", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Epilog", "\[Rule]", 
              RowBox[{"Line", "/@", 
               RowBox[{"(", 
                RowBox[{"geo", "/", "gridScale"}], ")"}]}]}], ",", 
             RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "}"}]}], " ", 
          "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"Graphics", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"\"\<box\>\"", "/.", "options"}], ",", 
             RowBox[{"{", 
              RowBox[{"Red", ",", 
               RowBox[{"Line", "/@", 
                RowBox[{"(", 
                 RowBox[{"\"\<obstacles\>\"", "/.", "options"}], ")"}]}]}], 
              "}"}], ",", 
             RowBox[{"Line", "/@", "geo"}]}], "}"}], ",", 
           RowBox[{"ImageSize", "\[Rule]", "imageSize"}]}], "]"}]}], "]"}], 
       ";", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"\"\<Domain size:\>\"", ",", 
         RowBox[{"Dimensions", "[", "walls", "]"}], ",", 
         "\"\<, FMCPUTime:\>\"", ",", "\"\< \>\"", ",", 
         RowBox[{"\"\<FMCPUTime\>\"", "/.", "out"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"{", 
        RowBox[{"prms", ",", "out"}], "}"}]}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
    "The", " ", "following", " ", "code", " ", "sets", " ", "the", " ", 
     "variables", " ", "manObstacles"}], ",", " ", "manSeeds", ",", " ", 
    "manTips"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ObstaclesInterface", "[", "options_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "opts", ",", "imageSize", ",", "box", ",", "r", ",", "arrTable", ",", 
       "pairTable", ",", "orientTable", ",", "range"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"opts", "=", 
       RowBox[{"MergeRules", "[", 
        RowBox[{"options", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"\"\<imageSize\>\"", "\[Rule]", "300"}], ",", 
           RowBox[{"\"\<rArrow\>\"", "\[Rule]", "0.2"}]}], "}"}]}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"imageSize", "=", 
       RowBox[{"\"\<imageSize\>\"", "/.", "opts"}]}], ";", 
      RowBox[{"r", "=", 
       RowBox[{"\"\<rArrow\>\"", "/.", "opts"}]}], ";", 
      RowBox[{"range", "=", 
       RowBox[{"\"\<range\>\"", "/.", "options"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"box", "=", 
       RowBox[{"\"\<box\>\"", "/.", "opts"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"arrTable", "[", "l_", "]"}], ":=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"Arrow", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"s", "[", 
             RowBox[{"[", 
              RowBox[{"1", ";;", "2"}], "]"}], "]"}], ",", 
            RowBox[{
             RowBox[{"s", "[", 
              RowBox[{"[", 
               RowBox[{"1", ";;", "2"}], "]"}], "]"}], "+", 
             RowBox[{"0.2", " ", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"Cos", "@", 
                 RowBox[{"Last", "@", "s"}]}], ",", 
                RowBox[{"Sin", "@", 
                 RowBox[{"Last", "@", "s"}]}]}], "}"}]}]}]}], "}"}], "]"}], 
         ",", 
         RowBox[{"{", 
          RowBox[{"s", ",", "l"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"pairTable", "[", "l_", "]"}], ":=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"l", "[", 
            RowBox[{"[", 
             RowBox[{
              RowBox[{"2", " ", "i"}], "-", "1"}], "]"}], "]"}], ",", 
           RowBox[{"l", "[", 
            RowBox[{"[", 
             RowBox[{"2", " ", "i"}], "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", 
           RowBox[{"Floor", "[", 
            RowBox[{
             RowBox[{"Length", "[", "l", "]"}], "/", "2"}], "]"}]}], "}"}]}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"orientTable", "[", "l_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{"Append", "[", 
          RowBox[{"#1", ",", 
           RowBox[{"Arg", "[", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"#2", "-", "#1"}], ")"}], ".", 
             RowBox[{"{", 
              RowBox[{"1", ",", "I"}], "}"}]}], "]"}]}], "]"}], "&"}], "@@@", 
        
        RowBox[{"pairTable", "[", "l", "]"}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"obs", "=", " ", 
              RowBox[{"Table", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"pt", "[", 
                   RowBox[{"[", 
                    RowBox[{
                    RowBox[{"2", "i"}], "-", "1"}], "]"}], "]"}], ",", 
                  RowBox[{"pt", "[", 
                   RowBox[{"[", 
                    RowBox[{"2", "i"}], "]"}], "]"}]}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"i", ",", 
                  RowBox[{"Floor", "[", 
                   RowBox[{
                    RowBox[{"(", 
                    RowBox[{"Length", "@", "pt"}], ")"}], "/", "2"}], "]"}]}],
                  "}"}]}], "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"manObstacles", "=", "obs"}], ";", "\[IndentingNewLine]", 
             RowBox[{"Graphics", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"box", ",", 
                 RowBox[{"Line", "/@", "obs"}]}], "}"}], ",", 
               RowBox[{"PlotRange", "\[Rule]", "range"}], ",", 
               RowBox[{"ImageSize", "\[Rule]", "imageSize"}], ",", 
               RowBox[{"PlotLabel", "\[Rule]", "\"\<walls\>\""}]}], "]"}]}]}],
            "]"}], ",", "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"pt", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.1", ",", "0.7"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.9", ",", "0.2"}], "}"}]}], "}"}]}], "}"}], ",", 
            "Locator", ",", 
            RowBox[{"LocatorAutoCreate", "\[Rule]", "True"}]}], "}"}]}], 
         "]"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"seeds", "=", 
              RowBox[{"orientTable", "@", "pt"}]}], "}"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"manSeeds", "=", "seeds"}], ";", "\[IndentingNewLine]", 
             RowBox[{"Graphics", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"box", ",", 
                 RowBox[{"Line", "/@", "manObstacles"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"GrayLevel", "[", "0.8", "]"}], ",", "Dashed", ",", 
                   RowBox[{"Line", "/@", 
                    RowBox[{"(", 
                    RowBox[{"pairTable", "@", "pt"}], ")"}]}]}], "}"}], ",", 
                 RowBox[{"arrTable", "[", "seeds", "]"}]}], 
                "\[IndentingNewLine]", "}"}], ",", 
               RowBox[{"PlotRange", "\[Rule]", "range"}], ",", 
               RowBox[{"ImageSize", "\[Rule]", "imageSize"}], ",", 
               RowBox[{"PlotLabel", "\[Rule]", "\"\<seeds\>\""}]}], "]"}]}]}],
            "]"}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"pt", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.1", ",", "0.1"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.3", ",", "0.3"}], "}"}]}], "}"}]}], "}"}], ",", 
            "Locator", ",", 
            RowBox[{"LocatorAutoCreate", "\[Rule]", "True"}]}], "}"}]}], 
         "]"}], ",", "\[IndentingNewLine]", "\[IndentingNewLine]", 
        RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"With", "[", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"tips", "=", 
              RowBox[{"orientTable", "@", "pt"}]}], "}"}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"manTips", "=", "tips"}], ";", "\[IndentingNewLine]", 
             RowBox[{"Graphics", "[", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"box", ",", 
                 RowBox[{"Line", "/@", "manObstacles"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"GrayLevel", "[", "0.8", "]"}], ",", "Dashed", ",", 
                   RowBox[{"Line", "/@", 
                    RowBox[{"(", 
                    RowBox[{"pairTable", "@", "pt"}], ")"}]}]}], "}"}], ",", 
                 RowBox[{"arrTable", "@", "tips"}]}], "\[IndentingNewLine]", 
                "}"}], ",", 
               RowBox[{"PlotRange", "\[Rule]", "range"}], ",", 
               RowBox[{"ImageSize", "\[Rule]", "imageSize"}], ",", 
               RowBox[{"PlotLabel", "\[Rule]", "\"\<tips\>\""}]}], "]"}]}]}], 
           "]"}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"pt", ",", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.8", ",", "0.8"}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.6", ",", "0.5"}], "}"}]}], "}"}]}], "}"}], ",", 
            "Locator", ",", 
            RowBox[{"LocatorAutoCreate", "\[Rule]", "True"}]}], "}"}]}], 
         "]"}]}], "\[IndentingNewLine]", "}"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.696674273723648*^9, 3.69667437668331*^9}, {
   3.6966744187607*^9, 3.696674580678789*^9}, {3.696676388027997*^9, 
   3.696676389698161*^9}, {3.696677460370161*^9, 3.696677461938962*^9}, {
   3.696677529750347*^9, 3.696677560746086*^9}, {3.696677827244782*^9, 
   3.6966778286916037`*^9}, {3.6967440746557283`*^9, 3.696744076990041*^9}, {
   3.696748702269796*^9, 3.696748705467681*^9}, 3.6967509309083853`*^9, {
   3.696750991146703*^9, 3.6967510084469624`*^9}, {3.6967534402804537`*^9, 
   3.696753549614977*^9}, {3.69675357990416*^9, 3.696753625139617*^9}, {
   3.696753698186026*^9, 3.696753730599957*^9}, {3.69676005562677*^9, 
   3.696760060136447*^9}, {3.696762609713399*^9, 3.696762643163315*^9}, {
   3.696762719694241*^9, 3.696762729888015*^9}, {3.6967632859264793`*^9, 
   3.69676328842507*^9}, {3.697223060656838*^9, 3.6972230812381783`*^9}, {
   3.697223143936859*^9, 3.697223233735861*^9}, 3.6972235322616034`*^9, {
   3.6972650312733383`*^9, 3.697265031627269*^9}, {3.697357335204723*^9, 
   3.697357335916531*^9}, {3.699959925021803*^9, 3.6999601897385*^9}, {
   3.6999602624921494`*^9, 3.699960342250226*^9}, {3.699960413112316*^9, 
   3.6999605950233307`*^9}, {3.699960625628323*^9, 3.699960627817987*^9}, {
   3.699960664824939*^9, 3.6999607135928917`*^9}, {3.699960762632213*^9, 
   3.699960800393361*^9}, {3.699960832427185*^9, 3.6999609093886642`*^9}, {
   3.699961194695447*^9, 3.699961197770788*^9}, {3.699961373121544*^9, 
   3.699961392359694*^9}, {3.699961640016672*^9, 3.699961640555069*^9}, 
   3.6999628624162273`*^9, {3.699963050990679*^9, 3.699963151568894*^9}, {
   3.699963182470962*^9, 3.699963184408908*^9}, 3.6999632173726683`*^9, {
   3.699963286131319*^9, 3.6999632902773333`*^9}, {3.699963366027404*^9, 
   3.699963370163209*^9}, {3.6999634525236273`*^9, 3.6999634812565727`*^9}, {
   3.699963511278963*^9, 3.699963519982814*^9}, {3.699963605226797*^9, 
   3.699963629035927*^9}, {3.699963880430421*^9, 3.6999644613972683`*^9}, {
   3.699964501867324*^9, 3.699964901374199*^9}, {3.6999649883724213`*^9, 
   3.699965017280257*^9}, {3.699965048618168*^9, 3.699965072428604*^9}, {
   3.69996526053201*^9, 3.6999653158208313`*^9}, {3.6999656107258577`*^9, 
   3.699965680152277*^9}, 3.699965717945023*^9, {3.69996620771616*^9, 
   3.6999662687818813`*^9}, {3.69996630900515*^9, 3.699966318122311*^9}, {
   3.6999663765383387`*^9, 3.699966436307438*^9}, 3.69996651155767*^9, {
   3.69996679599114*^9, 3.699966847764018*^9}, {3.699983000189371*^9, 
   3.699983002475236*^9}, {3.720185400831444*^9, 3.720185464744219*^9}, {
   3.720185528942474*^9, 3.720185565473506*^9}, {3.720185600108745*^9, 
   3.720185609250939*^9}, {3.729539959536723*^9, 3.7295399890357647`*^9}, {
   3.7295400260751333`*^9, 3.729540065535597*^9}}]
}, Open  ]]
},
WindowSize->{795, 855},
WindowMargins->{{Automatic, 162}, {Automatic, 0}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September \
21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 239, 3, 148, "Title"],
Cell[821, 27, 334, 4, 51, "Subtitle"],
Cell[1158, 33, 645, 14, 144, "Text"],
Cell[1806, 49, 682, 13, 144, "Text"],
Cell[2491, 64, 22886, 548, 2070, "Input"]
}, Open  ]]
}
]
*)

