(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     27770,        569]
NotebookOptionsPosition[     26971,        538]
NotebookOutlinePosition[     27330,        554]
CellTagsIndexPosition[     27287,        551]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Euclidean curve length and Voronoi regions ", "Title",
 CellChangeTimes->{{3.6993809964591637`*^9, 3.699381036243622*^9}, {
  3.6993811219320602`*^9, 3.699381149211528*^9}}],

Cell["Propagating a state through the fast marching algorithm", "Subsubtitle",
 CellChangeTimes->{{3.6993811621765547`*^9, 3.699381180322505*^9}}],

Cell["\<\

This notebook illustrates the computation of geodesic euclidean length, and \
Voronoi regions.
These methods are implemented in the HFMState class, intended to show how a \
state can be propagated through the Fast Marching Algorithm.\
\>", "Text",
 CellChangeTimes->{{3.699381041410324*^9, 3.69938111463236*^9}, {
  3.69938119970326*^9, 3.699381332698579*^9}, {3.69938276794473*^9, 
  3.699382770302102*^9}, {3.6993828020956993`*^9, 3.6993828150604763`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"To", " ", 
     RowBox[{"used", ":", " ", 
      RowBox[{
      "Please", " ", "first", " ", "execute", " ", "notebook", " ", 
       "\"\<Common.nb\>\""}]}]}], ",", 
    RowBox[{"in", " ", "the", " ", "same", " ", 
     RowBox[{"directory", "."}]}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"prms", ",", 
       RowBox[{"n", "=", "20"}], ",", "dims", ",", "out", ",", "h"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"dims", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"2", "n"}], ",", "n"}], "}"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"h", "=", 
       RowBox[{"1", "/", "n"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"prms", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
         "Setup", " ", "a", " ", "very", " ", "basic", " ", "fast", " ", 
          "marching", " ", 
          RowBox[{"algorithm", ".", " ", "The"}], " ", "methods", " ", 
          "illustrated", " ", "here", " ", "of", " ", "course", " ", "apply", 
          " ", "to", " ", "any", " ", 
          RowBox[{"model", "."}]}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"\"\<dims\>\"", "\[Rule]", "dims"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<gridScale\>\"", "\[Rule]", "h"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<seeds\>\"", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0.5", ",", "0.5"}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0.2", ",", "0.7"}], "}"}]}], "}"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<speed\>\"", "\[Rule]", 
          RowBox[{"Array", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"#1", "\[LessEqual]", "n"}], ",", "1", ",", "2"}], 
              "]"}], "&"}], ",", "dims"}], "]"}]}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<exportValues\>\"", "\[Rule]", "1"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<progressReportLandmarks\>\"", "\[Rule]", 
          RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
         "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"Geodesic", " ", "euclidean", " ", "length", " ", 
            RowBox[{"computation", ".", " ", "Define"}], " ", "the", " ", 
            "scale", " ", "in", " ", "each", " ", "grid", " ", 
            RowBox[{"direction", ".", "\[IndentingNewLine]", "For"}], " ", 
            "a", " ", "point", " ", "dependent", " ", "scale", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"e", ".", "g", ".", " ", "euclidean"}], " ", "length", 
              " ", "on", " ", "the", " ", "sphere"}], ")"}]}], ",", " ", 
           RowBox[{"use", " ", "an", " ", "array"}]}], "*)"}], 
         "\[IndentingNewLine]", 
         RowBox[{"\"\<euclideanScale\>\"", "\[Rule]", 
          RowBox[{"h", 
           RowBox[{"{", 
            RowBox[{"1", ",", "1"}], "}"}]}]}], ",", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{
           RowBox[{"Optional", " ", "stopping", " ", "criterion"}], ",", " ", 
           
           RowBox[{
           "when", " ", "a", " ", "geodesic", " ", "reaches", " ", "a", " ", 
            "prescribed", " ", "length"}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"\"\<stopAtEuclideanLength\>\"", "\[Rule]", "1.2"}], ",", 
         "\[IndentingNewLine]", "\[IndentingNewLine]", 
         RowBox[{"(*", 
          RowBox[{"Voronoi", " ", "diagram", " ", 
           RowBox[{"computation", ".", " ", "Define"}], " ", "the", " ", 
           "flag", " ", "for", " ", "each", " ", 
           RowBox[{"seed", "."}]}], "*)"}], "\[IndentingNewLine]", 
         RowBox[{"\"\<seedFlags\>\"", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"1", ",", "2"}], "}"}]}]}], "\[IndentingNewLine]", "}"}]}],
       ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"out", "=", 
       RowBox[{"RunExec", "[", 
        RowBox[{"\"\<FileHFM_Isotropic2\>\"", ",", "prms"}], "]"}]}], ";", 
      " ", 
      RowBox[{"(*", 
       RowBox[{"FileHFM_Isotropic", " ", "also", " ", "fits"}], "*)"}], 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<log\>\"", "/.", "out"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"TablePlot", "[", 
        RowBox[{
         RowBox[{"\"\<values\>\"", "/.", "out"}], ",", 
         RowBox[{"{", 
          RowBox[{"PlotLabel", "\[Rule]", "\"\<values\>\""}], "}"}]}], "]"}], 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"TablePlot", "[", 
        RowBox[{
         RowBox[{"\"\<euclideanLengths\>\"", "/.", "out"}], ",", 
         RowBox[{"{", 
          RowBox[{"PlotLabel", "\[Rule]", "\"\<euclideanLengths\>\""}], 
          "}"}]}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"TablePlot", "[", 
        RowBox[{
         RowBox[{"\"\<voronoiFlags\>\"", "/.", "out"}], ",", 
         RowBox[{"{", 
          RowBox[{"PlotLabel", "\[Rule]", "\"\<voronoiFlags\>\""}], "}"}]}], 
        "]"}], "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
      "Print", "[", 
       "\"\<Voronoi flag and euclidean length are set to -1 on non-visited \
regions\>\"", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{
        RowBox[{"Min", "[", 
         RowBox[{"\"\<euclideanLengths\>\"", "/.", "out"}], "]"}], ",", " ", 
        "\"\< \>\"", ",", 
        RowBox[{"Min", "[", 
         RowBox[{"\"\<voronoiFlags\>\"", "/.", "out"}], "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
       "Print", "[", 
        "\"\<On the left part of the domain, geodesic length (w.r.t the \
metric) equals euclidean length\>\"", "]"}], "\[IndentingNewLine]", 
       RowBox[{"Print", "[", 
        RowBox[{"TablePlot", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"\"\<values\>\"", "/.", "out"}], ")"}], "-", 
          RowBox[{"(", 
           RowBox[{"\"\<euclideanLengths\>\"", "/.", "out"}], ")"}]}], "]"}], 
        "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}]], "Input",
 CellChangeTimes->{{3.699355323064364*^9, 3.69935565573547*^9}, {
   3.699355692376671*^9, 3.699355805305409*^9}, {3.699379760115653*^9, 
   3.69937977050404*^9}, 3.6993798035567217`*^9, {3.699381343091857*^9, 
   3.699381766296589*^9}, {3.6993818040970097`*^9, 3.6993818600142193`*^9}, {
   3.699381916217976*^9, 3.6993821299895573`*^9}, {3.69938219564993*^9, 
   3.69938221774962*^9}, {3.699382251762224*^9, 3.69938247828176*^9}, {
   3.6993825554503508`*^9, 3.69938265843478*^9}, {3.699382771920095*^9, 
   3.69938279302201*^9}, {3.69938282624128*^9, 3.699382850936058*^9}, {
   3.699382994793545*^9, 3.699382996190102*^9}, {3.6998168713310957`*^9, 
   3.699816877545764*^9}, {3.720187255888266*^9, 3.720187257451576*^9}, {
   3.720187368634012*^9, 3.7201873803832293`*^9}, {3.729540808989717*^9, 
   3.729540819739024*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Field verbosity defaults to 1\\nField arrayOrdering \
defaults to RowMajor\\nField origin defaults to {0,0}\\nField sndOrder \
defaults to 0\\nField showProgress defaults to 0\\nField \
voronoiStoppingCriterion defaults to None\\nFast marching solver completed in \
0.000244 s.\\nField exportActiveNeighs defaults to 0\\nField \
exportGeodesicFlow defaults to 0\\nField exportEuclideanLengths defaults to 1\
\\nField euclideanLength_exportGeodesicFromStoppingPoint defaults to \
1\\nField geodesicSolver defaults to Discrete\\nField geodesicStep defaults \
to 0.25\\nField geodesicWeightThreshold defaults to 0.0001\\nField \
geodesicVolumeBound defaults to 4.225\\nField exportVoronoiFlags defaults to \
1\"\>"], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.729540820543305*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt3Hk01ekfB3DTioqGQoOppGkZQnaRpkR7JqeoJEnTkKIapVRHTSotjLi2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   "], {{0, 0}, {40, 20}}, {0, 1}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"values\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.7295408206019917`*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztWHs41GkbnuTcJyradNhFJytEEelwy6pkFTthOyxrVVQ66rTJlrQtbYpS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   "], {{0, 0}, {40, 20}}, {0, 1.2000000000000008`}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"euclideanLengths\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.7295408206589127`*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztkLEJgEAQBA9rsAEzu7AHSxCMbd0SNDEz2H8PZx9+4Jh09qbtWPchIsb7
5ijlXLrfrEJ3ulqF7nS1Ct3pahW609UqdKerVehOV6vQna5WoTtdrUJ3uroU
utfNtdDdLq6F7nbxV+h+2lnQO1r/3wO9p/+vLWdD7+n/a8vZ0Hv+8wUqUxLO

   "], {{0, 0}, {40, 20}}, {0, 2.}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"voronoiFlags\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.729540820709877*^9}],

Cell[BoxData["\<\"Voronoi flag and euclidean length are set to -1 on \
non-visited regions\"\>"], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.7295408207224894`*^9}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   RowBox[{"-", "1.`"}], "\[InvisibleSpace]", "\<\" \"\>", 
   "\[InvisibleSpace]", 
   RowBox[{"-", "1.`"}]}],
  SequenceForm[-1., " ", -1.],
  Editable->False]], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.72954082072613*^9}],

Cell[BoxData["\<\"On the left part of the domain, geodesic length (w.r.t the \
metric) equals euclidean length\"\>"], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.729540820729693*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt3H80lPkeB3DK7kVk3a5rkzamsUON/JitG3VvoihtQvQDK1qk2yLhZtux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   "], {{0, 0}, {40, 20}}, {0, 1}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> 
    Automatic}]], "Print",
 CellChangeTimes->{{3.6993556263890877`*^9, 3.699355727495241*^9}, 
   3.699355764937627*^9, 3.6993558057765417`*^9, 3.69937977346548*^9, {
   3.699381868869339*^9, 3.699381886761387*^9}, 3.6993821327620773`*^9, {
   3.69938220597718*^9, 3.699382218305005*^9}, {3.6993822576136017`*^9, 
   3.6993822669799423`*^9}, 3.699382363791168*^9, 3.6993824602555723`*^9, {
   3.699382608426169*^9, 3.699382625794599*^9}, 3.699382660019725*^9, 
   3.6998168797762413`*^9, 3.720187225098418*^9, 3.7201872582480793`*^9, 
   3.729540820751642*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{917, 855},
WindowMargins->{{Automatic, -19}, {Automatic, 0}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September \
21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 179, 2, 92, "Title"],
Cell[761, 26, 146, 1, 29, "Subsubtitle"],
Cell[910, 29, 469, 9, 87, "Text"],
Cell[CellGroupData[{
Cell[1404, 42, 7347, 165, 789, "Input"],
Cell[CellGroupData[{
Cell[8776, 211, 1274, 18, 310, "Print"],
Cell[10053, 231, 5418, 92, 210, "Print"],
Cell[15474, 325, 4751, 81, 212, "Print"],
Cell[20228, 408, 1113, 22, 212, "Print"],
Cell[21344, 432, 642, 9, 24, "Print"],
Cell[21989, 443, 750, 15, 24, "Print"],
Cell[22742, 460, 660, 9, 24, "Print"],
Cell[23405, 471, 3526, 62, 196, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

