(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 9.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    177154,       3138]
NotebookOptionsPosition[    176102,       3099]
NotebookOutlinePosition[    176460,       3115]
CellTagsIndexPosition[    176417,       3112]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Time dependent speed and value differentiation", "Title",
 CellChangeTimes->{{3.6993827283584948`*^9, 3.699382756437742*^9}}],

Cell["Time dependent speed demo", "Text",
 CellChangeTimes->{{3.699383732813905*^9, 3.69938375224631*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"prms", ",", 
      RowBox[{"n", "=", "20"}], ",", "dims", ",", "out", ",", "h"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dims", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"2", "n"}], ",", "n"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"h", "=", 
      RowBox[{"1", "/", "n"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"prms", "=", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Setup", " ", "a", " ", "very", " ", "basic", " ", "shortest", " ", 
         "path", " ", 
         RowBox[{"problem", ".", " ", "The"}], " ", "methods", " ", 
         "illustrated", " ", "here", " ", "apply", " ", "to", " ", "any", " ", 
         RowBox[{"model", "."}]}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"\"\<dims\>\"", "\[Rule]", "dims"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<gridScale\>\"", "\[Rule]", "h"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<seeds\>\"", "\[Rule]", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0.1", ",", "0.5"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"1.9", ",", "0.5"}], "}"}]}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<exportValues\>\"", "\[Rule]", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<progressReportLandmarks\>\"", "\[Rule]", 
         RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"Time", " ", "dependent", " ", 
          RowBox[{"speed", ".", " ", "Speed"}], " ", "is", " ", "linearly", 
          " ", "interpolated", " ", "between", " ", "the", " ", "provided", 
          " ", 
          RowBox[{"times", ".", " ", "\[IndentingNewLine]", 
           RowBox[{"(", 
            RowBox[{
            "Falls", " ", "back", " ", "to", " ", "first", " ", "and", " ", 
             "last", " ", "speed", " ", "outside", " ", "time", " ", 
             "bounds"}], ")"}]}]}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"\"\<speed_times\>\"", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"0.2", ",", "0.35", ",", "0.5"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<speed_0\>\"", "\[Rule]", "1"}], ",", " ", 
        RowBox[{"(*", 
         RowBox[{"Slow", " ", "propagation", " ", "everywhere"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<speed_1\>\"", "\[Rule]", "3"}], ",", " ", 
        RowBox[{"(*", 
         RowBox[{"Fast", " ", "propagation", " ", "everywhere"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<speed_2\>\"", "\[Rule]", 
         RowBox[{"Array", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"#1", "\[LessEqual]", "n"}], ",", "1", ",", "3"}], 
             "]"}], "&"}], ",", "dims"}], "]"}]}]}], " ", 
       RowBox[{"(*", 
        RowBox[{
         RowBox[{"Slow", " ", "left"}], ",", " ", 
         RowBox[{"fast", " ", "right"}]}], "*)"}], "\[IndentingNewLine]", 
       "}"}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"out", "=", 
      RowBox[{"RunExec", "[", 
       RowBox[{"\"\<FileHFM_Isotropic2\>\"", ",", "prms"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<log\>\"", "/.", "out"}], "]"}], ";", "\[IndentingNewLine]",
      "\[IndentingNewLine]", 
     RowBox[{
     "Print", "[", 
      "\"\<Front first goes slow, then fast, then slow on the left side and \
fast on the right side\>\"", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"ListContourPlot", "[", 
       RowBox[{
        RowBox[{"Transpose", "@", 
         RowBox[{"(", 
          RowBox[{"\"\<values\>\"", "/.", "out"}], ")"}]}], ",", 
        RowBox[{"DataRange", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
        RowBox[{"AspectRatio", "\[Rule]", "Automatic"}], ",", 
        RowBox[{"Contours", "\[Rule]", "15"}]}], "]"}], "]"}], ";"}]}], 
   "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6993828615767612`*^9, 3.699383387432989*^9}, {
  3.699383427593978*^9, 3.699383438281151*^9}, {3.6993834936994123`*^9, 
  3.699383721134756*^9}, {3.6994629214349623`*^9, 3.699462974757053*^9}, {
  3.699463021036889*^9, 3.6994630223832493`*^9}, {3.6994630917381563`*^9, 
  3.699463142359885*^9}, {3.699819704177643*^9, 3.699819709951812*^9}, {
  3.729540836003962*^9, 3.729540842028818*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Field verbosity defaults to 1\\nField arrayOrdering \
defaults to RowMajor\\nField origin defaults to {0,0}\\nField sndOrder \
defaults to 0\\nField showProgress defaults to 0\\nFast marching solver \
completed in 0.000269 s.\\nField exportActiveNeighs defaults to 0\\nField \
exportGeodesicFlow defaults to 0\"\>"], "Print",
 CellChangeTimes->{{3.699383348633525*^9, 3.699383389696075*^9}, {
   3.699383428619399*^9, 3.699383439039081*^9}, {3.699383511180781*^9, 
   3.6993835657889147`*^9}, 3.6993835971422997`*^9, {3.699383704226028*^9, 
   3.699383725911892*^9}, 3.699819710863113*^9, 3.720187313232243*^9, 
   3.729540842757724*^9}],

Cell[BoxData["\<\"Front first goes slow, then fast, then slow on the left \
side and fast on the right side\"\>"], "Print",
 CellChangeTimes->{{3.699383348633525*^9, 3.699383389696075*^9}, {
   3.699383428619399*^9, 3.699383439039081*^9}, {3.699383511180781*^9, 
   3.6993835657889147`*^9}, 3.6993835971422997`*^9, {3.699383704226028*^9, 
   3.699383725911892*^9}, 3.699819710863113*^9, 3.720187313232243*^9, 
   3.72954084276707*^9}],

Cell[BoxData[
 GraphicsBox[GraphicsComplexBox[CompressedData["
1:eJxlmwVUVNvbh8EuDMBWbERFbMHAVwUVu6+FomJ3NwZ2IwaIoqAoigEqooxc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   "], {{
     {RGBColor[0.148, 0.33, 0.54], EdgeForm[None], 
      GraphicsGroupBox[
       PolygonBox[{{1690, 403, 1135}, {1666, 403, 1133}, {1133, 403, 1170}, {
        1201, 439, 1712}, {1712, 439, 1166}, {1688, 439, 1164}, {1166, 439, 
        1163}, {1164, 439, 1201}, {1170, 403, 1690}, {1132, 403, 1666}, {1163,
         439, 1688}, {1135, 403, 1132}}]]}, 
     {RGBColor[
      0.26521687416719186`, 0.39512048564843993`, 0.6497745329502272], 
      EdgeForm[None], 
      GraphicsGroupBox[{
        PolygonBox[{{1161, 438, 1686}, {1129, 402, 1664}, {1162, 438, 1199}, {
         1128, 399, 1125}, {1244, 479, 1736}, {1172, 404, 1692}, {1089, 363, 
         1640}, {1664, 402, 1130}, {1137, 404, 1134}, {1130, 402, 1167}, {
         1125, 399, 1662}, {1640, 363, 1090}, {1714, 443, 1173}, {1208, 443, 
         1714}, {1662, 399, 1126}, {1736, 479, 1204}, {1171, 443, 1208}, {
         1692, 404, 1137}, {1202, 479, 1244}, {1092, 363, 1089}, {1686, 438, 
         1162}}], 
        PolygonBox[{{1201, 479, 438, 1164}, {1165, 440, 399, 1128}, {1167, 
         402, 443, 1171}, {1170, 443, 402, 1133}, {1163, 399, 440, 1166}, {
         1132, 363, 404, 1135}, {1199, 438, 479, 1202}, {1134, 404, 363, 
         1092}}], 
        PolygonBox[{{1667, 1666, 1133, 402, 1129}, {1691, 1690, 1135, 404, 
         1172}, {1173, 443, 1170, 1690, 1691}, {1689, 1688, 1164, 438, 
         1161}, {1126, 399, 1163, 1688, 1689}, {1713, 1712, 1166, 440, 
         1203}, {1204, 479, 1201, 1712, 1713}, {1090, 363, 1132, 1666, 
         1667}}]}]}, 
     {RGBColor[0.36015133342811534`, 0.4302768264443856, 0.6073551266748459], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtkEtKQ2EMhaNNr1tQHDlyD4oVrLiALkFLfSC14NRZB9WBWrBVSsEdiK92
YHUFLkFQi92I3yEZhHw3nD8596zsntSa82a2SpWoUWH2MWd252a3VB++Z9aB
x3AP7nlohvAavA/X4dPUt+g31AS+pleoI7iBZhGuwjV4D76Cz+ADuAtfetzT
Hu0bpp8+/AxP4RF8kW8HzCbpR95n8Az+LMKrPJ973NPdJfgw/bSLuK09dQ+d
9A/0beoYbtLfcqd2D1LfgJ+oH/jL45407x7/qf2b8B+8QMC/8EZmpcxe8u0j
/dtDUyqbLWc+yknfmmuH8tXbdeavHjkoj27mPKbvpOeqx7/Jwxb8D6lqQRo=

         "]], 
        PolygonBox[{{1197, 437, 478, 1200}, {1134, 364, 405, 1137}, {1089, 
         323, 364, 1092}, {1125, 359, 400, 1128}, {1167, 442, 401, 1130}, {
         1091, 364, 323, 1051}, {1205, 442, 483, 1209}, {1136, 405, 364, 
         1094}, {1199, 478, 437, 1162}, {1127, 400, 359, 1085}, {1242, 478, 
         519, 1245}, {1244, 519, 478, 1202}, {1169, 401, 442, 1168}, {1208, 
         483, 442, 1171}}], 
        PolygonBox[{{1641, 1640, 1090, 362, 1086}, {1687, 1686, 1162, 437, 
         1159}, {1175, 444, 1172, 1692, 1693}, {1087, 362, 1129, 1664, 
         1665}, {1665, 1664, 1130, 401, 1131}, {1211, 483, 1208, 1714, 
         1715}, {1124, 398, 1161, 1686, 1687}, {1083, 359, 1125, 1662, 
         1663}, {1715, 1714, 1173, 444, 1210}, {1693, 1692, 1137, 405, 
         1174}, {1663, 1662, 1126, 398, 1123}, {1247, 519, 1244, 1736, 
         1737}, {1049, 323, 1089, 1640, 1641}, {1737, 1736, 1204, 480, 
         1246}}]}]}, 
     {RGBColor[
      0.45721186679936154`, 0.46438755702213985`, 0.5502971773447843], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtkUsrhGEYht8xj68pK0IhC/6AnUZiyikryT8w5JAckpWNLLBSWMywUPYi
hSmH+QHKDxAN0vAHTNauu+dd3D3Xe3/v9xzepyu/OrVSF0LoRWl0lIRwkwph
10I4RgX4LPGz/HPiIyrDJfxnVIXLeFW4gWQ/8DvKwE943+Z3dPc63n/AW0KL
8DZeBU64/yqP8xB+G9xdH0IPfg2vg/MI/jDxDn3Bt8SsvsE1BvhLO/8S72Nd
1dT/ypOQ7wq/gn9h3p/6/IAt1lLNQpzxlO95NAf34c2Y81bi8+vOCV4p9vOp
mubvkCHPW5wrBTfDOe40EicS5yZ4Ms7bCnfGGduJA2gBnidexp5fzPehvWyY
S7xObDHPo3xj8DK8Zt6reu43r626qr8Td6qdD8Z/9fbZOKPmLqJ9eNZ8Hs2l
Pe+Z+/o+HXkz8V1qpznzs/w8fKCdwYfmb6p3K8Ljsc9R851pd9rnP9IzUnw=

         "]], 
        PolygonBox[{{1248, 482, 523, 1252}, {1281, 518, 559, 1284}, {1091, 
         324, 365, 1094}, {1242, 518, 477, 1200}, {1084, 360, 319, 1044}, {
         1240, 477, 518, 1243}, {1283, 559, 518, 1245}, {1251, 523, 482, 
         1209}, {1136, 365, 406, 1139}, {1207, 441, 482, 1206}, {1050, 324, 
         283, 1014}, {1138, 406, 365, 1096}, {1093, 365, 324, 1053}, {1197, 
         477, 436, 1160}, {1205, 482, 441, 1168}, {1082, 319, 360, 1085}, {
         1195, 436, 477, 1198}, {1048, 283, 324, 1051}}], 
        PolygonBox[CompressedData["
1:eJwtkDsOgUEUhS///CLRSQhRWYHWI0EsgRV4R0OhpRQanWfDJmg8NsAiJF4r
sADfzUxxcs49507mzKQbg2o/KCJR4INuSKQDSkakFxApwy/mJ3igwyz7LNaY
qyCJV2EvBl/BG32CD2RHsAQLMFOPbAJHQQmdh1tkbZBDd/Ca8Ih57KC6hTcn
q8Mh7s7QwfNtJ+3280RS5AljO2m3hjszNfZ+7fFFRzj7gm/MdxB3/YvGvrvr
fM3P7j0fYztq14LrmYW3+mb0Ct6R7R1UD/FWZBtj/0H/Y+32L/Af824vfQ==

         "]]}]}, 
     {RGBColor[
      0.5542724001706076, 0.49849828759989406`, 0.49323922801472275`], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtkrlKQ1EQhs+NcyOKoLhCiEJeQAQhV0EkFkEtbLSw1ASCXQQ7C0s7W3Fp
fACDjbgUGh9AsFXRwuDaurd+40wxzMc/c2c7N1denq2mQgh5rAlLxSEMIuTw
a+kQKlEIoxLCIzyEnkZfhTfQF9GP4Gf4HJ7CqvAK/haLtTD5M+SMo3egvdMk
AydoJa+foF95nQv4FK7Du/Ce8w5cF2ONb/oMZbRXsW+1xg/1s3AGrSRWX/c4
xN9jNbQDMd7X77A25rwkR3z3CL8gVl93PfP6umvNv73Gb/s8OmPee+lOc2nb
8YtZesRYtTu/yQ3+CWuFH/DHWIOcE43Hdme9d0tss73AA8SK5GTFZv3X4XWf
U+8xInbnCtwNF+AxsbfUveaxJWIT6L3o336rT56pIKZr/Nd1jTfHNuewz6F9
38ht9176tv0+W1Gst+qdcJfP0Ic1fF+tN+3/ySR+KzJWLfG7fdD3D0DwTEU=

         "]], PolygonBox[CompressedData["
1:eJwtkD0OAWEQhmfttyI6iZAIEg6g0dBpNlQkGr0DrEScwQEotNQSjcRVEAqE
E/hrPWO2eJJ3fr6Zd77KcNSPEiKSAwerpMgSJgQLT8QLRGo0BHAil4E8+RZk
0Xs4e1Y/oosQEhfh64sMyFeZ8WLmG8bUI89m644UtTo9abgTX5zpK/Ud+omn
J3MavG3CHGYwpbalpx3P051l9Dr2ozXtCWM/2tNBF5x501wJfXDmXd9s0Ddn
u/8enHnXG/QPHFoC69c4EZg39agzP779jf5RF4+9pN2mN97RD/gBMOEo0w==

         "]], PolygonBox[CompressedData["
1:eJwtj70OAUEUhS9mJRKJjQSJTryAit1G4gX8vQBWtPTUOhUNKl6ChvUCvIVF
S+h9k5niy7nnzOTOmVIw6Y7jIuKCA30lsoiJzNFlUmQFW+aQbI3u8HuoMY/I
BugUP4MLc0R2Qg/4I3wSIkWyHFkb37HoOU/W4KyADvEB+HZvFX1BmnIReuXs
Bi1oQpaszr2MMrk+P9v3n6iP9+Br33+jP+0hyUcr7C07prfuH9h/95Tprftv
7L9D1LXvecr01H0juMPD9kyx7w8LNioi
         "]]}]}, 
     {RGBColor[0.6513329335418537, 0.5326090181776483, 0.43618127868466106`], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtlD1IVmEUx+/73ufeNws/IiqDetWUWnTIj0IoyoKkGqLAVzFLy+Z0qYag
Gpx0igajphojCLQPaMhayyAoLW0qKK1BzRRrqX5/zxkOz/n4n+9zb9W5vpMX
slEUXYJi6FkaRS8yUXQnRNE8io3wTejaAV2HrkHv0M+B3Y7+OLQJzOaMydK/
xz4JLcDXoFubRFEJfrPw5eDmI7PJR3jJp8k3iHwZzA1oBH4gmCz9GfiJ2LDV
6H7xLkHL6JqRFyPT3QLbB/VDB8h7lbw9ifmVe51fwTeib4Aewk+jux8s56eM
yc+Rv/E+Ta1u1f8D3SQUw//lXSZmHsw2qJocHeiXwb722lTPnPepGbVhr4Xq
oH/B+Ar8/sQW53ds+UPWctT6K7kb/HlsV1Ij8XvQFXwn6lNzGs1YHyPEekXu
UrBTwWJ8CLZD7VK1fkYu4v3C+yi2ustSm5/mOAz9jG22zT5zxRTuXmqYi8Hu
Zdh9bqIf4u0JVsuQ68QPuvyYmOPEWK89gOtF1w2fTWw2Ee9Aajs85rXX+TxK
g81Sd6e5r96A9oD+MHJrsPpXee0beQvvCfArsflsDbYT3cOGYDZhFrHvRt4H
35vabOqhXVnrW/1rFg2uz1FnMfi9qjO1nIqfd3/FKcPe6THks+TzU/8H8T+F
bk3ObOI7/YY6fE+6S93jhN/dtN/o2cT2XpGzGWlWR4Pdum4hj74y2Awqgn3D
+pYX6HEGuRjMm9TuW3c+pltI7LvQ96F+1FdJsDwFr0d71r7HoCe+R+2z3TFV
xPiOzwz2t/qX+H3ov6Ida9dNwXoY8Xvt8m+x0WtSbbNg7qaW53awO9YORslV
k9icWniP+L5VV2Vi39h+n0Gb16Q6VM/LYHdUcNuw96E8U7Hd0Uf/Flv9hnYQ
qwnsCpgHXrdqVr1dvrMWryeXM79+/weN+21q5/JXnHVgDyXmW5Qz/3rvX7L0
O7H/B/RZrJA=
         "]], PolygonBox[CompressedData["
1:eJw1kz1szWEUxt/2f/7v9ZFeNZBIuLeNloWB9GO5DSHREBYJg++2ZrVIWDB0
YrIJm1kiKREWrRVhoKpMlhaD9jY+F/yePNfwJOe8zznnfc4579s9On74XHtK
qQECLBUprWtLaQgczCmtBS84fw4ewG0kaBPcPlDDPgN/FnsM9OEPhv0GGIEb
DseOg/3Y85z1c2EfWFmmNMHZJNx7cA+um7OjcNtBe2kNQ616HcRWONupXPAR
fxlNG+Ca6Bsk9jZn0/g3wR38ajhXmqT1FZjHXgCf4frD2q+Di9maG63+B+Au
hGudBxNwj7JzVeMJ3IEwNwy6sDuzZ/UMTKKtB38JexHM4G/Otr+C1/i99LOD
Xo6BHuwf2bPRjLbg36LmVEuv7j6UvZ/1YA3cLAhiP+Dfxa6Tsw3/COjC7iV+
OXk+s9y3opLScbgTYC/8AnwHdhWswl9d+m6diatUrE05e+C2ls6VRmmdCs9C
+S/x68Rfwb4KRondVVrLZbAb+xTx19q88xvED2T3op4W0fcOfE9+cw+xL2Xv
R/s7Te7j7Hlox9PhnWl3qnlS7zHsK0e5neFdqr5mVw/vqgZ+U38m/PYK9L0N
91D9378+ROm3qBnP4T8N70IapOV+4V1rxpr1m8K7VU/q7U+4tnbyF/tX4bvV
41zhN9ts5TfVO/iW/Md+Yn8Ka9FO9He+hPVp5nrLtYpnq5mPoPUfz7t1CA==

         "]]}]}, 
     {RGBColor[0.7483934669130998, 0.5667197487554025, 0.37912332935459947`], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNlMlvzmEQx3/vb3tFa3flfZVab5aSiiW2qDipi0QtjS0OpERJyoGWhAsl
IbZISpFIqCVCXFRiO/Da1z+giiKRlNo/4zsJh8nMM888M/PMfGcGVa+dtyYM
gmAXFEGlaRDchV/kcA9+AmqGzqE/nwmChjgIXsJbMjrXx5JPw5uwaUU+jLwZ
p5XQfKgGX3X4aEuCoAqbpdiMiXW3Jfxna7zO7Lk/BB2ENsaSa+Gz4TkoD3VF
knPo5+KzP/IndKNj+bc4Dzi/I24OuSGVvw3cL0GeiNwTuRz5KzavsZ2aKJ8p
8Eb0O7FZFCuneZ5jndfkOFTFXTU2m7BtR+7FfSFVve4HXr9U+ds/PhKjA10Z
unec29E/Qr7mdTuK7iw2N7EpRrcPuQn5O/l8cZ8FaA/6Y/Bv6FuQbyH3wP5K
pPu+yJ0e397sj5TTz0TxLO4NYl32Hr+A13NXIP/jiWrTF+rn9TZfFr+dt72x
ec/bHdidQS7LBsETzhnkH/CRoeQQus75De+v8m5CqPrY+2W83QrPZ1VTezMK
qkpU6wHotyI3I4/NKnb+v95XZISFWdgsxqYYm4GJejQJXhLLZlCsf9h/yqEV
3DVgU4L9KuTtyIORx6Sy+xDJv8UzTHR53sXYloWKtSgU1gxX7wPh5WSoXB9F
0uV5V+v4WUgOu1PNiM1K3nE8x+YoUm2/RsKQYWlcLN/9vf41/lf7j+HG8PMk
Ui6W03jvmfWugr/MSfXXYvxM9npaXbf5nFouq1P5szot4H5bqH50RKqD1cPw
Zzg8h25Gon8X4f9SJOz34f6q481wNySRr1Lv22afbdsVoWOi4HX9289E85+D
3051bzj6nKqP1s9q6LTZEHcp8inTI4+N1VOb49+xsPPLsVfps1rwvpzymTTs
vY2FRcNka6yZt3xewS/4rPVOhWnD9spE2LJaj0Ce7vjphm6549/mYLH/2TAb
OpYtl2exzk9j2Vqd66HHPrODU/XMct3O26fob6Mfgn6vz7jN+vNIO9l28/BE
mDRsjvL9Wuk4NjzPTPTXCY6N7o7dtlT71/bXOvz1iIUx253rfT/XeMw7gfKY
lijfbFZ7xPpmeD3iO9l22wHfLbZjLIbFKkq0m8xXI/qhtvfRd6baxYbvh5HO
ph/G/R/9atEs
         "]], PolygonBox[CompressedData["
1:eJwtlMtvTVEUxs+9Z597Ua0qU9xbjeeMFql4P0KvUWMi0dI2HjEgiBqUgT4G
Zp4Rr0hbpUi09QwmSKoM0KKi/oJWvRIJRYvfZ+3Bl7PWXmvttc+3HunKXaU7
40EQVAEHhsIg+M53SiwIfiIXJYLgI/on8AW9DXR6PY3tF3oK3yH0YvT3ockv
wT3kN6H5KiYf+5QoCNaTcDb4S8JW0E58H7iCvB+fCuRKUIbeQ/wgsRPQCxXv
7G1rQAo5nQyCOu6qB1u5e4SzWcil4Bv+2zi7Fjcf+fZzNh95HhiDrQG8RL4M
irDXol+K21kd8lnue0SuAXCf2BJnuXeDDPI6Z/JasAP7oshy14DFyKuUA7lM
+bi/B59+f9+gMw7Fjzh+xb/u4+x0zHI2YRtL/AJic8EA+gfs4+L2D/qXH6Dc
3z8N36yk6ZvA6shyyqazlei9xMfjxvdV5HfkfB5YzftCq4FsMXy6uLuKmNq4
cbglshqoFnngE/7lnB3A1qKcyEXOalcMNuE711k9D4Nq9COgI2Y1r8U2Kmm1
U32WE5/D2cKY5cjg25wwPs6Ac9imR/Y/4nQ0sZOSxrXeoLeoRuJWHD/Efw9v
rOHfmsEw9pPoFwM724t82+vi4A5yB3iKfAEcUy/7flENZhB/l7MX2FrAKeTH
zvKpXg/wzUtY78tHvkdDu6sL5GDLiqz3VA/VbiSyu/QGvS3F/xyKG+fiPhuf
J35+rmPP93zp/7dj/+NsllTTt8i/OWvCtxGcUH2cca8aaLaGndVWM6JZESfi
RjHH8a93Vh/FHMX/K2cTY9af/3eBs14Uv+rlR85knXWjp/y+0Mx2E7vZz9NB
sBR5qv8f8dGL/SZ45t87Tv3urHfUQ+Xon0PrNZ2VoCf9vIufZZH1mGzqmUJn
HOtu5WgPrUca/H5YgX/a2axqZ2l3TU7a7OuNeuvMyGZHNVftM77fxUE2sef9
PlBPqjcLfD71zNvQdojua1VO7qp21rvaEdoV4khcaYYHnP1zo+f/Rmg7stO/
/zV6bsJsqtEt9MKkzZreq10ljip9fedoNyYtt95QERlH4moDKEDe6PtBNdYs
Zvz+0v/Uo49PWC71o2ZjSWS1085ORbYz+nx8G77/AHO5zx0=
         "]]}]}, 
     {RGBColor[0.8454540002843461, 0.6008304793331568, 0.3220653800245377], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVktsVVUUve+de28TfhGYivKKaEEnIkZK1JGfhn9bcCCEKsYfAzDQfywg
Cm0H4peotDRQpLQG2joiShzYNmIEaj9QjDMTTaCAE/nZluhartXgYGXve+45
Z++zz95rn9ymrSVbslEU3QQCcDETRZ1AF1CH77XAOuBtoNTflAuBh4H3kiga
wAaP50XRM9D3Qp8CfYfXveA1nPsIcCWOopmYM5hCwsYljF0GCrD2JYzfxngV
vlcBq4FqyzVALVAMlAC7Mb8f8xfD1hvQT0DPh/4c9HroU6G/Av0r6HOhf4bD
tWHdScgxyEFgCPgbGPD3g6nGTmFOC2QFUAm8CJ/3wtdy/C8JOtNF7P1N0Npf
LY8B7cBr+HccdnOwuxH6l9DnQD+If+XedxT4Huixba7rAHKw0QfZhT0/Bg5B
/xryH9gfyeh+VgTFZxh7fwC9Cfptxg//ngCWAO9j/ADGb2F8XlD8T0PPBd3d
D9CnxJpfBJsbgULo0zC2HXIZsDyjsXsyuqv8jOLF+PF/zmPLLecBa1PJCdh5
IOjOf4StYYxfwXgf9p+NeIxm9N2Lf9fxXYb4VGD+Lsy/hLHvGAf8b4k1r4dz
gbeC4vcH5mwJiufv0I/Fis8Q5KqgnDmP8SirmP0CHML3T/gugq126CPQV0Bv
g34B+nLoS7PKz1lAQVZx5x6nU+3Rgf0Xp7q/QdjZHlQb+yEnEuXoNt9blfP3
LNYfySoPhoCOrPLyDNDqf5uAwqzslzrHmGv0i7nE/B6x7AQ2wNZhyJ2JzsB/
DdBbHU/G9T6svwH9JmsA+i3WF3B/0Bi//8JZFkDewbmKfWZ+rwEe8vdYKr9m
JKpp1nada5HnrXGN15knxn3OYeD5RGebAR/roXdBX5InvxiXc1ndH+9xK/yq
dA5UOMerXIPjsfy5F3PfBBqhz8+7yxFV5pqd9q8Rc7oZU8yZsC/nHQvaJHdc
9x3QD+bFYd/Hk1hbDbkecnqifL2CGOyGDw3IgQ9TzeWcGmB1ED9dSJQvBY4d
13DtNazrjVSPrNU2x2qHOWOH/W635Pn7PLcZmJGKjz6CneZYtXAEY8vMoX8G
xZXcS+6b77pj/bHOWe/5QfFi3Iow1hurvlhnjBFjtRnju1wjrBXyC+9gZVBc
eNZa5gP523dc41xh7nTbfoPvm/e+MBH3/L9vlJrD6Qs5/+Ws/F3neWNB3JKD
j4ticRD5qcecMeo6ZD22x+ohM81Tl1PV0TTYXZRq7GoQDzL++4Lup9h+d6bi
DfJHrWux1XnCOmENdfisg+5tjPOziXiyGWjKiLuHzOXM5YO+u5mu6ULbrHXM
XnVvYo+azAPmxbUgbiHHdJrvR3zm2ealftfuTXNbZJ7qMs9xrNCcwtjOSRWj
n4N6Evnk9UR+tDrv2+07+9gR1wRrozPoLNOxxwagDHs/ilh1u0cyNydz95h7
Gv1nT2cv5/0uSFRz9PeG417j+mlyrrM+2IvYw8oc0z5z/mSPYc/hO4PvjT2J
aoA5cTDWOc64jtm/eRffBtlvdE6+6zfDbsd+ePJe3dPY2xibC+aKpxPd11Pu
4ecclwP2tye6y8tcw77U4rs/6ffGrFT13OEYpXk6NznmvzcQ9Kuxcoo1TG4f
CNp/bqrezLhMZV8DyqGv9Huky2+1MvvHtwbPkO/ezHgxR7dldM5x126Tx7Y7
ruXWK1PF/7FYd0Cuqwf2pJpD+ytira2MVevd9vuLoDN+DrnPufsO5vyWigvJ
36dS5XEP9P6gnGRubk61J/duti+0t9+99U6i9wTfFeOQS1OdjzxBLqafG7D2
eFD+TeE7Liiv+J5ib2NPm4D8JKhnsVczfnt9xk+D+th4orxrcm3zzcc562Pl
y4TjyPpjTyVHHE01tzpWHs1zzrIuaZd80+C8ZnzPug7Yf+grfT4RxFWFfr/9
Cztzk3I=
         "]], PolygonBox[CompressedData["
1:eJwtlUlsV1UUxu///959/8S2RnArhVIcimwEI0OCK0VaBBmUBZQUiSMLZsug
pgyBlgXgkIjaYhwKBQMtS0A3lhqMRiylVKNsWDBVYAMGWjD6+/je4kvOd8+5
59177nfOq1q2cv6KYgjhXZCCyhjCPBZqwM0shEksLi2EMAUsgffjf4m4OeDF
JIQW+ACxnWByKYQm+FfYP4PN2BWsvYq9DLwALyffVHKNAHXkqwRXyXUFnCbf
ADHzseeBufDx0XungWFif4cvxPeKYvDPIv+5os9wEF8ZvBl7B3gOvhwcxW4B
4/A1gFvY/4Ce6DPr7F2gGX4JNJL7HbCK/Bej7XVgBXwxfEPRd6zHvs59Hsb+
m/uc5XxDrLUR2wr2ED+l5Nw6w07lIqaZ2B1gEXvvRMd+DnYRX89aS8E1byR2
A7wNex3YDq8jX3/Rdz7A3pkl1/oXnUn1LfmuqsGM6DuL7wRv627kW0OuVrAe
exT+dnzfgCX4e1Lf5ST4nu+dY21uyN+c8/VLJMT+gf8U/lr234HfVg5i+1gb
iT1CNSF2NXsWsOU9sBa7j7UB9v6nemFfYc9mfE2qMf4vWOsp+M77yfUXa0P4
quDT4KeIfxI+Xmv4JuX6GQTX4WMT+6QhaWks+eoKvvPyzJqRdiaAR7HPJN7b
DUbjn51Y29KAtFCV+tvKoVw6o84qDS7Avpe6V1QT1eZGYm3rTDrb5VzPqoFq
8aHumOcfxnctda2mgqvEd8MHC66hajlK9S9acxm1Lo+OVU8NZa6havk4/AJ8
ZvRby6/e0xvqLVXTk8R+wPfX8+0vwUfYn4JDwW/QiH1CbxS89lninlfvq2cn
sn9Prl9pdCt8d2Ltqn5l+MpYm4yvAdTDr+F/KJ8ftfC78GrsYeLPY69M3Fvq
GfXOEfgPwfXTbKjNv6+cD5C7Ou/XI+CtaM1L++rpr7E7wL2ie0S9sh9+RrMD
tEsPJe89DN6AV5NzVsFnejnzTNBsqOQcP2L/FK0Vaa6atX3Edxf8Rt8R/2Z0
Ln3vGXIfTN0bXWAb9l38G9m7CXzM/vLMd90HOuHHQAd2LziOXZHZJ40chf+W
eDaqhhPx/RvdS++DTxJ/U98+q5kXvSbfAbAXewtn6Cy45zrZ357rQXdow/dI
tHakMWlNM6E173/FSmPS0lLwBP4ucvbk76W7PBg9m6XXwcx31t2lEWnlqdRa
ENfse5rznib2V7CFvbNTz7b7b5BaE9KGNPgY8X8mro1mgGZBTfRZNNM0656P
nvXShLSxHd5b9EzSbNJM0mzSN7dFf1Pf1sxrwJ4ONhX9z9O/TzNQs1Brz0Zr
WtqWJg/z/TGqX3AN+hLPDPm11gsfmbn2elO97eLUs15vsDtzz6v31YPqRb2Z
3u5b8Hp0DVVL/VMq4GNK9h0Cr8HXJNaSZopmy/9HFAd1
         "]]}]}, 
     {RGBColor[0.906791459050414, 0.6369786476260352, 0.3110535230562836], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVktslkUU/f5/vpkfJQUXspCHCbR/8UnBiJpUt0oLbUVFKaAm6kZRqAgF
FypF8EEFl8LCthgBbamJsjOoGGmLiY8graBRo0bQKKBiTJTS6jmcQ8ri5t6Z
+5i5z5npD6y8Y0Uxy7KfAAFwKRa3AG4GlAFnC1k2AqiNWbYG62bgoZRlp7DX
l2fZJaUsewj712D/BPYngf4d+9VF6Y1av9r2aJtnUO6HJP5hyL+Ps38GHAdM
LAhznWC/CbLTYP8bXHAYe8PAg8AjgLOAMvQboVMF/BxstoJuAr0Weo1F6bcC
GrzmfWvtI+9xAvInAV+DfhvwDqDTOrRx0OsuwBDk9gB6AbtwVgfwkzirFvfc
C/5y3PMJ7G/EfnM+5s+Egs6vOh/XpFhMhPwc0BXg/xZ0P/IZr46iYs57nPOT
PgLO5LIzGbrHMtmm/nVJMfs8SG7Y8emE3B+Q/5A5hMx94M2BjaexHuA5wP/y
3pBdGSQ7YL/nFiR/L+B642UFFcsR8I8ClibtzYbNTeZTbnlSXpifppLi2x5l
j3ZvYEwKytcaQF1SrE6EsfPmmk+5RmP6xlhsTTpvKez/6ft+XJTdCsd8E2Re
BV6X616U3+jzmDvyapNq6Tuce9gxG3a8z7rOqpP2WIMd1m81nzqsR7iYzQbU
AC5zHHmvKaBPA/8FuBx29oPfi73JQXzyEJZsFuBa1rxtcD0lKcefBp25wDEo
XxCPXvtE36i3GHAP4G7oLAFeBDwDeD6gHnAIh9WRDrrLK4BtgCpAA2ABoNKY
OrTxLGADYFxSr/ZAd19S7xxAbPsKmgnso++T6pN1yt5mj39BOqn3T4Ie9Fzg
fBm6YE7wjB5AN2BLUq+tzxXjbvO2A1oAjztG0THbmXSPjlxyG3zvZtNtgNYg
vC3IXpt5jAvrf5Vz2Ow4tvg8xmd8Er0Vutut/y7w31ExfCkoD72eEayt/oLu
VOM7skZGo+61DvL9STE4lCtns7KxHNb4LhXulbmOcZ/t3prUIxfl6okG18cV
QXozg3rgmOfqXUn8ylz5X+UYLomaNU9F5WDQOem3D6x3zhXm/QPiJF5n7vnh
+VgN59qB66Nm4eai1m8lxaQtV4+xn74Nine9a+w9z/R9wMej6rAhiDfD9ch4
MW5rg+pukt+SLyF/JeWwXw7qoYVBORiyP/S1zuf1Ju3typX/xc43c7jfPv4Y
lfvHYOfOoNysDqqrQff8+T6pcg01u1cavFfOFOtzPZvrXrnrlTkv+z14LWo2
7AB+nbMY9M6ouTDFc4L8qUFrzl3O3/+A64L8Yk/fFhXzipLmCvU4hzY7H3yj
eqLmMOfxR7niyD5+2fngOzAuV12xvk7lqjm++c9HnX0jdF+Mmv03gX4T9Fd8
W0HfHpQL5mShc3EEdD14I5DZHTWjT3s2rvf6GeDpkKdrA6BjSW8f/x0NfgO5
PoN7PAh8FfYrot7xX5Mw33XyHo56x6tg4+JcbwrflrooGf5Z6AN9ecGyjAl9
eDQo78x/l98TviuLPEuPYn+ZeUtJJ/XmeOYZMtWsD+w3gp4JeihKn77yPagM
+mcdjPpbdLl3Gv3XoJ9Lcr2RfKs/84zmu7HadfgLdHdFxZNxfQP0KOj5pbFY
TMDeiqAaPQb6k6A5wPekO6iXOctbguqVPddpXwd8j073Nv9e/ENNBd4S1Ev/
gN4TNA85F9uD+oezkHFnTB+JqvFG/712uM7nlVSjrNV5UX+dJsv1YT0Ntg64
lvf6P3Z11H+J/6YDlrkfNv4HgT9d0Q==
         "]], PolygonBox[CompressedData["
1:eJwtlUtslVUUhc/f85//8gjtRAdqGfBoFVECRg2IjimljwSJ0IKaKBNFAYWC
DEQIIi/BIY/EQkys2lISdaaAAlI0AQm0hEdCAlHUhIJGEiVcW/kW6w7WzV7/
3mefffZZZ99xryybu7QqhLAS5KCTn+NZCDfAoSKEb2II3XzvB99iN6QQtrLg
K1BXCuEP+Hx8C0AH/h/hsZJrIsa/8Lew3wbb4efgD2M/Alrh6wjcz14DoJf9
HsLfTO4OkMh/nW/PYs8EY/C9B37C3gcWYR8DteS5Bf8E+zT27+T+Doxl7cjC
9mHQja+V/VaxVwtYgu92cm0rwEf4/4Hvwt4JtsHXEN+Zec0HxF/A346vDczD
/w7+jzP37FP8N+D3Vblm1b6Z+PPY/4MmztOV+6z/gT78v1T6sxy8Qb4TcN2H
ejiBn9fhX/LhIpjB+rPwRnyzBfzryddLro1gB/nO5M49kfgr8FriW7HrQRnf
PvjfVe7hOuxHwavY20A1+cvkLOvuVCP2qML92A32w6eR4yV8T4NF+Ab5VpO5
50/AB8hXH3zHLfjOwx/HngLmwj9j/bnMNR8gfohvddhD+GfC9+Lvy9zzLvgP
8MHMPe2HN8Ob4B26Q+yFuc/+IliJf3Tu2qpBQ2ENS8vKcUzxydpZrTvCrk7W
lnpwp7BmpV3VXIfdj78u+ExN8FpifsO+Bk5G90y905uYlaw56e178DP+NpK9
n1nj2/Gdil47Bj6tcI/Va51pnvbPrc17PcF3HH4zc08O4++Nzq070d1cj871
FJhV+I5118Pwq9I//EndFVgIn5rbVk0r4Jejez8ADmHvAT3YG8Aw56kBz5Hv
fjBI/I5oLUizetuaAZoF0uhr8MfA4or+y4VnhmaHNFUPbylZy5dAD7F9gFLu
aX4csc34L1T0/gW+g9G1qSbV9mCytjUjNCvejH476ofe7uro2vVGX8D+MPot
6040C6QJaUOaHUUveqJng+5Ud6sZplm2HqzCPpu7l9KstLsr2qeZuBt7WfTs
mAN+TX6jql1n2AKfXjmPZsAmzTadMVhT0pZmiGbJVM1E7Mbot63+ajboTnW3
usMR7D8e/6Tgnql30rC0rDdesNc1eFPwHsuJfZlvY6NnpGalalStilka/Yb0
ljTDNrJXY7Ke9R5qWNuWPIs7QTv2n7m1oDc1QPwccowPnkmaTc9U9PC13gT8
3eS1J8Ba7IaSZ7VmkGbRkuRY7Vlf8ozQrNB/yvPRNagWaWo2sd3Jw1E9lVbK
lf8HaXSy/luie1mS3rA7k/f+C+xNrlm1603dzD2TNZuHwec6e7I2dQad5Ujy
2gfIdVS9Kzl2CHQlzxzNHs3EdvLdBcA08Dk=
         "]]}]}, 
     {RGBColor[0.9222963365857569, 0.6757408414643923, 0.35911864341584643`], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFVUtslkUUnX7zzfwUSdCQgCKJggaIoFC68r0SsFURg1JaViSysBRiSsLD
Jw81wl9XygJ+pBvRvlipG8RX2kKihBSkMVECSd0gtjWamAixco7nJC5O7v1m
7r1z586595u/adtzW4sQwudABDYCbwH7gQt1IfQCfcB5oMffH+cQuiF3lyF8
AqeTsM1YO4O1Y3XaI3YAO4F9QCvQBjwPu0WQU/BbBX0G9GvQ/wEWQr8XMTuz
bJdDfycrxtpSZ+8B9jqnvY59ZwrhKVxibtKZ3c6jH74jkB/B955CNs1AK+yO
Qm6A/AM2TdDvSLrjDeA60JUVewN819QpN+b9CNb/xX2vIN9qVk3egM0Cx2as
G1h/EXJp0j1WAI28f5ash/1wVn2PlzqTZ/N7JeKuRPxzSbbTgVuAUdhPQp4u
Xb+gPJYkncUzF8O3AWvrIFdD3gXcDZyHzXzI5qh7MscHk+454rNZI+rnSq2z
XrxTFXafQT5aCeFd+L+POH+yftBPQD8CeSXrDiPwfRy2s4E5hWJcd03Hs9Z+
h81L8OlgDRHnatQdl2XVb4H9+rH3M+RanDuVFfdW1hN+K4BGYHuhd6wZQ8Bg
oRxPAV8E1eY9oKsQR8nVHpz5fZTNPJ/LWJ3F/9wi17a4jmM4t1qoDp8Co649
32CPbcmDp5FrHXx6Yf8Q9J9gexD6cZxxmpwsxUv2Cb9ZJ97rMeC3LDkT9j9k
7U2WutMEMA4cw94k5IeQw4V07j2RxbEK7C9n1fBv6AeRyzbk+BfsZ1Z0h6Yk
7jL/y659ozn6dlZ+u+C7sKK7tvvM22E7J6rew651L74PIU59Vh7cnw287vxe
g/w2yfcbyHmIeQTrLdCfjOIkudnp9+Nea9asWIYcBoJqfTEovzbPkS/dC0Ow
mQt9DPvfRb3HIvfqlqR3X1yRzz7Pix7znVxvz5oXa9zjOz2zOGuoP1uKL78E
ndHrHqX/y1lv3wKbh7PO5Ayj3Qz3LDnN+v7qd2BfrwK+9qzsLnX3Js+OenO7
wbb02QoMl7Kn36tJHK8lcWLQb8H+JD87kr5r7o1B17XmulN/BXIgi+dvlnq7
Ib8re4L98wFwGHn3s4+i3oA632RWVu9zBjRk1ecs9IlSvCV/yYsO589/SZv/
LQP+5j/mhaj1HVEc5X07ovhE/2rUDFvtum13nTjHOTsYvz0q1gnH5toh539/
Un9xfqSK/Piv2G9b3mcqSfKuux1rV9SMYx0OQJ9W6k35v6p5TjCX2xBzM+QD
5hpnzWbPiapnDntiwr26JGpu3Rc13//jg3tm3L3Mc8bMuaN+468gW6Lqt549
6/lHrpDfPLs5aVZyZnYl7W903deX4io5u87/iB/Zg55rnA0876RnJmciedAH
256kmcbZ1ge9gP5MRb120f15yvy4BHkgae5x/nFu8u2a+G9nHdmfUXmT69PN
9Wn+HvW78C1vAqT4HPk=
         "]], PolygonBox[CompressedData["
1:eJwtlElIVmEUhu/vHX6zICOwsKBs0MoGh1UptcqpSAnMqVVUi5wgA4fmmfS3
VbWx0pVlaqtalVpQGlSkhBI02LRpNKIWWVnPy/kXL/c99wzf+b4zJO2o3VoT
4zjOTuCBiOs4F/jWgJ++4/Qjj8Gvg9mB43Ri9DTkON2gFzkP/QJ0uWAY+w70
D9B1gCvoi5Ab4IUgGV4GjsMrQB36avxro/5v8T/iWWzZtKJv8iyWYpxGLsAm
QsI3QHLYcQb4NxSyMwex3Yt8DH4UlCCXEH87sTNBKvwg/oP4XgLl8BWu6eLA
cngRMZ+jewF60F8GEzHmcwhe5pvvPpAFr8enAt8ToAH+EozCp8Ar+GPwHn4b
zCO3QuK7/IsB3b7FUKzNwEe3EnkXvBXEI48gL8Q3CeTjMwVSuNs/5Gzi/UB/
Hn4OnEH3DDkdngGKkRN9i70YTPIev8AieAH4HpiNbHWHbfB12NehawPzOX9L
2HINgS50a33zlY1sl3pW23pQSbxpyJnw6SAnsDvoLqpZPvZ3wVzX3rQdvhrs
hq8Hf7EPB/ZWt8BV7LLCVgvVpBnbeLABPgd8wXY0MJ4AvnJ2i2+2emO99Uzf
YuuMP4H1qHotR8Am1zWufn+DrtK33tKZOrvQs7u1g1P4d7mW2zuQGNiMyFc9
3IIuNrB6aIauuVZz3acPPEL+BGaErCfS0W3C/2w0v1mc1+jZWerpTvSHkbvg
I6AH+SL293R39TC81bdclXM2/qWe9b58IthXcl61YzmpVx66lrvqk4Z+Dfbl
IZsZzY5qJp1yjEUXBhkhsykPrKfV28nIWcgfkeNCZrMxsBnQLPRoBjiv3bO7
TIB+9G2u7ZKToAne7Frv6o1U+wnPaqmafUa+75mvaqxaP/HsLSbBsHaFa2+r
++ntk6P9qBkpRu71rRfUE5rtAd/eLoF/d+BLPLuLfNTLja7lpp1RCh93bda0
A7ULU6L9rJpV+dYzqr/eeA/6b57lqpzHtK/4l+fYDGuWNYOaxd/oX2v/8e9m
dN4VWz2qXtAOSEWX5tmu1ExVBHZn3V0zPB7YzOitNBOrfKup6qeafODsoej7
qQbalSnqUcd2xDLXeka9oxwi+O/3bfbVYwfgI57lqp0/qF52rbbacX2u7WzN
k+5YhfwfvsbMcg==
         "]]}]}, 
     {RGBColor[0.9378012141210997, 0.7145030353027494, 0.40718376377540927`], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxFlUtoV1cQxu8/555zlbpQolR8Z9NoxCRShVZUhBiNqJAqmsSiCze+Uo3Z
GUVd+agv1JXxgYIQi4/uChV8JKW+Ciar1o2PuNLUFiOCZqH2N3xTXAwz95yZ
78yZ883civXbVmwty7LsLBKQ68hx5ARyOmbZAJs30Jv5PunrXaUs60N6kctJ
+mKeZdMA+IL9KnQ/MfXYi7FnZVofgazN9D0buUXsa2LvEjvd/ZrQ5UHn/o38
ytpzpB/5B/8FrI0EuxufR4YRhLXOsUcHxb1CWvH7BV1ZZNlg1L0OsH8QacV+
y9pYMJ9hP2DtMN+P8Z9bKPcJyMRMvnXIQqTWz5uBXkPsTPIvyP8rpBG7Eb2j
JPs7ZAp479DvkRaktqSYemKHowfAaUbX+N6SMsVUIFvJ5xi6qlDtZnvtWpOw
p+XyM+whpBP7N6QHOeW6G9kfdHerwV78mko687i/t9U45sqhDuyvC8V14P80
CbuP/V1R+DvRb5LOHmL9+6D6t6CfOFc2BdXXarcFuwv/++Ccx/9l0JvWsnYO
+2fsM+gOZJ/xDr056A2MR/MLveO2qLft8XvO4bsd/W1UrpZzJ/oKkhPfWAhv
rfMj57sNn2/Y/5Ozx2K/zsVry3kL8jLK1+7UwP72MsWsJPYJ+ij7g8QMcpdH
YDQH4e9Ax6RadmH34TcJu8Fqwvpke6dcXLD3tW/j03jnWVMufrTj+3uS38P8
81va235gfQO6Guxy7L+IuxmEZ7yxfOucr3YXq6v1iPkNOBcMz86rdz4/jIqp
D+JRRdlnvHa//51cfWr9Ot7jLKbXef3O+WE86c01G4Z8b0ZUzpb7paBaF943
w0rqAbuz3b2G2LnYU7E/4tuSxNVq52azc3eq95b12IGkfluBz4RCORsv7O2N
A1ej+nu4nzcuifM/Bb2fveM9YquT9l+w3u1cvcD64aSeaca+FsUB48J7n392
13K+N7JeE9WTllsl+lDSrNydi3PGo9XoVUm528zo9Br/X+sGr/2oQr2/NKpP
rF/+ILdlUWfZmdudF8aPRUF8M97tx/8C+B257v3M5+dz7/XrzhvjxdOg+DY/
/36Qv83F22CN4ftWVF6TnRuffA6PiqrZTJ9dlUl8s9mclYRv31V+l+X4X2J9
T0k1NR4YH34MmtHW4z/g86/35ptc88jePuXqH8vhy6gZbbP6iGH6fXeyvzGo
l20OzUvKoz8ovxbnkJ1vedjb9Ph/xvqjyfOy/bYkvTrX/mj/L9l/xWpi332O
YVhno/bO+Ix85bhXkt8ZnP8A01DuCw==
         "]], PolygonBox[CompressedData["
1:eJwtlEtIVVEUhvf1nH1uUYNMIu2JI6PSa1SDICNIe0CBCvkoatAkSytrpkY2
SisrqlE+IqHQSHIWFJRJ9LBAG2WT9NoopUJtEE6072+dwc9Z//7X3nvt9Ti5
J86Vn81wzh0FITjknbvFwkmwPOlcI4vdCeeGQE/k3ELg3AZsB4rg79GnsbPx
/wL/AJ+J/Qfhj/F/gfsdEMEfhKbJ5zX8GvpdtFrwh7tnQztLZ46iV7HWDr8A
1hDPMPpftPXwMfRs9P3Y9SBEn2etJo4/H60A/0r8m0E9Whm8IWFvaoFPBPaW
r3zy4DfY8429nvUyzmvgewztCmjEzkTfnWEx/sZ/irXF7J9AT8EH0Few9hO9
C7sWuw6tGKThFd5ivQcqveVYsSrmg/Bxb7k4DWrYOw5GncWYxi5PWnxj4Cm+
OeAAdi6Y5f5HvO8zvr2gL7Kc6jzFcJX93XH+9f5e9CR8S8LeUALv9Bb7G3AR
+1Rgsah+s/AR/OcSdofueghGElaTt5HVWL6qaS97d3irnWJUrDe9xf4M7OIt
y+J86v2qXQl79sT9oFyk4/qo53ait7M2iO8U6MAuZq0QLQU8saS8naUzF9Dm
QsuNYh6Hf+K87856clVkNVFtSpRztCZvb1fPqfeqWTuOth0UYv8AixJ2p+4u
Stp7VJM2/Pu99Y64apXH/WVx/JqdKLRY1ZPV7J+P56kUHIZfQu9JWE6fwJtD
q+Vl0AY/r5nQXp2B1sn+/rg/m7BbwW3s58Dj+xGedhZzAfylt9xlsf4KeyPf
JejbwCbsitBmRTFcx386nkfl81dkPaveVQ228r6ewO5S/+egp0KLTT11BF4e
z5tyUAcv8Nbv+sdkJm1mtFc1meSsysBi0cxVYVeF9na9WW9f6W32NfOa/X2B
1W4tGIGXhpbL/3diN8X/L/HWyHzkuw7sjWdc9QpBn7ceV/8qhy2B/YOm4//V
DGd1BJZr5bwrsH+YNPm8Qx8I7F8i/T52fmC9sxRsxj7jrffVM+qdrrh+mvGs
yHKofKhmQ4HVVPepByfVq/DVzmZkGP4P3/26vw==
         "]], 
        PolygonBox[{{1833, 1832, 1355, 655, 1390}, {1391, 694, 1388, 1832, 
         1833}}]}]}, 
     {RGBColor[0.9533060916564425, 0.7532652291411064, 0.4552488841349719], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJxNVEtIllEQ/fQ+vlWQgqhFQistU6OoXaE9bNMDJC2DCFpkQvm7LR+70grU
VdmDICt7QblSIVJ7gQm6jcKUglI3ifZa2jmcCVrMP/PPPXe+mTMzd/3JTE1T
dpIkPRAH2Yyf7dD10LVZSdIOaYNkonStT5KdsLNgfwFmD+wU9jzscugAiZAz
8J+HrgH+Hc6GEbMQvtmQJLtg74XvJf4vAzMOzE34F5DIDeghnHdBuiFMbi+k
GnIcd07Ad8zpjDGJbYRUQqogbxDzF2JOIuZS1N08xCyC3mdxSqPy/IY4V/F/
FDICqYK/DH7vVTfrr4Pch0wxJuQacGN2pz6q5gj8CmJtgF3sdYdxeLbOvsk8
9iOPHujcNEkuONW4GKRZxyCkICjPfPIB6YCvBVgfdd4PuxL3h4FpxnkO/s9n
i7sOJy6Wg3w8W+10luv0v9h699mJE3L1G1Jg/JFHnyoH5jITdT6FuiaDznez
VmCGgMnAtxV2L+wjQfywZxnIuSBee4M44xlxpal4OAT/jijeyB85vWq4h/BP
wH8H3y1MNDPM7zVwH6DHoK/j/hywz6EXgS+E/RP40054ztpIUO0vgvibhcwk
yq/6v5lgn4psVtirbUHcMT65zI+6N258j9oczNns8H+CuqrINTmxPargHsE/
C39PUIzUduRx1EzdRc5bUsUhZ5ejZq7Na47qbBaPRukyr/rIyxrID689+miz
X2T1vDIO+7izlkMX4jfYvnOH3htvy8A8gd3PnYd9FnYN7I1eOXIGGJs9KLBv
51ofDyDmKae407A3wV4Fu8xpb1gL42as/+xvd9AMc5abgmaJM/UUdsS9mlSc
zhvHX512lrvbHMVJuVfcNnunpq22BqfeLNj9R8D34bzViyNyNQF7IIgT9od1
st4S6KkgXvc4zTt36jTkkr1ph4H5HtXrnKBcmfMz2Feicqnz2oNrNtOfonry
x2s/GbPRabe547fsjey23vx7Fy7am9eZCLfk1S/2LURhO4EZsHPigteM8W1u
d/K1Qt823KjTHgzau1Nt+8p8O42rFq/3mjk9AH4lKA7jtVk/2dcVYBpxrwLn
eanszUHfn7F9K4/SE05zNGnvKd8LzkMd9Nso3z2vPpfbG8r6Goz/g0Hx+Z21
qXitha8k6k3g2/AXkwLZyw==
         "]], PolygonBox[CompressedData["
1:eJwtlEtIlFEYhs/Mf/5/VkG50YqCVmoxkyRdNlmjSKsIhmbIIoIWpVCOa0t3
5VSQrsIuFFnZbdMqhW4aBSXM7Fs4UlDpplLosrPn5TuLl/97z3fe75zvcv4t
J8uF/rRz7hDw4HQE+I6CX7FznfAm7P2gCn8BviNYy/pL7CzfNfh2gbbIgrSm
nEuBjsS5btY6Wd4Aauw/Bj+BPQKW4D2RabUmXw39H7Q18B59As9hJ6AL3see
PvbmQR390RBPMdqxm8P5BdDqLQflchVcwD8P73WW4yl4AzxPPuvBT+IvspZB
u4A/B88Ro5SyO/TA92Wcm2LvNBhF2xVZbRRTseci006BJvafh1dCfsext4d8
VbMcdhuavrTdwWVMI+00+Ij/LufPcfYKmMW3HO6vGozgP4d/ImV7HuEfgj/A
roIn8Cr8N/Ym4s8nVhPVZhCcge8mngbgGtjI+fe8aaV5h/8bZ8TYWZCHb81Y
7qpBP9p1oX7KYRX/58h6rx7shRe8naU7VuAz+D9x92fgDvYr9A18l9DfxP4S
9NrTwv4Sa+P4ymAP9mzQa08z/g/wurP5yMLLsd1tAXRw1xuxxZ4Bg9gHwVjo
n3Ipcr+hlNXsMvpibLV4DXbgnyT+c2dnNOK/GNksac0ndqbOVo2+4ssSr5iy
mOXEZlZ+zVSMr8KesTAv6uVE6K/q85i9LaE/qp/e0sPIZkGaGP99b29jEjxN
rCaqzQHgue+wN98wuIL/cKi/Yp7VPIZ50dol+HhsuepNX8cuedNq5gfwD8RW
K9VsG/F3wrvTNjOaHfVQvdSMr2r+vc2qcnqL/h98c9redB3uvc2SanQksTes
t6x/RG9kPVKv1PN2znsT4qvn6v2Kt7eqmfuB/lZk/xLtua3axtZ7zYDeamNs
tdEddJe/3t6CclhG/x/9spwI
         "]], 
        PolygonBox[{{1268, 537, 1265, 1752, 1753}, {1359, 656, 1356, 1811, 
         1812}, {1753, 1752, 1224, 498, 1267}, {1812, 1811, 1327, 617, 
         1358}, {1457, 774, 1454, 1875, 1876}, {1876, 1875, 1422, 735, 
         1456}}]}]}, 
     {RGBColor[0.9688109691917854, 0.7920274229794635, 0.5033140044945348], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtk7trVFEQxs/uvefcyghqXFAbu6gbHyvGKlldCztfbIwEXwhREGLESsgG
GxUiiEYQtRJ8gVYqWPgCkYDCmj/Ax4KIsdSIUUv9Dd8Uw3z3mzlz5nwzd+WR
sT0nyiGEYSzDdsQQpiCWFCEMpBBWlyDzEKbB78C3wRVix8mpkbuUcB0bwKb4
fgrfKDwXu4OtKcSfjMq1MxXz1Ozg33DxNPYB/Brfjz+KjWDtqO+tmbh+v+sg
NRfAPSM+BP5M/SvgPrjNxGuZ3mBv2Qnf7efs/IOkviZ4ywVwCzwI7nh/26wG
eJSzm/B/iK0wTcBdxGbB38rSxfTpwf/ClsG999rjJdX9mXT2L/E6uAz3lRr7
wFVwDh9cz/VRPXa7RquSNDFtSuQ0yFlMzlyUFsfg55Lu/U2dtXx3wa/DH8IO
g/fjz2CT4An8TJT2DXBW6I19cOdcX6v9OEpP07VD/Xn6nLH38X0VvhmlgWlh
M9he6PsafBNcwD0C7/X9KXF2A/xLck7D9/rc32baEduVdklx09VqjfvuWHyM
/Ca+lzqjSbEmuMf1/EKdIc+pwt+0+6n/Cl+BfwK+B34Y1Zv1uNzfbnNu+H6e
At9P6uVWrl4WZnrnJdekTu4Wn12Wa96m/zz4X9S9dv9kUu8t+I9RuzwCf8Dn
0sJ/h78IPpuprtW/bLPNtIe2jxt9n4fxP5J2YBH8rqidsX+xnKsf62s2k7am
cUiau80/T7rrvM3dNbkLrvo/spt6N6LefB1f81kPgj/5Dtt/8CJKk+dRe9H2
+f0HyRF94w==
         "]], PolygonBox[CompressedData["
1:eJwtk71rk1EUxk9y75tMVVA0ULu4qY1II6R0aEPr0FFoUypiqwipINiAk2Li
ZAspODiIOpaqYLcODiKCtA5q7D/Qj0U0btah4Ki/h3OHh5znfc493zl9szm1
mDezERBBJZgN83sEDGHfz8w+4PAUVIpmz+G9vH+T9gqft/jug5MFs3H4KHYD
7KK34R3sG+A69iVQwy6BbfQ6SR/kzNZAh/eP0B+bxwzwLfgOdh79DPw4bybI
3Q/+wL8H186BMXgNfsK8hi6+v+A9MI42AWYy7+UOOEU/jeC1KudvtEA9ZWLV
wSz+d/n2Dt8noIz/5+C9qr4Sejm6bwsswj+ib5jXrNoPo9eqmg/Qzyd/5bgC
X03+mpHB98g3Zj6jBbSraR+a3xz2TPRcitHEv5p5L+qpn/pu4bNgHkOxWtFn
2wVv8M9Fn5VmVoN/ia4dgm/Y2zHZ4BP6vbT/n2CS+JtpH6pZtb9HPxr8Jl5g
l3kzWPCZa/Zd4n0Fm9ol+Is9kPcc+/Bnmcfuw3ee+K/TPWkfEf1huh/l+5d5
jXqrGHvou2Cn4DmUSzvQfHRT0/Bj6V5u68iJ3+bby7SvFXx/pPtRT2fha9F7
l886/ELw/4J2cBG7nu5HMxki3nK6V93QAdrlzG9FNzNY9G/StJMlfCuZ16Ka
cujXgu9W8avYrcTVs3r/D8kqc1E=
         "]], 
        PolygonBox[{{1857, 1856, 1392, 697, 1425}, {967, 221, 997, 1579, 
         1580}, {1107, 379, 1645, 1066, 1067}, {1361, 657, 1358, 1813, 
         1814}, {1814, 1813, 1328, 618, 1360}, {1300, 540, 1777, 1301, 
         1302}, {1426, 736, 1423, 1856, 1857}, {1731, 1730, 1186, 459, 
         1225}, {1602, 1601, 1028, 300, 1026}, {1067, 1066, 1064, 298, 
         1622}, {1269, 538, 1267, 1754, 1755}, {1459, 775, 1456, 1877, 
         1878}, {1878, 1877, 1424, 736, 1458}, {1302, 1301, 1333, 621, 
         1795}}]}]}, 
     {RGBColor[0.9843158467271282, 0.8307896168178207, 0.5513791248540976], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtks8rw2Ecx5/t+/1uDsrBrHBy1JZMWWjSmFzZNKk1zY+bJhe5uMjRxZAr
4R9gBwcnihb/gLaLWilJSu5enz6fw9Pn/X097+f5/Hi+A6tb+WrYOTfB8li9
UedOAIXAuZ+IczPobvQ8q4aOs5/BmMHbgD3i+Q059+I7dwG/gT8QYxHVl+iW
eV7xzHH+nnt2OVuEF+AJeBPdD/9DpzjTxdkR4he+Q/Q+egpPAn8Iz1BU68mz
v221CUtZ/YuwFc5UOLtHXIa/w4/hn4HyCvwbvY7e8LRWqfmc2PC1ZukvyWrB
n+GbYZ1JjjUI9/C04fFAWYw4DffhH57OSGY1KXejO4hFanmzvGt8h/DX0Vfo
JevRI38ZX5s7zwKdTc5yx81/jT/say7JWQ+0R+m1Yr2XiVWbc9LXN5C+pL8e
6+vJapLabrnDi+jMD/guwTuJd/Bx4hh8VH4Ue19586b1mPX0/5C3WIAVfM0r
+ROmhaXtnhJx1no8xZ+1N6qhh+0/2UEfWV+y3xfVN0hLnVGdicz+H8RgUEc=

         "]], PolygonBox[CompressedData["
1:eJwtkj0vg3EUxW+f//O0i8Ti3WSSUIlWNSSkjRI7wtJUvG1CQiJmMRqJL8A3
MImZgS8g7UZiEZEIs9/J/Q8nz305955z/+3I9uHKQWJmoyAF5WDWzXcGVImf
Kf7kzIYhtfNmzeC9LbBJ3JOZLSbe/4M7BieFH8Ab/T2wC3cOtOG+B++Ngxrc
Y2pHoFgwmwDn9C/MZ76oX2uG3Q9gmX497/PSSNCrw6/JJ7yXzD3Iizx34H4E
53bYVySfD+5F+1/h70R/0vwkXwOXzO+DIfSuon4XvCb5dLxfb1QhblBbKPgN
uiWkfps8yutjcG150NucZH6Ldi4xd0v/LuoH+mfxftVy5Dexrx295APM9IMy
e6ZAJXOv8jxIvRX8t5HHWeJc9LMK1pm/z/wW6bfgl8gb8V7tOo3+tK9UcA1p
5YgT8BvfVzPf7KtGff0HAv1i6lrS3KD/FO/XDX3kq/F9pTEJ/x/rN0FO
         "]], 
        PolygonBox[{{1427, 737, 1425, 1858, 1859}, {896, 895, 893, 97, 
         1514}, {1461, 776, 1458, 1879, 1880}, {1794, 1793, 1299, 579, 
         1329}, {928, 178, 1536, 895, 896}, {1362, 658, 1360, 1815, 1816}, {
         1837, 1836, 1361, 658, 1393}, {1484, 1483, 833, 60, 828}, {1880, 
         1879, 1426, 737, 1460}, {965, 964, 962, 178, 1556}, {995, 259, 1577, 
         964, 965}}]}]}, 
     {RGBColor[0.9998207242624711, 0.8695518106561778, 0.5994442452136605], 
      EdgeForm[None], GraphicsGroupBox[{PolygonBox[CompressedData["
1:eJwtj88KQWEQxad7+8jO4irKxlYpQrmyUDwBG6Wku1Uk2Vvb8gAW3sCfhT0r
b2DHjpfwm+ZbTOfM+eY7c6aULAbzQESKVEhFKZEXeKepOZEZj0FaJKFP0Kfg
iD4DXniPwRh9AnaoCP5E/zmb139l7/kIzVf5B6zgc8N/xWzD79KdPfQ3fAdf
OpvR2aszfQyvUjv40Fk+/VuHt32eFtj3/mv0Df0W/QB+nXHVGlQW3gwtt+bv
wgveU3Odnd2rd+f93oGzG/QWvS9HneDH0PJpzj0zf3HaJ5w=
         "]], 
        PolygonBox[CompressedData["
1:eJwtkD1OgkEURZ/fMNhSSIKlazBGJZFGZAHQokGkI9GCBGu1o1WoiSyBBlwA
Ce5A6CjFbw+emzfFybyf+97cmZPuU/MxM7MKFGAazEacc5gRn0azPoJ3qBya
taLHqh2TvyR9F/bRZzS7haOiWQPNDYzpTSCHP3hF95ZQfIA2gwG9L3bv4Jq5
Hr2H4DN52qFdbXq3cB/8bnn4pf4Na7iiXoNn4mF0D/JyRq2E9lIQd9K88irx
R/S75aGOfkP+E92DvKyCv01vLBfdg7ws0CzhIvgu3XGePMiLdmjXZ/of+dXf
3SW9POgt/2OyMSU=
         "]], 
        PolygonBox[{{1396, 659, 1840, 1398, 1399}, {1463, 777, 1460, 1881, 
         1882}, {1397, 698, 1393, 1838, 1839}, {1882, 1881, 1427, 738, 
         1462}, {1839, 1838, 1362, 659, 1396}, {827, 20, 828, 1481, 1482}, {
         1538, 1537, 932, 179, 930}}]}]}, 
     {RGBColor[1., 0.95, 0.75], EdgeForm[None], 
      GraphicsGroupBox[{
        PolygonBox[{{99, 58, 59}, {823, 17, 824}, {1517, 99, 868}, {1482, 19, 
         827}, {59, 18, 19}, {1862, 699, 1399}, {58, 17, 18}, {1538, 139, 
         899}, {897, 139, 930}, {1860, 739, 1428}, {1515, 139, 897}, {1396, 
         699, 1839}, {1399, 699, 1396}, {1498, 59, 830}, {1462, 778, 1882}, {
         1839, 699, 1397}, {1885, 739, 1430}, {1882, 778, 1463}, {826, 58, 
         864}, {18, 59, 58}, {930, 139, 1538}}], 
        PolygonBox[{{830, 59, 19, 1482}, {899, 139, 99, 1517}, {1397, 699, 
         739, 1860}, {864, 58, 99, 866}, {824, 17, 58, 826}, {1883, 779, 778, 
         1462}, {1430, 739, 699, 1862}, {1428, 739, 779, 1883}, {866, 99, 139,
          1515}, {1465, 779, 739, 1885}, {868, 99, 59, 1498}}]}]}}, {{}, 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{823, 824, 826, 864, 866, 1515, 897, 930, 1538, 899, 1517, 
         868, 1498, 830, 1482, 827}], 
        LineBox[{1463, 1882, 1462, 1883, 1428, 1860, 1397, 1839, 1396, 1399, 
         1862, 1430, 1885, 1465}]},
       "0.6`"],
      Annotation[#, 0.6, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{820, 821, 825, 862, 865, 1514, 896, 928, 931, 1556, 965, 995,
          1578, 966, 1557, 932, 1537, 898, 1516, 867, 1497, 829, 1481, 828, 
         1484, 832}], 
        LineBox[{1461, 1880, 1460, 1881, 1427, 1859, 1395, 1837, 1393, 1838, 
         1362, 1816, 1331, 1794, 1329, 1332, 1817, 1363, 1840, 1398, 1861, 
         1429, 1884, 1464}]},
       "0.56`"],
      Annotation[#, 0.56, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{817, 818, 822, 860, 863, 893, 895, 1536, 929, 962, 964, 1577,
          996, 1599, 1025, 1622, 1067, 1107, 1646, 1068, 1623, 1027, 1600, 
         1026, 1602, 999, 1580, 967, 1558, 933, 1539, 900, 1518, 869, 1499, 
         833, 1483, 831}], 
        LineBox[{1459, 1878, 1458, 1879, 1426, 1857, 1425, 1858, 1394, 1836, 
         1361, 1814, 1360, 1815, 1330, 1793, 1299, 1776, 1269, 1755, 1227, 
         1731, 1225, 1228, 1756, 1270, 1300, 1302, 1795, 1334, 1818, 1364, 
         1841, 1400, 1863, 1431, 1886, 1466}]},
       "0.52`"],
      Annotation[#, 0.52, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{814, 815, 819, 858, 861, 892, 894, 1535, 927, 960, 963, 1576,
          994, 1598, 1024, 1064, 1066, 1645, 1108, 1673, 1148, 1705, 1186, 
         1730, 1226, 1754, 1267, 1753, 1268, 1775, 1298, 1792, 1328, 1813, 
         1358, 1812, 1359, 1835, 1392, 1856, 1423, 1855, 1424, 1877, 1456, 
         1876, 1457}], 
        LineBox[{834, 1485, 835, 1501, 870, 1500, 871, 1520, 901, 1519, 902, 
         1540, 934, 1559, 968, 1581, 997, 1579, 998, 1601, 1028, 1624, 1069, 
         1647, 1109, 1674, 1149, 1706, 1187, 1732, 1229, 1757, 1271, 1777, 
         1301, 1333, 1335, 1819, 1365, 1842, 1401, 1432, 1433, 1467, 1468}]},
       "0.48`"],
      Annotation[#, 0.48, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{812, 813, 816, 857, 859, 1513, 891, 924, 926, 959, 961, 1575,
          993, 1597, 1023, 1063, 1065, 1644, 1106, 1672, 1147, 1704, 1185, 
         1729, 1224, 1752, 1265, 1751, 1266, 1774, 1297, 1791, 1327, 1811, 
         1356, 1810, 1357, 1834, 1390, 1833, 1391, 1854, 1422, 1875, 1454, 
         1874, 1455}], 
        LineBox[{836, 1486, 837, 1503, 872, 1502, 873, 1522, 903, 1521, 904, 
         1541, 935, 1561, 969, 1560, 970, 1582, 1000, 1604, 1029, 1603, 1030, 
         1625, 1070, 1648, 1110, 1675, 1150, 1707, 1188, 1733, 1230, 1758, 
         1272, 1303, 1304, 1796, 1336, 1366, 1367, 1843, 1402, 1434, 1435, 
         1469, 1470}]},
       "0.44`"],
      Annotation[#, 0.44, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{809, 810, 811, 855, 856, 889, 890, 923, 925, 1555, 958, 991, 
         992, 1596, 1022, 1621, 1062, 1104, 1105, 1671, 1146, 1703, 1184, 
         1728, 1222, 1727, 1223, 1750, 1264, 1773, 1296, 1790, 1325, 1789, 
         1326, 1809, 1355, 1832, 1388, 1831, 1389, 1853, 1420, 1852, 1421, 
         1873, 1452, 1872, 1453}], 
        LineBox[{840, 1488, 838, 1487, 839, 1505, 874, 1504, 875, 1524, 905, 
         1523, 906, 1543, 936, 1542, 937, 1562, 971, 1584, 1001, 1583, 1002, 
         1605, 1031, 1626, 1071, 1650, 1111, 1649, 1112, 1676, 1151, 1708, 
         1189, 1231, 1232, 1759, 1273, 1778, 1305, 1337, 1338, 1820, 1368, 
         1403, 1404, 1436, 1437, 1471, 1472}]},
       "0.4`"],
      Annotation[#, 0.4, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{806, 807, 808, 852, 1496, 853, 854, 887, 888, 1534, 922, 956,
          957, 989, 990, 1595, 1021, 1060, 1061, 1643, 1103, 1670, 1145, 1702,
          1183, 1726, 1221, 1749, 1262, 1748, 1263, 1772, 1295, 1788, 1323, 
         1787, 1324, 1808, 1353, 1807, 1354, 1830, 1387, 1851, 1418, 1850, 
         1419, 1871, 1451, 1870, 1449, 1869, 1450}], 
        LineBox[{843, 1490, 841, 1489, 842, 1507, 876, 1506, 877, 1526, 907, 
         1525, 908, 1545, 938, 1544, 939, 1564, 972, 1563, 973, 1585, 1003, 
         1607, 1032, 1606, 1033, 1627, 1072, 1651, 1113, 1677, 1152, 1709, 
         1190, 1734, 1233, 1760, 1274, 1306, 1307, 1797, 1339, 1369, 1370, 
         1405, 1406, 1438, 1439, 1473, 1474}]},
       "0.36`"],
      Annotation[#, 0.36, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{847, 1493, 846, 1492, 844, 1491, 845, 1510, 880, 1509, 878, 
         1508, 879, 1529, 911, 1528, 909, 1527, 910, 1547, 940, 1546, 941, 
         1566, 974, 1565, 975, 1586, 1004, 1609, 1034, 1608, 1035, 1628, 1073,
          1653, 1114, 1652, 1115, 1678, 1153, 1710, 1191, 1234, 1235, 1761, 
         1275, 1308, 1309, 1798, 1340, 1371, 1372, 1407, 1408, 1440, 1864, 
         1441, 1442, 1475, 1887, 1476, 1477}], 
        LineBox[{848, 1494, 802, 1478, 801, 1479, 803, 1480, 804, 805, 849, 
         1495, 850, 851, 884, 1512, 885, 886, 920, 921, 954, 955, 1574, 988, 
         1019, 1020, 1620, 1059, 1101, 1102, 1669, 1144, 1701, 1182, 1725, 
         1219, 1724, 1220, 1747, 1261, 1771, 1293, 1770, 1294, 1786, 1322, 
         1806, 1351, 1805, 1352, 1829, 1385, 1828, 1386, 1849, 1417, 1848, 
         1415, 1847, 1416, 1868, 1448, 1867, 1446, 1866, 1447}], 
        LineBox[{1445, 1444}]},
       "0.32`"],
      Annotation[#, 0.32, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{916, 1531, 915, 1532, 882, 1511, 881, 883, 917, 1533, 918, 
         919, 951, 1554, 952, 953, 986, 987, 1594, 1018, 1057, 1058, 1642, 
         1100, 1142, 1143, 1700, 1180, 1699, 1181, 1723, 1218, 1746, 1259, 
         1745, 1260, 1769, 1292, 1785, 1320, 1784, 1321, 1804, 1350, 1803, 
         1348, 1802, 1349, 1827, 1384, 1826, 1382, 1825, 1383, 1846, 1414, 
         1381, 1379, 1824, 1380}], 
        LineBox[{945, 914, 912, 1530, 913, 1550, 944, 1549, 942, 1548, 943, 
         1569, 978, 1568, 976, 1567, 977, 1588, 1005, 1587, 1006, 1610, 1036, 
         1630, 1074, 1629, 1075, 1654, 1116, 1680, 1154, 1679, 1155, 1192, 
         1193, 1735, 1236, 1276, 1277, 1779, 1310, 1341, 1342, 1373, 1821, 
         1374, 1375, 1409, 1844, 1410, 1411, 1443, 1865, 1413, 1845, 1412}]},
       "0.28`"],
      Annotation[#, 0.28, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{982, 1572, 981, 1571, 979, 1570, 980, 1590, 1007, 1589, 1008,
          1612, 1037, 1611, 1038, 1632, 1076, 1631, 1077, 1656, 1117, 1655, 
         1118, 1681, 1156, 1711, 1194, 1237, 1238, 1278, 1279, 1311, 1312, 
         1343, 1344, 1376, 1822, 1377, 1823, 1378}], 
        LineBox[{983, 1573, 947, 1551, 946, 1552, 948, 1553, 949, 950, 984, 
         985, 1016, 1017, 1055, 1056, 1098, 1099, 1668, 1141, 1698, 1179, 
         1722, 1216, 1721, 1217, 1744, 1257, 1743, 1258, 1768, 1290, 1767, 
         1291, 1783, 1318, 1782, 1319, 1801, 1347, 1800, 1346, 1799, 1345, 
         1316, 1317}]},
       "0.24`"],
      Annotation[#, 0.24, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{1043, 279, 1615, 1039, 1613, 1040, 1635, 1078, 1633, 1079, 
         1659, 1119, 1657, 1120, 1683, 435, 1196, 1239, 1241, 1280, 1282, 
         1313, 599, 1781, 1315}], 
        LineBox[{1047, 1617, 1010, 1591, 1009, 1593, 243, 1013, 1015, 1052, 
         1054, 1095, 1097, 407, 1697, 1178, 1720, 1214, 1719, 1215, 1742, 
         1255, 1741, 1256, 1766, 1289, 1765, 563, 1287, 1249, 1250}]},
       "0.2`"],
      Annotation[#, 0.2, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{1084, 1044, 1041, 1614, 1042, 1637, 1080, 1634, 1081, 1661, 
         1121, 1658, 1122, 1685, 1157, 1682, 1158, 1195, 1198, 1240, 1243, 
         1281, 1284, 1314, 1780, 1286, 1763, 1285}], 
        LineBox[{1088, 1639, 1046, 1616, 1045, 1619, 1012, 1592, 1011, 1014, 
         1050, 1053, 1093, 1096, 1138, 1140, 1696, 1176, 1695, 1177, 1718, 
         1212, 1717, 1213, 1740, 1253, 1739, 1254, 1764, 1288, 1252, 1248, 
         1206, 1207}]},
       "0.16`"],
      Annotation[#, 0.16, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{1127, 1085, 1082, 1636, 1083, 1663, 1123, 1660, 1124, 1687, 
         1159, 1684, 1160, 1197, 1200, 1242, 1245, 1283, 1762, 1247, 1737, 
         1246}], LineBox[{1131, 1665, 1087, 1638, 1086, 1641, 1049, 1618, 
         1048, 1051, 1091, 1094, 1136, 1139, 1694, 1174, 1693, 1175, 1716, 
         1210, 1715, 1211, 1738, 1251, 1209, 1205, 1168, 1169}]},
       "0.12`"],
      Annotation[#, 0.12, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{1165, 1128, 1125, 1662, 1126, 1689, 1161, 1686, 1162, 1199, 
         1202, 1244, 1736, 1204, 1713, 1203}], 
        LineBox[{1640, 1090, 1667, 1129, 1664, 1130, 1167, 1171, 1208, 1714, 
         1173, 1691, 1172, 1692, 1137, 1134, 1092, 1089, 1640}]},
       "0.08`"],
      Annotation[#, 0.08, "Tooltip"]& ], 
     TagBox[
      TooltipBox[
       {GrayLevel[0], Opacity[0.5], CapForm["Butt"], 
        LineBox[{1666, 1133, 1170, 1690, 1135, 1132, 1666}], 
        LineBox[{1688, 1164, 1201, 1712, 1166, 1163, 1688}]},
       "0.04`"],
      Annotation[#, 0.04, "Tooltip"]& ], {}, {}}}],
  DisplayFunction->Identity,
  Frame->True,
  FrameLabel->{None, None},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{"DefaultBoundaryStyle" -> Automatic},
  PlotRange->{{0., 2.}, {0., 1.}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]},
  Ticks->{Automatic, Automatic}]], "Print",
 CellChangeTimes->{{3.699383348633525*^9, 3.699383389696075*^9}, {
   3.699383428619399*^9, 3.699383439039081*^9}, {3.699383511180781*^9, 
   3.6993835657889147`*^9}, 3.6993835971422997`*^9, {3.699383704226028*^9, 
   3.699383725911892*^9}, 3.699819710863113*^9, 3.720187313232243*^9, 
   3.72954084287636*^9},ImageCache->GraphicsData["CompressedBitmap", "\<\
eJzMvXdcVdm2NchV1MpVWlU3vPvCfe97/V53f7dyUEvLnLMgZqIRkJxzzjln
FEkSBBMCBlDEjKIYEHOOqJizo+dcO5x90KpX1X3/aH8/jufss+NaY405Zljr
TLZ0XbrIztLVeoHl38Y6WzoutV7g8rcxDs60qecfDAz+sNTAwKDxbwb8HvRW
fhH/rPlF/vA/vP9U/NcTr169hPzPoJe8Da9f4bW8bay87eXLN/fjY5X9pirb
NPtNfcuxb9vWQ7qlvuK/D/H6+VNsXlOIgMBARMUn4/ilO2K/TatysXpbm3jf
tnMTIkICEBQWhW0t7Qaj5WNfPHmANSXZ4ti41Gxc6HxMX7xAVV4qNrWeB54/
xsaKAgQHBSMkLBrNh0/Jj/g+9tSvR+22A+L8r5/dR0F6Evaf6eQP2FRZgJDg
QITFpKLjcqfBGHHMBwDtV16yAscv3xPHPe26huyUZJy4/hhP719HQUYSgoOD
6TnScPLKnVHisI9wfFcNIsIjEB0dhYiICISEhKDp6FmcO9KEjPwKumE61cNO
lGQlIjAoEImZRbj14KleQ32Ejr11cHDxR8vRDqxZkQzfyEw8fPEah3Y0YM/x
S3hw7RS8HJ2wsfkg9m5ZDUdXP5y79VC5iX31pXD2icSR9g4UpoQjPH0Vnr8C
9mypx9ELnTi8rQLWTl5oOX4SuzdXwMnREydvPFAefX1BGnLL6qXmevEEDbUb
cfbWI5zdsxFOLgE4eOwE1uQnwjsyC4+ev1Z6CI87ERPsiz0nO8WhLx51ob6m
FjfuP0JldiQ8IqiFT51C9XI6NCgR956/Nhgpjv0Yl441Y5HZPCTkFOPgwYPY
tWsXLt2+h5vnj2JTw04Bxq0l6fAOSUX7qRNID/NBclEtbzf4g7btemMtXSqr
coe4hWc3T8PXJwAnrz3qIX99eHs1AqNyIYH0KZKDfbD54CnpRvrQy0sUxPmj
bOsRsUPX2RZ4+IXjxoOXBsoprpw7jmOnzssgf4nUAHeU7+gYIw+AjUWZWFGx
WRly4pCy7HjkVW0XBzzvOg8/N0+c6Xw8Wtnp8W3Ehwdi36nb8iHSnbS3teDy
rbtSc969CF8XV7RevC83myG9vML+hirYObhj8+5jhsqhL+8jNsgfW1vPiUPP
t22Df1AM7r96o7XKk4NR3iQNPTy9jpCgMLSd7VQede/mUkSmVisDGmWJQVjd
3Ka2Fg2g3AgvbGq7KD3a3dPw9ovAxc7Hamt1Xr+AnXsP4jHh9z7h1sdxGbYc
vvi21uqh7YTMWKzedEic9tWTO4gP8MDBS10GapPxNrnJDOVLnT1xlIbMKYGW
M/s3E67dqZWfjpZb6+ge2mRtjaqGPdRuUK/27BbCg8NwoOOGuNrtM4cQRsP2
2pO3tlbZ9sNSWzy5huDAELSd0bXWllJEpK5WW6s0LpBa60j31qo/fEFC5x0a
xb7heq11ct9mLLK0QEBwCNydHBGVvhL3nr0cJT/Ckwd3cffhE9SX5nKrabHC
DVZR3yo32G3E+rrj4OUuA+XQ2zdvEad1IikyiBtNabPNNKoWLFqC0NAQOCyz
x8q1DTw2lO45tX8L7Ja54vD5Tl33PO9EeFAoWk5IDdZ5uhWhQSG4/maD1RbE
IqGgTuz24PIxeHn64fTNx8rTHiO29AlJwhP+/Po+Yvy8sPXQWd1gfIXS5CAs
X7dPnOHG8Z1w9wnHzYcv5JvphYdXjsLTOxD7D7fjzNmLTLAGSodfbduOIH8/
+Pt4oXBNo661XqM6PwnJxbXitI9vtMPLxRtn7zyRR1Yvsc8GMjeBoWHw9PJG
y+k7MswMcWrPOngFJ+Pk6dM4f/nGaPlhLrbvQe6KSkHx1EaIDvFAxfZWtdVe
P0ZiiB/W7moXO5zctxG+gQl48PKNVrtwpBmODm7YuHU7smICEJVWimevobTa
484LCHZ3xIqqWqwpzoKzbyxu3HsxSv76aHMNHJx9sXV7E+IDPZBOj6nCvTeB
sBOh7u7ILCjF2jVVqFqzCfcfv1Bg8ur5E7Q218JusRWWVzUoMLl0ZDucHV1Q
27gdmbFBCE2v5FtSjnp47xZW5ybBcqEdWs7cUY7qvNQGVztXlKxeg+rVlajb
spdvZaT89e1LR+Fqa4tiuoXGejJj9s5oOXmtp9wHe2oK4eoVjG07tiPY2w0r
1u/TVxS9BPDb9mxFYlwc8osqydY8UbvpyslDyEpNRHLWcrRfuDVaOeTVc+xt
rEF8bCyKKmvQ9fSFStN8uj1bNqCouASlJcVYVU7fP36hDIYT+xrRsPuw/Em5
g1do27UFifFxyC2uxs37Tw2U/fH6KWqrq0gf3Fc56sXje6irLkNxSSmKiwqx
vnYXj7eR6h28xrnjB0lqJCEhMRW7Wk/w18rVuHcaN1QgNjYO5TXb8Pj5K6VJ
BKh+h07sId/k65cv8PLVa/VxpBt/jecvXsoNKW178OCBMM2tra30iV574NCh
Qx+r7z761W3dvqUz0Ft+Jz59+Bu3ffQP3u//79cQ7fwHHDhw4A84ffq0TBN0
OMOkbWcdQgP94BcYhrpmySid2L8JaXnlQt3cOHMYsZGh8CcGLKjahKcvXveU
+5KvUVtbi46ODvokvW7YsAH19fXi00fytjVr1mDTpk3i04fyturqamzZskVv
v9WrV6OhoUHvfCdPnuR34lMfeVtFRQW2bdumd77Kykrs2LFD73xlZWVobm7W
26+0tFTIQu22kpIS7NmzR+/Y4uJi7Nu3T2+/FStWoKWlRW+/vLw8bli9/XJz
c0Xbd9/G7aU9lrcdPnxYb7/ly5fjyJEjevvl5OTg6NGjevvxtmPHjr2x7fjx
43rHZmdno729XW8/vucTJ07o2vkPvMsfuB102HhwrQM+TuQc7DyMQzs3wpGU
/qnr93Hr/DE0Nu8nSX8ficHeyFxVhzPtrfBzpX33n1Kwwc915swZDcnRtrYj
ZO3OiU8fydvOX7qKG51Cm7L4F9vOXbiMW7e79PY7c/4Sbt+9r3e+l6+g/BP2
mLedPnsBd7oe6J1v3/4DuHajU+98J0+dxb0Hj/X2O9FxGg8ePdXb1n7iJB49
fqp37PHjJ/D4sf5+tXWbcf/hY739Wg8fwbMXr/X2O3joCF5I9623TX4W9Vje
Jj+eut/a9TXqNmW/A4fa1Db4UN125Fe3qce2vnns+ppadZvg7D+gpqZGgQXJ
gsZC+EdkkzDhfy+QEe6Hmj3HFVnQdeEI/HwCcPGu5Bk1lqUhdsXGnvL52tra
BBvwiCEmUozH6eOtOL2rEk8v7aOb49eeeH6lBc8u7xefPnzLtrft10O89sLj
C3tQnBmJjatJ9p7fj1c3Wunenl09SPu/uH4Iz6+1ik/viddeeH71IK4ea8RK
8q82V+eh8+R2cmCf3zhM37242YamjcVkgQvx+s4JvLrTzva38xjb0dvHcHxf
LSqKsvCssx2vu3hAvbrbwSb4HnkQ907iZOsWrCad/aLrFInXM4YkUM9wyz44
g03ryE3ds4l8gIvUEo/OM4ifXiIlScL+8XlcOt2CyvJCvHxKAvllJ90tXVaY
1Tv8WflEr4Zin43rVuHO5cN8vHS698WpHt5sx5ryPLzsOo2W7RU0sA/zPYg7
uHhiF7asL8bLOyfo9HzjdML7p/Gqq0N86iNeDfnB0FhbRqqmgh6aepufn4z+
XZIVncfFpw/Fax/tNrrR47hMlyjJTcSDi/vpEtym1EKdR/HiRpv49J6mnZ9d
P4yayhXIjPLA0yst5NFLffbyxmF+Jz69K/cZ9SmutO9EXmokDpLH9PLaAdpf
wsSLqwf4nR5OtNs+EK/vUrvvQXPNStIpUpjq1KlTAposUPh/MhYGOui31GYg
LEnnl1WlhmJV4wEF+rdOHSD3Jww3n0pfH9hejvCUSi302TAy8V6+fFmB/nO6
rWu5I3A1f5zB9+K1D64tH4djGROQafcDYhZ8jb2J43C7YhY6y2eSZL1ZZsJX
KzPBzYo5qI+ehMjF/dGaY4IHGxe+jzvrregUXRsW4ELVQiQ6DMO66Kl41OiI
Bw0OuL/Vnu7lgfRKnzs3OyAvYDLKIo1o8Ozx6Ykne3zo9E/2+uDeTh/khcxA
TYYlXh+LxosjUdSMbZHU+K+PRuP0Fn8k+Bjh6p5w4HwGI+dMGgP/TCqens3H
8gx3NKyPIQw2Eqa2EOnc28LS8MFWguAGJASb4cHhOOBsOotIOpTgei4DZxoD
kRVohCetdNrjMXxVvijQHoOmlUtRGj4dT/f6Gnwq7pRws98X7VW2iLIbjjOV
C/Gw0YG+4yfsw09LT+6A6zU2SLAfilpqrK4NVrhLbfS5aKneymeU+o5Aos2P
OLfSmBv6z6Kd38HNVSb8EZ1lM7E5fCRCTb/Aao/+uJAzlvvpK9Fl7+JKzkjc
rfdUsXHp0iXFxIr/SYpokVSXgdCkCnXvyqQQLZJuEpJCQsJw44n0dUtDKcKT
K7Smlc8LjSl8euUA3wmurZxKiOfXnrheOBU3CqeJT/8pXvvgRhFtKzFGY/R4
RCz4HkWew3C5dA7uVpsa9MOtyvncItQau9JmIGzRT9iRSrCqt0ZX7dL3cHfj
Uvr6Qb0tLq2jBnUahZqEmXjc5I6H21174cE2V+qPR01uuN3ginSfyahNM8OL
A8EccdgXyDx7IBidu4KR5G2ElkpnEpqJ9N2Lo3GMi44k7Kp0R1bYPDw9kUZY
yOanO5vF313IQeepVUiMssO5I6UEqSY2Yvcb+btH23Hh2CrkRlni5al0Oi6L
8UTHEclezKVzelD7WuLVsfie4lK0+UQCqlKssDmLNh8M6Sff3bP9gbjb7ItE
t/FozjXD0x3uBp+Jp+pND+iGew0uyPGegLKgibhXS22ycenncpt00Ud6h1L/
8Ui2G4yLxbNxp8r0L6JF+3Dr4lzRLOQ5D0H0wu/QkjAWt4qncw8Z/LfoGUNc
X0k9UzQNt+iP/u8ld+LV5RNwu95bdHMvyR6TxNZB6eTeangGxEEoIzxEXIA3
6ltOKlB6dL0Dvp7eOHHtkdhhXU4Mkko2a6HEivGXoPSeeH1fwOhWsfR3k27w
4vLJOJk1kb8Vn5Xt/ED/IUOtkx7wzAoj5Lv8jJjFP6I1czo1hBk6qUn6KlBb
Y4bDebMRsWQQNsZMxf06hpq1CrVNNri41hpxjqNQlzQbj3foQe3xDjdc3eyG
ONfx2Fu0FC8O6qD24mAITtd5IdZ9Ki40+BN9JMhQM8SrjmSUpy7G2iwbhpah
grLLy9G6PRXp8U54dLOeWnMbyyJCGe3xZAcZz0yUpSxmZPbUAGxdtg0aVtgw
mBWAvTgShxXhs3BwlQ3jXwKYIcE/COUxs1EcMo1Q6/q5ZsRURExHltso3N6w
hMeaQXdglfiPQ4r9z7hYMoeB9WcZWNSWok33p05DhNUPKHQdhAv5k7kvDP4P
0Q8fS32n6aPLK6bgSOo47IoZJf6/UjClex8qqBORBELc+vXrdYh72nUFkd7O
yCquRlVRJpx8YnCt67kaGHr5FIXJEQiKz8HmmtVwsXdBy5mbvxVxEu4/E8NB
e1dXC6biWNp47IgagSrvQciw/g5RFl8i2uIrxFp9hRWO/VEXNAT7E8bgTM5E
8dTK8fT+3xT+o4e7WTIDm6PGI9Tye6wPGY2b1IjUpioo764xx8liU8TZ/ozq
0EnSWFdA2UuA8ny1NWIdRmJzyhwGJUNVdOXTZg+cqLZDtNNYnFznpIfIl62h
2FXqgBRfI9zbH4FXx+MNesrkR+/x4FAc0oLn4uB6H8aVjvzOZABXS7C6KBhr
SkOY8DSwbEJ9dTTqCxz4GAWWr89moTDOHIfXuDHwe8hXeXksHo9ao5HsNQUX
69zwrCWIHloZL8E4UGqDeKcxuFHnQBdxk8mvF2HfHY2Z8xFnPxxXqhbgXp31
GwAtlgF6SR+gt1ebCpBuCBmDUPNv0BgxUnQLdYlMFJ8KwGnBdzJzggDnGp/B
yLL5nrpZ6uKEhV8jhv6PtvwSefY/oCZgCPbFjxbdfX3FBHRt9nkLYAVp4taF
dizPSkN6bgFOXLwlh6Oow+7fRHVpAZJS0rHzUAe7Vwa/Fao9xetfVJhdoYHV
kjAaG/x/Rh5JN77vGEu670XfIN9hAGoDh2NX7HgcTJqEndGjUen1MzLo+eJJ
4vG+iYu+RrHrAGwJHYbj6eNV0uW2+ldxrd7oLDXGsezpSLIZiHT7QTi1Yia6
qs1U6BIXnC83RyKJnbKg8aJnukH3bNVSRNuNQFO2KZ40a6C70xN7CpcgzmU8
rjV44nmLhkwPRaA6dQEKo+czlAhIcSp0cSIJ57aHIsF3Jq7vi2FC1UH3XBYe
nitBKjFq2+48Ybc/kKH7+sE2lOV6oFWCew/1EPItO9KRFjgLV7YHilGi4JdM
OG7sDEGiB8k4MtRP9wcafKIY75Yg3NrmjTjnsThYuIAxq4ff42WLEWEzFMcL
SXYq+O3BjUP3sipwvADvZS14e/Nbgedc558Rt+h7tBFbUof8u+iLPwms3pQt
9tncSWiKHCH6L476k2kpfsE3yLHrTzgdhp0x43AoZQo6qO/aUidjT9xYVPsM
Qaat1P+8f5zl/8baWAs8f/lSMfU6HPf9RVD+hxaUKhiZLw8kjkGF508ChAyw
DJK3a3yHoilqHFqTJ+NUjhFO586g/41xkv+ypf/589k8YxzPnEb7TUBj+Cis
ch+EtKXfItL8S6TS/3VBQ9FBA/WGjmP/KtrlHXolg0LG/0qRMSr8RiKMeLYp
fqJoztur53+itC5h9XKlBVKchqHIbyxu1yxhqtVh1RbtqxYictlwHFhpKdSY
itXdPthI2jLDdwp5KX6k3IKoMSQgECjwcF8IckJmYVPeUo3hlzTmznI3ZIeb
4llHutCYEvCIN85mCgVwqiUPSaQz71xaRwpg+3uKzCS4Pr+zBVkx1ri6m9yR
8zkGWszebo1Hsu8MPDwYxTyrYvZ1ewLa1rphebARHu8NwNN9CmZJDxBmSyNn
ojyCvmty+0yjB65ttCcDQ6M0ZQYJbhsN3S5l+sXq0IlItB2MC5LQ/LNm/B+l
3mO9tcL5J9ICwrb/rRtTXSL5tjNmJLHSj4igDo1d8C2K3QZhS9hotCROQkfW
dD1gdBAwTmQZif/fBpCGkKGoJ2X04uWrX0cuO9g65H4kkHt9+XiB3MPJNCTI
ssdZfU0o+4oofgAawsfgSPq07jfzDt8NMUkH3ZMC2jO5fE9GOJI2CbVBw5BG
AiHC7Euk0/+bQ4axXO3BDfBXWQnw6O0k524PsXHkgh9UT4iVgBaiV1dbIMNl
OJZ7jUbnhsVaiD4kiLautECE7XAcK1skOEfSpoYsU/Gw2QtF4Sbcxz0YlwpE
iVpxY0cgErymSSZadYMM8fJEMsqSF2J9ji1TqZ4IuFaKTWtjUZTjg1cESi00
uy7XICWMaO94igbVkufUvjkQBZFz8awtFi/pIj1UNygRm/OtUZ0wj2/pE60S
IOEa7zwGN2Ul8KkMTX6q+43kBvlMQEXIRNbretC8X2eD9VFTEGv9E84WzuLG
/FM3qdqcOBkhFt+hJnCYUGM66/ahqgQYDq1JY1DuMVC1/qVug8l6ThA4VOGQ
bUytwFgw1GEhcwoub/Dv3d1XUmDIcRhONJw8eVKG4Xt4cfUgdkX+LJiYx0P2
sv5khUejLW2qeq2ObKO+4lLvi2FwOpchZ4TDKRPJ4A8VnBhh9oUgdNaoV5mL
i6cZ/JMMNybFWyXTcW6FMbmBPyN2SX8co3NQk6hwY7F0vdIMue4jkUN/t9a9
Abed2fMQRYx4bo2NEJ6q100dc7fRg7zuKajPsNDzul+SK3R8vTviPEjY7wjC
6+MJesLzfmssUoPm4HCNn77wJOv97GIxclPdsXNzqrDe76pedxPOHC7G8hgr
vDqdoe91X8rFtlIXrElbwNdSnCJmwrJ4U2zPX8C39LHGcHdu9ya4jSPDbfWG
4V4Xa4J0lxHoXL9Y4xn1YlZEfdx0RC0ZKFQQj1sVa9SQbG1qQsci1Owbws1o
gStyKgz+RXZpJaxNx0VylzYRfyUu/EbwBdPOtsixaM+UOJAx9ono+J5oT5+M
C+t89RQhR2L27t3LyRODt0CN815XrlwxUKoGnl05iN2RQ4miRuBw6hSClgov
un2+zHsCX0xnpwlfh4he1/gOIdH7FZJISO6MHqn4Rv8ka0LFp9kYPlb4NPvo
tAQrLapurjZHvscoQtYo0ZCEqnflYXt34xI83GKHTalzREhHiP8mN52P7Y7L
9a6IdRknWOFlN4+GPd/s4Jl4cjhG2L2eGlN7uiEYif7EAocSyF7qh3OuHC1E
YtQy3Dq3hs0syQado723MR2VaUvYxuoc7RxUpS/B9kI7OnWi6tGQGGVayw2e
gSOVdgL2H6vedrDwtsvCp2utK2PqwEorRNkOxblySw2F9RKRru1kcSMWDUB7
3gzifwVT1IyEpxvl81DkNRzRC74TrjNh6l+6RUjOkRBkB4B90zgyqzWBI0nw
TRV8IawX9XC/X4ETYUXkPMvLy7vD6c24H5mZiwdxIn2SDCHjfxFnfldiKHqA
jqyp2BAwVNxNypJvsSd2lCJbSQ7IhpCYiQ1hc/wEQcxboyeIwdO5ej61pQ5D
NwhDOR4kGjwJQyJMoWKoi9yM+5vtRBQj23s8uhpcOGLWS+MVH6mwFV7x+VpX
0Utaufb0QBgZqHnYmLWYWUK1hTiZIjzb4kQikrPZhgoWLq/A1vVxwg6SF6FF
Tt3qcGxZ6cAUpPrCZ7JQEE2yaL1Hd1+YtVqi52Rc3eIhfOGPFAtIyDlW5YBY
x9G4VmuvZwFZnF2psUf0smHYmzVbI84kSmrJnYOwhf1xKNNIBFX/pPElWKyx
05Zm25/G9yTuhX+Ww2yK5TudPVGEN9iUJC2mzggbIywe0wR16udvQc4bUcFf
dhV6Cry0p00Up/l38fqOgGZ75lSsJ9hGmH+B9KXfYT/5sSzt6Rb/orieZL1a
06aIONOawFGSa7/a9ONuxivbbQSWe4+WI1k6iJA6uFNni1xSDqvCOP7lpg3l
MUS25S1Asgf5CU0+oju0culmcxDiSS4d4148obNf3I1PjiaQop+PfdVezBP6
9utCEbKTXXCgKUvYrz6K93mfvM8cNxx60/tkxyA1YCaukanU8z7JVF5pCkIy
Ieb+Lj/hfX6oQfGD3f5I9SLXPsdMONaqZtrmyu/Ek68OnYT79dZ6iGkvMiXE
DMBOEkZdGsRw0PhUwUyhFFY6DxRxOtXLeldFDBPOanLywkyp56y/x3ayXWRM
mBEM/vzLcPktnqU+XP5D5RYjIf47MieTCzsY4WQ0WfW0Jo1VQzt/ljHDBraD
RHkcPUOhx1Bcr5jHRKrFzDXCDOtrdgEZJCpmepGVWorrG2yQ5DJGhH+fqC6g
IR7v9BbsXhg2g1o/SKt12ta4INFrOu7sDdcLstHox4WmMCT6zcKtA3HChVOx
Qvbl7MF8JEXb496VGsLKDt6q6usXd7YiO9YGl3d1c/3OZ6PzALl+ft1dP0Nx
vdZqVxSEGLPr95Fsl8iCYlOmJXJ9CQvbXLV2iR8y1Wk4bsla5zO5IVhWs62K
WjoIm6ImCNH4J43Ld5x0CvswlV6DhSEibPxV6QE54sv+OsublCWkT6PH92B8
/NM/Eh7/qTE9DI/2jMl0P4MQavp3gm9/wWyKBvuTfHMkkHGx0AQZRIr8J0e3
P9aY3KtVlkh2GCoc3nsiO7BUgcd9Gjzn5GjWnnxzESHQ6uGubR5I854ieEWn
h8m4HArH2rSFKIk15ZCSam1IsTQUO6Ew3pIDqDrPK1N4XhtXR2F5ugeunq7i
3n+0vacGHvevbkRy6AI8Ppb8pvu1KUhYt7e5XxygWJM4j2/vQ40evrTJDTGO
Y3C6ypqli0oljJFjZYsRaT0EJ0vMhXZRMMKNc23NQiTaD0FV0BiNHpYw0pox
HWGW3wk5IEdahYDswTxP+7A0YP8nxuobYXVO0jAnwfLP/0iM/JdscRgj5/KN
SUNNRL79D4K76oOHSk5T0TQFHmyArhYaY6X7UMEgJ0nm06NI8OglvPML5eaI
tf0ZdbHThCnWRjrbiqwQSa75ycqlGtecJC3ZmjPrHUmOjMOZGuduciQI9/eG
IM1/BvaWOwnXXGtrnpKtyYkgW7PGS+MrfSTZmvOFqF8Tg5RYB1QUBuJiexnn
kISvRFARmcrzcqbyhZSp1CYqm1a5Yk3qAqKst7hMC5nZPpQBzIlKjidtiJ/J
D6ZKEzKpnZudkEgqfpscN5LQYSjC9HdIpeWRE1DgMawHq7s/yu3I6NiTPAUh
5qw3RjBlGChREh6xJ7MmiHg791KVz3AcJ8+Ircu//l5o6Ed/9KHxf+pBwwjN
0aOE25+0+BscTBorUUfhtD/K0Lgu+zwsRMKtvsfhLCMtNPiRThbNR+SSQdiR
OkMPGg8322Jb5jzh7NysF86OCg0OgjdRY6d4coTZT0+GcEbxVK0X4j2n4sbO
YO4nPdNybnsIksi03GnVejsfCGgQjHHvTCkaSLamxjmiNN8PZ9tKiDsIFs93
obU5G6tSFnVPLVZnLMW2lcve8HiOxCJP4/F8oIncHCyzRaLrWNwmIJC+kqnD
UORKKyONSJqNYTCoAoRYoypsElKWDcK1srkcnzVQcNElR2uCyYPm7rilhjPe
EUOTugNbQoYhZP4XyLUfKOLbrCXJxfm3X4aGrppM6yZz+aZUjvgmNP5v8dpH
cqGyp4m8UMj8v6PEdYCIu/N9qbhYyVG9GdgSLTkz+9Om6uFijRna8kmhL/4J
LTlzNLgwFA5MaehUrAyczM6LKjSavVAcboKK2NkcsFXQ8GRfAF6SIWEGXx4x
B8+Pxr7ht5QmL8Trc7Lf8r4UYWYwXCnAw9Mr0LwxFukJzliZ7YWzR0tFIq92
uT07LwbaTN7KGLrpte6/5L1c2ewucPq+RouyPk1yn4D9KywF9WkNSGvhAuG9
XKy04jiAakBYi7LzG7l4IM6slAIqWorYkSBBYXesBIW/yFDgJufRySmrDJvv
SA9+JQyIHKD7919xdrkyZvfu3b8UO+H64GvXrr0VFH+XQcFS41DKBGQv+0E4
TTuiRqqVCJ/LoLgme7g74iYIktuXMkULCi5C2JNhgnACxYkiMwEKrQS9tdEW
Ka5jsCVtrp4dYb69tdUNiW4Tsa/YmglCZ0f2BeLR/hARGmkutu/ms8Th8ZEE
ZITMw5Faf2FHeon+flfGCP1dyseTcyuxd3MiIoMWI8LXDPtlm9ND4988P5mO
9MBZuLwt8A1n5dqOYCR5kLOyU3JWJID0FAJ0c6YF8nwnGWhzDjfqHBFrPxy7
M2ZqbIcEjCMF84WTcoi4n6MiOmCYSsAw/UYoCB0w+ii1R6ghvzJ43hcocBqE
I2nThO34X7+CCupyLg7lMu//OQSij4ovZFQIK0J+B5szNluclmMVzFLnj3Kd
AfuzzXHjBU20SDRBhlwSTLcq56FrnSW2JBghxmYwLq0WWW+d8rTFyYrFIjfV
UbFEDxVP5GBHrMt4XG/w1DchB4JxdpM3Ejyn4dauEH3vhOTnsTp/pAfPleuf
sthblXJTF7JxdX8iMhOdUZrliuZy1+5e673DSUjyNcb9lsg380/r3IVeeCTn
n97TSM3rDV6IdRojRJJOahrye+GnF/mNY5bQqsyLlQuEJ9IYI3kiWpLYHjdJ
kMTeuNF6WGA3hD2ATCKIKMuvyQUZp4Y9//MfGdT4Srz2FsbieMYkEZdnDcG5
SroJfUowJuxOEpRwMGOanrRca4mKoAnCMe1cLwIZKiVwGH5bxjySWaPRuclR
ox/IwdjlhbVJ81AQaqzzSg2FoajJXITSODPuczXX056EksQF2Fbiwt2p9jaN
61WZTti1NQNNG2KwMZ8UwKU8rU9xZlsY8sLm4GlbjEjGqz5FRyIaVthiddwc
rjZ6XxMKZftVEWmsFYzCGhSRNbAdisuMcHrKT+WnFPGamiXIch2JMt+Rv2gN
uACDO/rPajRRsgZMBjzyOJx+NF0SjP/1j/QlvpY7mlF0OmcaSt0GIpSUyL44
ZqHpeh1dZkLAnChG+iGpCk0Z6CSDcavKHDluI0UJi1zX966iCTbZoZD0AOuC
x7qgOKfB7pJ3meo1Gc0rFomguDageW9PMFJoMAqrrfEgeDBe2RUhy8R4IRMl
5u8jxviFndFIjbHHs7tbsLcxA6vTrUU9hdZN2FvlhYpECx7kPTWeZFUysVW2
FV61hijDm8vNzm50IU9yNK7U2IkwZj/NE7BEZBHcnG6Ch934fm3EZKTYDcJ1
WRN+rhFQe8h8Bpt+TWN8lF7XM99z2UEVef3M9+v9RyoJW4P//kd2/Tcyy5/I
MsZ58hW2hg5D8Ny/i7g7J0PY9n/WzfY3EFmFUv+3SY7CR/LzcATqSqUF4u2G
oDZm6hu2/9p6a8RzG8lBPZXld3igY409YpzH4TIJMK6W6amJQR2udhGFXkpM
SNtVtfl21LNLuGd7yUP+FZn9UhryTfUp5A/sxJE9+ShOXKT6ApfyiAXsUJ+7
hM8g8UdPFITPEjkZ9gHeU3xD4nRmn42Js3RD3ZDvHeviTJDrOZrxre3q/dmz
Eb5ogPCueZR/rhnlB4gWQ8y/FRnNzmJdXI8V1pmcSSK5Hm1Jpp9G1+ncGf/X
P7KXP5AHuBzMVPP/khsg68reQgC2JIwTKQ6uZ7golT927/8tXPBn8T2O5Bhr
+58fsoMcRdZ+B/Pm6PW/0D3FVsLKn1tjy4XOOkeRiL4uwxy5AdPwmLTeU6Gs
5C44GIqSWDPUZC1mza4NH9xrjUVK4GycawrTFEp9RESQhRsHkxEXbo1H12tF
Xd+5IyXIj+ai5gzJbSBXgMONq5IWYG+5IwOhh1ZRHopBmu80MeLJT+HZDWrN
dfsaR5HO59zeQw0LcOk2G/5Im6E4vcpCFPZ92i3mGEHuwC7yAtnSa6HRRt3C
A4or+m4Va0O+00ShFBcK5RD5H8uYzt32N6U3s6TeVPJh8v+G/0jkvCvnUTXn
Jx9hKupDxmCly2Bk2A5Elc8I7I4T+4tYKGdy6Yk418IH/1WmmHa+XT5F1mRk
2X4varuOpYnMXzeKmYFNkeMRZvW9ktZXIbbGHLvSya8iyXSBfa46jZbYYieC
vmnuY9G11Vl0jqEmb3B/uwey/KZiKzG7Nkz1rCUY15sCEU9C8vxWf5E166mJ
Y3IqJC+SxMbpTBViRDFPz69EfroHWptzgKfNuHW2mryQ+XjWIertBczoEORH
zEdHrRefVutx3t0XgXj3SbjT5M3qRoaYoTB5OTQQGkgVaTjnXqML0unB5Nic
VllwrVe683BUBYzCXQ3n3Kkyw4n8GQi3/J56a7gIRWlzCc3k3bFpKfMYKoo9
yLRIUaiPxUfubAbX0fRp2Bo+DqXuQ8klHYBVHsPQFD2eBOF0Hfhofxltb1aN
/taoVW/x+qksQoxxItMIO2MnoNJ7OBIX/QD/OV+RSvoe8UuHItdpOGIXfI/A
eV8hzIxLQgejLngMWgmZys2zqfxn+cxSUdN0rCZzyuEOfnaCXXfU1dJzcgSs
Y7kJ07eCuq51VqgMmYBM1xFCx7GCfUfpgY1Lcbd+mchIVcfM0Csk4TjR2Q2O
iHEahwu1roJKFGLjfH9TkR1yQ2bh+ZG4ntqQ1LFE5EaaomWttyYH96FQLCf3
ZiMt3glPb2/C4xt1SAqaj0dHkyXEEam9PpOJldFmUm6PgKy6LscTcHqTLzJ8
p+Lhbn/hw76j8aQOlS9DEoe6tkihrn5qLtEdjRnzkUINLrKQ9NCfajh9feRk
pJK2uVExT0xG+EwFnynOF81GNHXcWr8hIkn+J021GOsblhiBJDXqQ0ZL/gt1
ldRh/TSdaIT9JI/Xkf5Jt+mPgLlfI4T849jFg5DnPBKJi/sjaP7XCJr3NTKX
DcR6GgT7iWFPZEzWVC69PU5WXl7Oc3XfCsae4vXPankSV7BsDR+L5Y6DxPUD
6T7CrX7CSvcx2BAxE42J5mhKXYjmVCvsTKX38SZYE0g+2rLBCCOjH0DgjVv4
Pco9h+FA0hSFShV8svLaFsHj8e+iFFuOunyqwectwufa4NGIXvyjaNnbVaYf
yo3N5WBclZJGnbQ2fLK+8CIn89I6a8TYj1RLgiR89sTTnR7YlGlBhne6iMIx
Lh/vD0FO8CxRXa8xuKy2TzcEITVwDh4e09bFfSjY8PWVYpTk+2N7bSJe32tE
ZsxS3NgbLYysYmzLkxdiT5kwtjIue4rpQ2yASyJNVCsr4j3kXqd7TcLuPHOh
Fftp4kRcRsBBt8PL53KViR4eDy+fh/CFA9CRL0awjMfeoorpKunvFDJXxa4D
FWmrhDqV8AqHQUPmf4lmUrikwf6qFqkZKSE4YehK3IaQ5/0d/Od+RZKuP3Kc
RmJt6Aw0JJjSk+1IXUDPsivNAg3xc1ETOgX5TkMQSfY9gPaPIEduTdwiPH/x
QkUeG+CdO3fyRPm3xe54Hv+NGzf0ClVkoOps7ns4kDwFFV7DBCH6zf0WiaRE
KgOmoD52HppSrOiP3ONkK0M0JlmSImxMtsJ22rQrzRLbk8ywJdoYpV6jiVd/
gN+cL5FtN1C0Az80XeCvajBgBg4kEk/y5D2iU54FRy35mRwJulk6AyV0E2l2
P+F6+TxueAMFp5yDPFNiKmYoHcybq4dTjgTszTUVTtTtLc6ChLTW+16Tp3AR
W0r1g4NMXh0bPZDkY4QHByI15fSGeN2RjIrUJdha5KSb0fEBicVcXD5SgNRY
B3RdqUFWrDWu7YnSYDUHFckLsHuVRhhK8YCN2YuxIWU+O6la9uSbSnUfJ9fq
KOxJnsIOaTYSe8P366w/1YzJ62sWIc52sKgQ6iI5qAUqvUOeyxBkLfuRnMAp
ekAt4orjKcix+0E4C8wlp3JmSNnnD2WVNkPU+dYEjkL8wh/gM/trRC0cjFKf
idgYNZcgYCmoajv1P2GAWovx8I7Aw7ZkCQ87ks3RGGuCVZ6jsCpyIa88omD1
+vXrOHfu3C9FFDkMrbPwH6pY5eHDeG2MGIdM2wHwm02W23IA3dUkbI43ozta
INBIACUoywhNku5ohyBVC9RHGiFj2U/wn/2leDLWoFJpoTAUf5GrV/ljW8oE
GnFfYpXbwB5aJhUTsoroNPY/oZi4mCFJ9PmhRl82p5kg2mYwrlQv0OhLQ3TV
SxU3tVL1hLbapqV4CdKIrR7u8dfzXp6T98LzPBoLbHVT2iRjfHlnBJIDZuPB
EW2W+wMpaH21BJWFQSjM8UF27BI8PZGqMfCcxDDHkXWaJIYhXsnZzR3LRXbz
HYXN9wYiw1uiUTLlOl/FDacqrdW6PZ2vYiimDZYGjEO+61AG4+cag17uNxIJ
S37AhbxJPOzVRJHkwU5E8uJvkLL0exGYJiT+RU5DKtMhdsVOxAqnwcJ2Bsz/
gSA+lgA5rwf3OzXZNsZjkqXo+3fFq7wtkeTMmkiDX6xCftOFeU/A7jjBroFs
djQxte/sb0inDMf68JnipDLU+orrvCdQtjPVUpjtIrdhQk6EmX0rUrSHU6cJ
PUoI+7MmTnY8faJIqhY69xemg0zIpzIHsnE5v3IGiZ8fhaTk2jVJSxqKaCiX
56z0GaMXBadW7yhbJOovLm+w0wuEPmiSHJddK5dwWrKnxhnlWZlJ3tPVSLUe
86UR8xVrmK+vCHdebCuAj5sptq9yE58F45Gv8oJc4+yQOZILRAAVc5Po7/mR
WOSHmKCtYplwm/poeI8TpCmkGu9uddbjPSZsnt1bEz2Vo2FaA703axaiyA0W
Ema1qSoY2Q3eHDUBEURqPBv2ZpEuEclu8LH08aJ8PYesEo95AtSfFddU1mi7
CV8pS38U1BJPJqYyyAgNiRaCWqivP5YxtTV+PlorI36vW0zPfbkV5e4/wXPm
18gh7G6KM1XOTbcqw5awtZMItClxHlZTN0eR1WfHhRXDMcljMtCiiAu/Ehd+
LWoC5KI0BUU87edo1lQRaeGSAg4aqIpv9XxcKTcjCzIE2+WcvMJTDzbbiex0
acg09kZ6aQPUq6yR7DkJD+SSOy1PcQXPlnxrPZ4iUrm4I0xEWfRFX1/BU68u
FWLDSg/cPhinM57EU5xy4fk7nbtDxTwzeZobHh2KRor3VFysdxWpnHfkJFqW
7xQ08xS6HTqhx7e7v8AS8fbDcGPtIlHC1U9jP69WL0SM9SA0J0jhFC2ODlJD
h5h9jYNJY97AUVvKOBHaLnb9WfZ6jXSl4kYCRweSJiOH7IPXzK+QZDsCtdFz
hQkSnJFk2fe34EhaFwaP7l5DQ10NNtY34Nqdh2LbratncbCtA113bmL3jmY0
72gSKzCxyjvUfuGXZ0xST148hIbIKdgYPU9Rcwb/pBKmlUDj7lRT1IdNFHqT
6/XZ4MoGUi6KV+L85Lkt+VbMCuU5TBrk3Soxwq7ESQJ5onS7Soc8Uf6UO0to
uDNlFhzaUn1hDvJeXruUfI0RYo4g+b96JJbjP00J8WvrUc9t8kEikVjXvgiG
ix6J8Rxxnv9AJCal/j/G84404eYKtMmIu3soQTtjTEZcPG7vCRNRlrs7fESU
pbeGvNoq7JDsNg53ZJe3r0Ztdm11QbLLaOxMN+HyJ9U4clF7WeB4LHcbyuz1
mUbYnl1JjWL1PbaEDhfh3T92i+GFzv87eX9DFU/3T7Jx5I+HU6eiiPDoS7om
etFg8uHmKN372W8E28snd5AS6ovQmCQkRofCJywZXU9e4sSeGiTlrMKls4dh
bTobth4BSE1JRnR0NMrX7/q16bm3z7eJyzLI/lXRiAwyciV2kQYu9RgO75lf
IM9hENqkVPSfNOGWs7lTRIaSk4Y8jZ6Uq4IvDrXUhI4RM+u4TPG2xkfoIvvI
CZN0l+G4U7NUjHsJXz3FNMYtaXOQ6TlOnb7IPNFWbiOKne93o7UXrWEoSzBH
XS5PXdSjtQtNoYLWHh1P1ayO8bEqs9Q/spAcYM4ml/jJ4WhBYwwuEU+p9yG7
PE2Np/RWQnj7pBBeU9Z8vj3FKHIopSl7PtKc+cmWaI1iKzlDkYsHCAxxY3yq
0rypcKJSlg3EKrI8NzWii3F1NE0iszIdrv6o1n5JZLaZ2pndznDS2tXBxqL3
SPn/8Tfh6gOcP1gLT+9w3OY1ql7cRoiXB7a3nRdLFD579oJXI8am0iw4uvvi
yPlbvTR46h7rE3iKmycu/DcNnnamkcqPmS4UfSDprk2hY5QpGH9U8MT+d/Y0
4fXwxJ1zQoDqLGXJDBS6D0UmIfFmheRzfqCNjVRbilLgDVFSUkobu7tdtwzJ
rmOwK1eblCIJQ3yVGzhdVLBpRVdLEC5s9UWi1zTcIXLR8tWrE0koTVqIpjJX
XQpamm6zMW8Z1mcs4oS1tqKZqyJLImYyYnprEo0n1jgiwXmsyIjrMdN2V1GU
GOcwAgfkiqt+Gu69uW4x4m1/Fvk5rU28QzAq9hqONJsfFH9SrecRMEqVYLTK
Y4g+jPpISpfk1grnwaR7vsQKj8mKfiYd84sY0lvmrKU2HSGaFXrKkkN4FVYl
msGrf26tzMUyJ3dsbT2rrBXKgQ9enJDjIu3t7T3egqJ/l1HUQF7DdsL1HjJ9
Vb4jBY9yXqJVKqJTUCRieTnTsdzhRzFD7XyeYKV+Mook13AGUsnDLPMdIcJH
CivdkWsgIxb/hGOF8/WsHkcuDqwwR7zjSLXHDNUIsDuOVtgi0X0iunb6vsFO
vJiOSG3pi66z20LELMIn7amidqWXpjjl4bFkJPuZiColnn2oDU/U5SzBuuT5
eHkgWCEjBuzKMGPUp8zWkhHnMOuT5yDHfRRTrM6+2aA6bBKynX6mkaPat65q
U2yNIU1h+a2yIIgegI7JPFTqPkSZmPW5Uj2UJfEQh80iLb5FoGl/rIsQ9k2V
LhoAiX+8FiN3O5ezdQuL0QUP1GUiNLFS3VteOkyhnpZNFbB2cMehszfVgBlj
iSXWxYsXcevWrbdh6X91Y6TmeGNyJvrDj8R7bbAaoP5ck0g9nTsd+fY/ktv7
rVjEh1qln0a7n1lhjEjitG3klui0uyG61pijPnYaUhyHsR1QyyPqlyHff6JY
BopMhcJDj3Z4ID/YCI25C7hbtTx0udFPTHvufAsPcSkMr9ZEPKSC52IuDqzz
QUmcmdBG2vp5Bl15ghm2ScXRvbRsV+eGOKcxuLpRmrylJaPrtQ6IsRuGowXz
xVScflpvttgMEZpI7KeatNRxEpUhpE33ywUvn2sCCScyJohpMIylU/pYOinH
kCq9hpE4/xJp9iOFg0dj3+Cvv4Clmzdvim7nRcT4/8rKSg2WDjesQEB0vrxA
9CvkRwWgasdhZf2dpw/uoPXgYTyRfz+hIikAcSvqtapJP96lj6n/1PITkebe
NFOsDxgNn1lfoMD5ZzE2CEhaTJ3KmSYmAGXTH1fwX9dhinN2+5MniXze6YKZ
LDpVTN2qtkAy4akh0YjH8DuafjhZoUQVlomogpaa2qvsEO82HrebfN6gpgq6
6bqcpVpqYpPVsTkQWWHzRbiAqclQU07DZRXbVtoyFSlYfM6rN4XNRGuZKK1Q
sgysv6vi56I6mouo3LQSifG/QgqOKKzEBfc8FW5t0GiOpWjVEddZJ1gPwFq/
n99AEtfac+Z8JdmubqzEjHSQfOqEhd/Dd+73qAwyVqOP//zLZk1vtbDOcy1w
c3LB/tPXce3UPjg7e6Ht/B0VO7fOwt3JHvX7TuLxwy7kRHgheVXD/6i4Zez8
l3jto1NIsdMRavoNkomTOCVLD2TwucawnSG5nbToaxQ6D1CmLfVVF4Gagk6S
SOVk1LIcBzOx67lzB3NmIcqaA54LRdnaO5pAAkevy8I4kOCmMNLjZk+xEgMv
GtItHMWeXJIPh6MiNKV0clbzaDy1gSmObwoUWU0VOGTTKpIX4sBql+5lGE8O
xSDdb7qYWMK5U2ViGufIeZo709IZeVqPlpZu1DMtDcfRlfPFtJ5+Gqm9O2MW
YpcOwJVVc8Ss3E818c1S7+FIJ43E4bzrK3V1T0p8k9d+4QSgHDf4TC2QlbJD
nMf04diT9TBsjjcXGulff5PO7i3Ip2lNITzc3eHmQQ28bgde6taXF78qsbUK
Pp6e8KK/oNh0nL95/7eC6b9lMDERsQOwN9kECYt+QLj5tyJbekoDJhGoz5yM
aIsvUek5SMwro8ZQwMQ1iZcLjRFHbbgpUoptKix0d40FCrzHiPllGhZiaXqx
egmi7Ufg1OqlovbHUBOJPrvBSayUdEOvwleq/eHoeVPhMs4Tasu+9lZ5oCjB
Cq+VOR691HqL5VGmOE04fK3JfrO54xAphwJubfcSoQB1BtvBYDTkLMCKwCls
33SpG3fUJc9GHhd51S7tpym44HBUnM1gNfT0qaqSpDke4Rbf4BRxD6ukz+QV
CXlI8uxSrnnJsRvYQ4sfhhPHBvMJWhxzWuk1VSWjf/st+FH0TNedTnTevafb
iZPa9+7ixs1bePrilYHhb4TLB+K1txD6Ul7GSsoa0U2RHfuT5utdKabIdRgs
Yq5ypEkCUi8RM+c0TJjp31EbNITtl4HWlvH851DL73CCBALbsg80wZRTrLWX
DMKpUnORnFAoibX2RjIQeX4TubMUSuIKsorYOahOmMumRZsbbN/gjlQ/Yzw+
FK1PScQv91vjREL72p4YUT6qnYj68GgyknxN0Lkr9M2Zy9sDkeI1pVu8QU5c
7wlAmuckHFwp5dq1vHRrkxNiyRRzkvq+6rsZyoUUU5DpyKp7/qeahjhXOIsM
/ndojh6pVyfF84U4Xpy65DuirAHyikIypKRBzFVa0VbfIchsADZEzuWe1PWc
1JnMBduS5I5NkTPD/68j5PogeleGkpRuFGgmB3welntOQ9SSUfA3H4o0p0lY
S86Asg+B63PF7tHue8jCrPYZAS+TL1BNxkspcv9MQRhZ8f0JY0XVxPaIEaKB
+mpLekpnoDpwFHl0A+W4wPwPFBG+zkKsR8grERHA3tEO7xob8uVG4VARr3ym
E0w8reNynQtR1ThcqnfTo6onLSHIC52N/RVOukiToVjPrW65HdbxAkQXc3po
lHhjiQuKY82UNIveHANeXiFfM8dAa2J5LYVk17G4I+fI+2ocOp5Uky05dNpQ
+fkKS0QsGijKNXVK3FDU6+a7DEGhy0BtlacozymaiiKX/ogh/DA1dYNWK89w
NP0W0YuHSgJc6bePtXBCXYwpCn2NiCzHwM9sKBLsxqMyeDYdotuHM34y3n5f
6ZgWa73Faz+9i9fSxfPcpxLOhsBhxgC4zxuG4CWTkeQ4nbYNg+OM/vCe/zPS
GYPhehj8VDWXZCrTzVEXNlHEFDjS2Z4plkbSw+COqNEImvu/sS9+lAaDkq28
WmSMRNuBYtFGNhYfaDI4l8pMEW3zswj/6eJSEsvtyJqPNLexvEKUynLNHliX
NA8VMbMZCj01Bqx1tROygmbybEYJe+9Js5Z2hYso5wOlQux8jphSmUjEdkmK
H2it6o5ie7EOwDPdbAsORxVHmAhTSHSmZTMueY93GK6W+mrZbFXAeBFxIrhp
rSNPHIiy+lZYQl1heS8RQ+BqsND5Xwls6RpYwltL0mQEzf8G0UtGykUBcmC8
r1rYwoKr0McIYQtHiO52nfMz/BZMQJKTEUIWjICTyUC4zxkkYTBklsDgtkQz
HK5SM8tvVonxj0esWrWKAxJvxV0v8fq5uInuN+A2dyhi7KdjRdBilEbaoyzG
mRrXCRXR9D5sAdJcjWUcDhA4ZC6sCpsjR9esqDl1INxBuNwRZ0TezLeIW/gD
DqVOFQ7Ap3IbcWUtzwFvCBsuyLAtZSzrtr6aaMSRzGliQaOjggQUHBqKioet
CdNFNOJ2zVJtaPROnR2SXcbgQIFkygw1NViXayXNpkyY7NFtrl5GgAmOrBcl
CnJc4h05n7MIO8rdBAVybH1tto1Y24r2M9QEsjZkLkJNqpk2U0SgVCdYXVov
+bJ9lUI2EmtNWabIcB2hqbnmGIS5WH/o1ApJYvTT5mtky7orRmtZJSzuiByB
gDlfoDlGFK7oYXFv/CT4z/2aSG206CviDYkvPhMwYIldFmiCGOvRhLkBcJ79
M8KspyDPfwFKIu1QFu1Mf06ojHEg52oRcjxnIpjxOXMg3Ob8RM7DKNTzYjyv
1Z+G4Z8r2b59O4qKit5WHcZfdXZ2vhWgPcTr+yq5MerzPKbCma7mNGuwuLNc
/4UojbLHKrqz0ihH6oGSSAc6piTKkUDbDbAWEmADLIehiuydEBDJClh7S85F
ymzyUgeKMkmpPMlYi9Pzy42x3m+IWBvutBRL/KSbzebcUBKJGql+TLXZNznI
4TCMGNpY515IbLk9cx6yvcbjgbIKjqGYO1gdL2lCDVty3e3uUgcxz/jFsTgV
m2R2T20NQkboPLw6k4nLu6JI/s3A7b1h2llmr9UyG85ihqgnFcuQWKAoWExt
0mYSeaHzFJdR2J05U48l2TAv9xyF1Vy1XW2mtcl5Lj+T8dXY5F5iyh+v6Rg4
5++oCx7NVkcFJVsgrtPzmfUVku3HKqDsJ5e2SAkfS5T6z4DnvEGwN/kJAQsn
IMPbAiURdkq3S73+Dr06voHPADKhsR6WePb8hQIyAhz/8oI2nPZrVWAaTKpe
BRNmrvsUYuWf4DBzEOJd5osRouDwPXFH7+lwSHdUHr4QMTbjYWf0I6KWjsFG
dg+FuJTT8qQEuHKPqCPbbpCwGVwVSRBU7DXPZF3pPEDMthF+Ppmhvsqy8cXG
Ivq/XrLVH2hiJPszTcTaGDfWSavJ9NHw5M0aWyEYj61aJFLeWt/2wgYHxLtO
EOEK9jp7aEIYXbuDkEwW45LefOM+UtXNsQTkR5nhZEMINhU4KEkCdmrUuNry
sJk4VG7Lbo6WfrkILc1roqxfdR6IMkORf8ZCWRSnrzoJ11pMS45c1B8X5Mqc
furjm4r1lCLZcOdJhltX6SQt18thNk5ud8dlQ8RYEanNdJ6glE98opRcyVqN
KamIHDRfsyFYZjwA0fbGKApbJmCwKspRQoGhisuyCFtsLkz4XcVgCvpYFtbH
mSHdeSJcZg6AC3FzgvMsFIUz6pw+UYYAAa4ixhGrQswRbDWcrHl/YZ23xFvw
HatAI8OsAI25jrNNJxWgGYp8ZQJ5+7ys0y2NLaaRfCRzqpjccnK51vM1RGe1
uZhjsClumpbgHmy2RV3SLBQETNatJmiIR82eYibybm1ZmKFYWmMtWdTa7CW6
2G1voe94ud7CeCtkh83F8Q0ezGaGGneYZz1laMJtPTSW9/Imd7GceedmJz23
g52i5eSM8+qRdMsqu9Vai2mgkuo17actW1o1FzGcFgkfLuzup5r0Gw/ITJvv
xTyTk3qO3gxsDBoFD3IE8z0mi8haX9nL5E5lHySf/ErPuYPgQPwWucwIBaE2
3KmCSj4SPdsThaG22FKc8nsd23cEghoT5mMVWXa21M5zhiPNYy6dlu2m02cy
cgivqI61Q4HfHOLawXCZ9RMKvI2E+aX7/ESDnF2EnEziGo7psqd+UrGSvWQP
a7xQcmIdgKLpBp9oY7mlvBb0COQ4DRYD9QNNCJNjuTHWg8XqRXo8pSnNOb/W
VpMJkKLuO1csEmB6pltZl6MnJ2o8hJR7elg7N7q3qOq6tS8SQY5TUBpnjudS
hZehJt724ECUiLfdbPQUa0r30FT8HKuyRxaZazaOZCTp6XQu9rm1NoixHYJL
8pIafTV6jgO6UURR/JMgnSpFGQqG5ulPGTY/aGvRWcpVev2EKItvRMycYwdK
I3NCYLX3MILUl2Qap4t8ZF85X8O0tC5iLryoC53IMsU5mciUJOD0iR6akn8v
mnrj7sUjyCHxvHRaf0Taz+Zz8rn/JKOoOFJ8RGW0Lbnz0+gWBsCH2HG1EFwL
9FFkijSbgSKQezhVD0W86CnXInGd/qnsiWK5hk80Y+1CgRGiFv4oqF3nmJIC
IZXFC+ZujJ6qJSFepoETAWtjZ3CsQecLuOL6Jhdh465t9dDracUXSA+YgVP1
PnoT614cT8TGPBvc3ClWbuj1llXp78lLbfSQqwQZj8x1hYFTDLRRWs4hcU0W
2TCFfhgry71GYY1IGJn10wyQ9jwT8eMJvOQo1wZ+qsEKLzofOPcLIfFPqe6V
NCBXuQ+Fq8m3qAyeyYrD4BMZK0oxMcc5lhn1R9CSKbKsEljp9/8NKwTtC0ew
gqgtzcuS7d9fNTKNbVRV1GLELRtPl/4RuR7TlIjexxqM7EwxRar1QERYfCvq
tDQYeZsY+kQTom2Kn4CYxf1xlTibZ3e9r2lI/qkZkTWq0mcabvtjJQukiRyy
tTBUk9nuIprQkGOlc+0kTc6lC7xil+oI9hRLM8n1VgYKQLju6lSdD7L9dXVX
PeTTvGoNQV26OVZHGnOO4ZPuVTP2w3FMzgj1fcNJ7I9zcvGVQiqdlabIchgk
Ft4jdHyqMeC8PE/QvC/EBASWPJ9q7BTPVHOZ8Q25R3M0QOkjeoZjUhGLR8HW
aADinOep9umz34uS7sGwa2eOoTTchk7mZPCv4mR9iEacUB1njxW+s2Fv/CPJ
17FCf28nf607PpKWDhATmkTlfraMD0N6OxXxC78WS8mrGsZQ/OgVz++pCRvL
YR2FOe6ssRAL2nIoW8Mcd+tskeExViwrSOPVUPbZeei2FC3mnlQTe1xvcLbe
U6z38EiUciqTeXoKVfziqBYNPUVC+kClC6cj1XwOrxhQGTtbTAV+2uzxiYYp
eBYEL399p2apyhQkfIt8xwp7qmEKNihMjFGW3+BCvqR5+2mjegVTRNkHp5a1
TMGal9Mg7iZfocTfREwFU/ia/RQOELGc9DQfi/wgawbAH3+5798ekOJfWD16
9Kja91fPHKeDbcRp/k3tewehlstDLUTUy898KOpjzRiEmr4nRyllLj3HD4gm
NSqyytnG/bQqJGk8Qt6iQngMHEydjIgFP6iF/FpuaMufLZb64UlOOm4w5LkA
wlXnuQE0LA21I3SrKxLcJuBCnateZP3x/mBRfMnBpNftie9rgkRbV9iKIgJ1
JTFDseRgYYgRmnNMWdYonc8XyPUeT49shIebbPpqguM8VZ9X6TpdYCIeo6+m
bpDjD3GLfxRhNa1UZVvBQyKKGFWOjhtom21fwiThjeeR/GFdoUVAvsdUWJPd
j3Ph4e/Iw/9Pb0FAT02Hc+XSL8R++Fd679y581YsfCCfVlYX7EMTVRIq5G30
rdjjQ/HaW4jXdTELEbpgBHlHA0V8Z0fKAgUuHEM4kDZLTMXmGCRXBXbo4KLI
jbA35IYhbhJfLHf9GeW+I3V+NSsNc2SRu7M10YiRoZWqV9dbI9Z+JM5U2+hJ
1cc7PMTg5toDvfhLCBoLbESxL6kMLUjYFarLsODfpuvRLU2X4zsZh4slJ/lj
rQJdIylQZRmfvprANguNUm9RHKgVn7xcQeLi73jJlk81IZsDCWMQMOfv2Bk7
UcRrlcbijC+v8MfxicRlY/SGJfvFHO/NcZ8igJLqacGd9rHSm9RrjJtf6s0e
vwImQgquXr3KP9nwPwdt9MH0jgyVUjkYw4DiSy4PtkGSuwUCrU3gaTUN0U7z
idhsxG1K+zmqAONjq2KWIclhIpZN/xEriQxpSKgPTkqqJXUmYoiLOKXZIVVZ
KAu8s1bh4pYM6+/UBAuHH46Tnx0unGmpJOp9DYXvSZ8hfr6pU565rvAQF5JX
RkxHlSg/UsWsUkue5k2OtjyVT+sEX2nwFT+3yL84xrOt3lMK7kiUlMbMQ3OB
iApqYzz3dvkj2X2cKFNhIH+s8bnW8gTLABauKiNxpROvjxa5ZCApVWPNT6nJ
GaOSOYi0+k4QMqtXbe3qhbzJYoZUJXk1LEr6qcXTUnlBJNn3iMUjpHmeSZYf
qeNaSsKlOU2AzfT+JDQtuHcl9vhE6nBmD7nDC0KXIdVrAUJtZ4kOD7efi5yA
JeI73kdIGnov4+93xGn0AdebX3swKdFNZPotRqQDOWRWRJ4zR8N8yjAsMh6N
pXOmwdViOpaajIbF1KFYNnusQGKa9wI5qCNu/n35jOxYVceSIvI2hu30H4h8
pvDQ+1jO5XBL7E6agaC5X6HMc5hG2ZESypoqyn82BQ9l+v+4mzte4jVc/LEj
9b7GgvA6/wxAXh+TM/5akcw/yJTgNFIVyT21808a3cW6qR1rHTl4rVrDpy2h
WEGeaYsolkpQAMju9/LQN6OBhNk7Tb5i0arrtfZiRZSPtMtubXVGIt3B4Xwp
/9hXjajbiIVGcp2HsJutJbp1waPF76zdUKbUGgpryGsCJpGlVMdrb/V3jnhG
XqjlT8TyHPrVwU6OxQVYULeZ/IwsXyvGjtRV/VSjlRtojRhnU/gsMoId9a75
lOFYMH0kbOdMgqsldePscdTzwwgBo+C7yJgcR0usJFe9OHwZGlel/mpG+dfg
ZyjDL5rFeTbh29l0ooDdkpkTYG8+C0EONEo93RDn74OEoEAkh9BfkDeSfO3h
b2sGh3kTYTV9OBbSn5vFFPEQReEcLZJDmn0kOJJAL/SfCzvj/gSU8QKD1EoG
H8mYFJnFiCnwmvmFWIVFN7AlsdkcPVoyuVmSyVWAeZOZMWe6+F1aUSyqKjRD
se5YbcwU5HuNZiDq6/Rx2LvcQkeKUh6lhlwEXgbvhSaPQia3udhOWvj7ePy7
2rK9wzHI8DfC6Q1OYi0vvTjiZg8C/Wh1nZGP1Fi5O46WLhLBaeXXaT7RxNiZ
wRPtBtP4lCZCaknxfPFsRFh+i5aEMXqkyCqVf/kocO6XJMkmCwvcV1NryyGg
HHJj/Of3xyaCIhkgLSlyUNHPfAjczcagMMxOF4z+XCXF5SG2hLrpBIyhWDpz
DOxMjeFna4lwD2fE+HkjMchfACMx0A+Jfs4IcbCCs9lkLDQaCStCrcOcsciN
8cHL3/crdfpQ7SGrPVlNCvJzNZsEs0lDYTPfBOFe7oRQf/oLQnxgIP0FIC4g
oAe9+BMm6C3iaHMigzjYH2mBroh0XQqPBTMIvCNgO2ss8shVYanxnta9YOse
Yi6SnPG246QiMWq8D+Um5Lz17jQLMSGDq8MO8I/kKXrRUPy8GE/E4OnTJJW0
UrHAfaj4rTvq5/c13Xy2ZB6ilg6WJqyTLNMZcxs0ps9FPteENblpy3VOr7UT
Uy/uyj9wqEXitW2SMb+7L1xUz7+jjV2S75ngPllac1QvdsmTcxzEMOmS5zx/
pHEwKyOMxPSJB/XWusyKDQ7kzRU1p1yszDWnfZUiJEGoY5C17Aft5AHJwZxK
SvsbseoSj3kVt/KSX4UuP8Nz9g/YGG3KYNWyah2NSE6mOM8diUIygoSJdxWI
CFZ1RI7/EvgsnC4IbfGsyQh1c0QCYzSY4CFgIiDSU0DkHQERwXHBAUgM8EKs
py08rIwQ6WWHFy9e/A7kfqBFrvjj20nxtCTTPU7cjqM5IdbDRVyPbuN9zS0w
eJNDApAe5AY3y+mwoFGX5G7JSFQsvFAqYRZwJFBypcd2SetoMclFZZnLBgmZ
2S79Dl9fjSw/kjZRTNVslecOK4TKC2+0ZU4VhMouL/ejjlAtUOAzBjXRIvCh
dWQurVkiHJmL65aJ2RMKq/J0WI5htZRyXbzqyDw9EIqVUfOwr5x/0DtBi8mb
u0LJuk1Wf6xTiXqJ9UdKrLHCfxLuN7pq7fsdYd9HoG3FPDFjqW+38Kic59Ry
Kf9qX+SC78RSpFou5WhgXeAQRJh9o7q8fTVRryrfESI8ujZcpNkMFECy/eLq
KpdZA+FtMU6YQBWQ7wgTn01gFHw1mfhq7jQEuzgIAFJ3S71vqHJUAhnbyhV5
v6NO7D09wDE84t3MMHfiUDhamSLK21PCfGDAJ+JSfRAbEIjUED+EO8wnBv2Z
5ORMumEHIUQl+usl7pqpz9GkP5LsJwj3TTHYIiqfPFeU6uQ7StEhpaGUMp1q
78Fi6rUu3CoRH/82qjTHwex9TfJ1X8YMusybXgyH5IuDp4jlKZ/qG+zteQuw
MnQGR1K0BntniT2KouezwX5HE1DlhcGyCIwPugVUuayBf/BD/o0h1VbzpLZV
Glu9UWerOfF3vsIKkYsGqLqjr0Y/1keMQ+rS75jj+mlGVkfWBJLdksTRthhT
Ha+H6WHyFblKJiKaIrW0ocjScKWNLTktgUuNe+hw1Uf0EHslZiQXbecZI8JT
YZRAGtHc0z0R7eONsvy83xZwo//VZKCCKEZTqpcV5hGaPGytCUnBjCQmK/8e
fDk6goHMF+ZL9lYhFoC0UGJbL2shDx3nTxRu1SrVwkoQKyWI2ZMuTHXUh5io
6Ig1gu+sL7EuYBSPP7XBOrKMxQ+V8QqejSLDOl2nCTlqlzZZTEpjR5Ip7D1t
uIuclfhlQ+TyZGvVWeG4x8ECCzFNtqvBWWTsempqwa7WOyPedfwb1pKt7I0m
PzG3/85eaY5aH83s14NVLmSgZvDv+SggfUUg3ZBsirUxMxjSv2pbJcvPC83m
uw5hoGlDd9fK5iF28Q9ojhohfjqynzLMqA14ZdOsZQMZXGqzkRxszyIf2eJ7
kpljNTiTKGw9+Vyc3Ymwn614uO9qwmYrSAdakVdiY24qyywJBXTBBBkAMhB6
vQV8iqjjgF5jY+Mvxfp4JmRXV5eaRWQYFofbknO8CKaThsF58UK+ggw/uh7t
4+Ngj8Xz5sBylgm87JYR9Px4Hx0S/cXNZQQ6EwonkGc1Ahk+izRaT0JiSZCZ
QGKa00Q9JPKqGLUh44R3siN6gt7QFaHw8BGIMv9SrCPLZPexkkAoNha/UVQb
LhIIOjtqTsZiklgf50GdtbbG5natLZJcRqOtZJFYYlENCO70kjJKuVbaIohn
B8NQHGOKnbyK3YmEPpoiV56gJpb01lEjl0yURc3EppQ5eNbs8aHWhG4hE+o4
8g0TyotuJtsPUbKnfTVBJ162PX7Rd6I8n2cRaSs/+CfNg8gd4bo4LnLVNhZP
ePSaOxBbEiyEC/ih3MgcF+NaKVeO45OzsSraWe0c1vsc6LCZORqL584UvS4h
712VegJdXWBtNh/mM2fAZekSRPv6EFUJkBr0+BU43r17Vyz2VVVV9bZoYXt7
u2b3Xrh2th0Z3pawZL/GypzHASGMT/+heish7m6wmGmCKeMmYN5sSywxs8CM
iRMwe+oUuCq3xoOFjuolcyYfmxrsA/+lswjmQxDpIPJmSgswCRcFmpID/SMy
nCcJeGrlHtdvl3oME4tOyauD9pVdEF4QlOdPyBU6H2ui+3sSJxJ99BdLBnLi
U6FJLs/rKJiD6DcSn1IdYk2CCdG2KALU5rn2Fy3mxU/05hy1lHMR4lwuQuyj
iVfzKgG1+tHtpzyFO2Q6duaKpMaHGpf5CLnMSW8xw1KxhCmilwzAldI5el7H
zYr5SLQegK1hw3R5zZ4inbXKQ+ttSOaXx7XrjG9E4b6ucXtJNS0EzdCFw+E0
byz7hDIsewtRz3rLcR5xyswpiPX34z6VaOdj0aXczZHeXlhqOh/Txo3FXBNT
LLVYQFCYDJNJE2C/wAoR9L3CnTJCf32OQPfk+fVzJ+BhPgHLzGaTXyPTXl/B
yXzOQFdnLJw7W7r+TAs4OYXDwysRXl7x8PMMgePipZg5ZTJmTJoIp8WLEe3n
q8LTUIanMOakF+PdLGE+eQgClproYjwSPFcGmIr8fJ7nNJERVlqwQS4gS6XO
SOKf5NIQAscm9sWP08V2NOx5rXA6km0HKhPL39dUj/Es2W1SeWwfTZLrVPlC
xDmOwi15eYKeGoa7uVmq3ri6Vd96c/T6dnOA8JHZ92Dr3VsNeMeLwM/OgsV6
Ae99gWLFuVy/yWiV1xr9UDN5YXWkEapC3zDhZUHjUS7yr6oJv0Oyd3/aNERb
fSMWJ/p/SHsP8DjLa2vU15JsgwHb5P/vPef895SbQnLOCbjgKlu9d82MpJE0
qqM+6tKo996bZcuyJffebWwDBhsDppneO6ETSiABQgIhYf3v3l+Z95NMCP/N
8zgYIVvfzKx3v2uvvffari5NV6dGR8oyrldTKrJYusepMNWdfivGS8Okt1sZ
eiKhmzqzC+O8+dIW8XOBShm1igWlI6Qjp5sj0CfiEYVJDxU4WhzrqqmGIyMN
prBQJJisKC1pQU3tiADOCJrrulDuKIfNbIJZ/PfCjHQG8qAAz5GdO34MgN3w
3hsvY6yhTBwgLSDKcb0C6fFxAr2hSLGmqQ8xjOoa6tOqqh4U31tVPYRageb6
2l44HYWwRtPpikBFXi4GmpkLSEhuxphIhkbqizm1nolk8d7sbUpg4fxwawJT
JPmKengsnl1uz7bKjFQReqZLVvH+cT3KCrSKhPJCbyg2F6+nCLtQyhFoHIHW
dotcQtMmPxPX/0S1stRaxD29n0SEQFpaeVkxlZJx+OfH2rFPZNKPH2OlXG8g
IYGSWtPGaqPxjjrFJUOecqCJunC8fCSP54qvk3u5yYS7KljfdrVYSvQ/PkPe
Kxt4NSAt9dGiLZXDqfZ5Qmmh0HgokYNB+zKcagqgiCvzALKKa7Ctw8VRIw8g
HjroCBbY9WFBjjLSBXq+UyZoUAnaCxNEHu3NCnV/Y6ME3gWMme7aGuQIPkiY
sZqT4CioJ7yIX8MKZBbAKSBTLWBUX9ePqhInUixmxjEF6r2Tkz8Kvm+KXLy3
vl7P4rXHGBBnoVCcn5iQEGQk21FR0cHnRzzKPPUxGLm1Q2ip70JlASE3EvFR
EcxoRzUe7cF/20h9EQO2VQHsnGv0ck8pl3smnTHMXs/0sjZxvUReTzcr7kjk
8fiqFEfo4nuMw++vedUOhd8bpFrbmztjufnhRXZHS5X4QQp+s9/G/OCd41kz
+IEDtw3G4USfmaKhuxQYL+/M4kxIxF9XT1QH7t6Rr/REvTzioQ8gjLL151Sr
iWrYhkUpDzbxtrq31fbO66QU7d2zRegv9FY5S4HEExTL5KH8dbyehLjOYum1
0PgpSesvbA7j8sISSQ660Okr4uxyNoJ/ZcZ7d7k/jEnD8Q4jabgscvaDzRbk
xnpiW0s+p7wahumqnGjIQkF8ENJNoWirKNFufwU8c9XfVorgRZE3KS6Zgt5c
Qu98hs08HbnO4koBmWiUFuThz19/PXvk/hr6J/7w4Vs4c/wwjpw4g7d++3v+
2odvv4T7H3oKfxXfp01Wf/Tuq7jytLHn6vsA3yeCvcKqlSSLXkNLRRnHXlOU
BcXFzQz2qhoa01Me26mivbWuG2U5eYgNCRL/zOZX7C5BfbiuUEDdG20FCfze
XSNVNk8IqPcXhKImeYMIoRQ77NdJohQxjeGc1TzE6OJ4ggWL+LytaCWO1XrK
8Zlmxsggl3o2pBTtUyYZAYL5xblIhtIa8diONJ7B+eJSpazBv3m2FKM1Efh8
hgbPi8FPV2Cqw4pvaPWS0heqj9kcH8vgmRppBIzrmZeVeqa2fuM6qaB+eVsq
pmsCiQnLpcyjHeHMLz7T+IU7r6kjZ9YthStdyrs7rzXrz1DusNemLTKrINGz
M3WFKsZkGaIyCZxF5rUiNUni1HmB1PDQ7khggagiK0Whwa167BpVmURanAWx
4RHIz6uZw2BeyKhwZzBXO7sxMTZJ82BXA/G3X32CoeZaDG6axtSmAdS0DOLT
r77Fa4+fx8SOw/iLBOLHLuxH2/CRHwDxQhnE/IsIBz06Reyo4FCkpxYIwA7S
k4pXoT5qjXJriH8jEqJ8eT7jurFefKm0CpbwUMG2kzj4j2hnWsX1kMA1sed2
h1XCtQe3uZ7sd6Ax1Qu9eUECdjLpyMKlfhMaEm7G/X2K+CDHoCtDIehOu5l3
68jxm/LwJzZFYkAcBhpGcMlgSmn0wnAsr3b93KVAsLElKfmlAXoh3VCuv7ea
zclfPVM+i1B89mALK2BkRemy6NUmLhRD1DfOVhi2HpFy9sGFGgxTpVRtD7hO
b4Gqxr62aNwxGEvonlkk3VjihUfHY6QiqZLxjRWsUUcuTDKRvrfHT7xJahDf
KgVxQUR2l29AU+p6ZVRWt4Xw4CLTOUGtSuPWoSXPIgVxpQCzsSYTieF+aK8s
U9SwlhY5mXfm5yoZlzUdFZW9FAtlFNXMRJEbyit6MTF5RFZqDeaXbzx2BvVN
/fj8b+Irf/scPY21uOepNyAODf72t+/w5nOXMbZxHFsmt6Cz0YmikhqcuuOK
fAaMVYNrDGeA008BWjqnMREmEcBbBNZHVOwv0J+6pKQDVmsuIiISkZ1ToxEr
d/0sDPIRaapqhs0Ug6TYGMHHaiUOrp6FWgefhY5CK0eTa3QhrgzHujO5f3Sq
OoZ7S68zFLbSsdfpw8avL2/lj3OxrnSY2ef1RP0GOcb/dr8Fo451eGiMRSxZ
5Hh9n0Ji3jthJDGf3eHg6qdCxKvcJVBSX975LRmzEkLxa3dPEp4+VYXvpM2d
JHZQ4WFzQ7RSeJCMDmjx1mviLNEAvOYtvXCmKlfqhxd2K+3ci6Vcl3Z59+es
4UoWHW2NznDP+lYzejOXzWjpdeNKK1l9070oh5B7RYLijF+OUyLAU1XrOinf
oTJrpXU96rNiuO16gRSyiIrnWsQVnm0nqU2l4PNJOGA0FaSlIjo0DHm5Naip
UVI2d4n4EprKK3qQklKE8PBEpKWXwVk1QF93k8/ElkNXr16Q6eLtE+jc6HLJ
OzreiSP3PqkZCr3x5F3ILijFufsexpHtw6hrGcWLr7yrCSdU9jh48CDL0eL3
c9XKh3Yq6DX0NdQLGMfCHJMojnE/QVs5EAv58cvKe5CQkAN//xiEROcgIakE
QUEmREXZUOBwZaruGnETh6VFpM5ZyTaRboShsZyCR6t8MAZrHJzcdBYlGg4G
xRzKTIvMq3BMJZwy83lwYzIP6B2v9+PMVP6EHx5QLonfqI4EN2jp6QETzrQH
cmlBIGfhjJbV+zcZ6Y9gGTSGcorKFg9Uy4NVD+3KVjdYqoNVC7kSx06NE9ms
SJOaQv8kp/V372vFdGcizo2nsdu63hnPhsXKmiFtamqhRPDJV2mo2IctQF1C
oMJ/yIWdeio+PWHgP9SvRoseDUYBsbyJsN22lEdFXLTenTsP+jJXYjMRoE1Z
16tVOLoMOrP94UwNI+hfI6kF1CKYFBMhLvwmF4lXLwGKq5SIRoaJ3Ky0w3UJ
3MAx0unsF+AvRkBALALC0mG1lSEsLB6hofEitNZqoZUvCvUkzFWPglZFobF4
+ufevXv/nnnkOJtHan8Y332No1ND2H7iHjz7yBn0bTo1RzMkpWuCHCk/+eQT
fP7558ofmc9nYqCxAT11tYp4aU5BpTirOrjn8m+vh93u5BMQFGFHYu4QUku3
I718O7JKRhFvLRCvNRoWS5Z4OzrVS5BGHLS7YxgNdQMozckVSXMw6zcjqhLp
Jh2RAT4iXugsNjJRpdWsGBvLIlGR4Mmdd1J+QKnwnR1hbHLCpQmdStE2DTMm
ClbypvDf7TdpLIpab1+YpGLiary9P3EWi7prKIaHQT6/Q2VRbpwkPDKdgqn6
sDmuvc9OnpEeoS6dy3KGsFAx6T5Rjh1diXw2aDCMVmZtF/++sd6MkxvT9Zqj
uzQk9sheGhKLkqTHatw9noS9jSH0ODeqjzNZ6ceGJOJp9UvCNaC6kms1H+s5
rzvXqWkn9/4qb7ooZKZEc0DOhNXibU1nO4Tr9OQgCwdEuptnWo8d7Q6e9Vkg
Xec0ApIS6ceihizFaYzbbk1AdLgZpeXdLiaxWCccefmN4lQkwC/YBotdXBcl
0wJRO5BTtgnJqRUIDIzleOtwNPP3V1T2ySdFgJjBTLP89E+p0COe4dl796C5
d5oyCPG/v2K6pxknH3hWuyG+/sN7GGhrQWdfNwYmTsnWYsbK0Dy89fpraBHh
nOSbxPh0JkIi+dV4ET1ZUXEboqNT4BsYB0taM1LE2RDnQ0AhpWRKvGW2EnFO
KqaQW9QLkykDgQGCPyUXolLECf1S1NKNAdSVlrFCkJeaojE44zGpLuDmjp6S
ZOmYePAxOTVYiA67L9rsfpL85s59Q6TUk7GwOB/yJfLgQDB60m7RbW1u0Muc
Zp6TuqQI9TK/em2vAJnDiwUZkV3QUKguJb59IpczDdohLk/PfS6yjAnBlmgD
5V8eb1OOybWKG+IDgqJWRWOHICq0D+TAgA1PHi3BHx5s4iMhjoiH1NRB83dH
umMpP79Ol1KrsLslEhdHeBhD7ud4+6gdfexOnsBTtzqjOpmGRzfFYDBrOd7f
pRQ7F0tNHc+Mh3DvJUUUWsanRRQip13pK7GlIooEIZlUXRgROYEgVd3F8hXv
zkXoitRIZCUmuFiBck4GmJmbYYqK56tAYtwKD+lGfHwWfP3FWUishq14agay
ppFWth05JSOwJuYrUTguC4VF7ZjcemSW3Cl+6mfvPI3q8nJcfu5NvPns/eKn
1uPFdz/XjsDbItvIy7Chb+L0nO933PPA22+8jpwkq8iD7Ez6SA/SgEwnvKp6
AElJBfATDCrMXIykwi1ILdsh/qT25FN8TrIqtiEzrx0R4Va+H/MLmlxnYh6f
tlrxRjQ4G5EgTmBuio3eQjepfrWpoxljNVlIFUdivC6L3vkFUoH5ZF8WOxqp
8xoGFnz/EPWh3Mz2zq9Kn/HrU2auKN7dQUVGk0yrbu8MYouTz07qtOqT42nY
XO6rCOt3OuRkg7Z5Pncgx9XZqSQbtIGbVjTrZdL5PCl213QuHtxbgN/d18DN
moR9ulLmS6oq+QqdHODBj5k5xViZP17cK+cUbrzy4cGJBGwp3aCPhhJ92lvj
q2lmS6SaBnXC0/KQ16cMNwS5F1RbV3Mhf+YNMVgQhJLEIJE1lFIVSjysKh/1
lnLrWWOOGclRwegSDEMg3m3GJZEWbxEk3IoKEQurjWE1I6MCAeISCInMQlL+
CMGHLgkVQvMFfKZhF2E1u7AXMbHp8PWNQnvHiEia/3aVRJvPAh49fxwNtdWo
rmvE8bsfx3eA5jV54fAUxqbP/qBK+vqrr6O2slVN+jXEK/dTTEwqfIOSYRUM
KY0et3RaYF15XDqrxJryyjchyVYicosopIuXKF6q+vd4MHtsqGqCOTwUlXl5
JLLNkbFOjVdthUmwx/oLXi5VAN3ZnWuiMhZVSeup+HKdXKYVufWEyJQ18dRF
jiw42+zNvEAwIllienpTBIby1uhGs9dKFa5j7WFcCBXs3NXM5+CQfMeYVa4O
fPNgLS5szWBbtm+f6NBoDpkoKNhu07dV0/gzjUMqBuI1OrZFYH/nTBEGCn30
eUw5O6CJthOKCZHMgqhXeTh3FacD1EAlWwdQO09P+jJc7FK69RbpzWdK9bU9
7VZsq4pmnVQG+u76WOTGrscUKf+CCulA7yvltLk+Kxa26CB01zhJJ9WBrmbN
WSJYxkTGCaD3EdDdtE+cY+UgrNY8+AaYYc7oEtFd4ESBzTw9UNrLtwla5ERu
bim++uqr77VR/fpPX+GrP3+tshcVuN/9DV9/85cfhPbLL78p2FqfmqQrEZhO
YmlpF1M1/3BxEgsnxUnc7k6PRw86Lb7TVryVoU1P7KY/9zSTnpy8RoZ6hgJ1
Den019YUlyE2NBgtlRVqw6EL6Vs66lGeEs6TKxTGRTiZL2kUx/sLUJXoyVKS
HNUpobu9M1oti8VwDqh/wNsseHIsQpCdm/Wy2PVS1/r7ey289IycJqnzU8G8
G2P+gXEzpc4Gk0Nq29/TGi0rScpYOJmRiAh9pXWJFrif6MDh3njcvSnJBW7l
Knh0l1KS+OycrvrT4pgt5T5ad9ZiiXjRQunejGWM4d9Kdgd0ZAnrnanLlAbn
ra6XTaf8HNnPJK3Ru7MWapfgWKbIdagX2BODFTZmLUZcl6ImMxop0SHorK3B
SKtB06COQcqBSS6p1gnAPA6HRG5NpnR4BybBmjdK0ZsxI14gBW6CiwobdxU3
hJikoq3oHz/x4/oMCLZl5b38w+dKsC0sbOGcNTS2iJmT+LluKv1IYeK0A4n5
4/CPKcEavxTEpHdrd4wBxOXiiiEQ+0UhM9NpZCdDaBKMvTAzixsTehvqXXec
Bx/7iZZyEa390EWccMBF1g8PlGF7rYXtUZRr1fWpsB66KYU9DWg7uUv2cecV
IERNLipT3TpZ32fCLqcXznWG0P1+rRQLX9+nkPUPVFOdeVKX1TN70pRxEFWU
cZMS29/cVsLr1jQnk8X6To527Os084IgkZwulPJUUpCOd5B9doGru6CA0wSi
4NRS7RI1FSJypiOI11AIyu0iIrHYVbYGu8q9XHUsd55vGMhajQnqk9ukEm83
HkufFlHAEe/vKl2pKiY15qfGhqGbWrGM0C3LzkJYUCSKSzr4BnaTlDxi3cRH
/cMykeSYcGHUg/GRXLQFIfHVWO2bjLDERg7XKmLcYHVMonPs7NXV/r/X6S+B
WKsgCDLcCB+/WERaq9T4qv6ca/XzExpfi5XeSVgdWoKg+Aas8k5EoLlS/Ldt
9D0zgZyV08DR2G6vmgnk5vpeZCTEIT0hXplgaGl2k2p9w9XZIuv0xkRjDtdM
5ktZ58lBB7uAbiqPkIR9d1ZmjjcGs8HPSy5BnyLS6UZvLtyKyCXT67t6QhRn
KJdq+dGxNIwWe/Gajs/vLJBzTa2qpZRTjUOon95Trfj6XKhhoqHBl8qwu9pi
NEMPDb4kQ+5sJJ+3OFdhShk2eWpHMhcaPjqaoimQFItpX9P5dh/izTK9eGt7
JLrSluK+3jADvSD4XhmOgDNhJc70prCRiXbgL3FzVwbbvvaUymFY0SLbqAwb
HYqe+npJi1SAXFvkQGRwKAoKmmcAme7tTgQHmxESU2AExLUcgykIRtpaRfCz
4daAPARZW/j3PuEFSHRspsxyrgHWZ64u2MvjBvv27SMRUx83kGGtlJnaRU5o
QkRSozhMO5VzdR3D2VowjvXBWVgVkAWv5I0IzD2M8IKDiMraBM+gDPErE/G5
oxS3Z5KMrJw68bdGISu7msRYmVw117RyCyS1ZM6i0x1NaMpLQI45gIckBdHQ
3/b+MuxtsnK9ioxvjB+XHZdHk5hk0Fr1mR/z46PK6g6q3X4okQxSHZ+biOTN
cVqB51oJUXsbgsRdYJaJ9ae3F2DcGYyXjuRREulm6OdyYntLLJ49Vsxi+6JZ
fYzR+s70a2ekihvL/dlN6HM9VVQ7fHqjeP/1ZydSZTr9zv4kdj4m2ynZGoPd
ZvsCuJHmpUmT63zHYW+lNzqyfKmccZ3UbHCgyYJcs49IXvjdVgblidMJskHj
fhmxAagtLuJPyl26RqnZ1hQWhix7JX3AhnBdKRh1aJgVIaYiY5RcpEdJv6gi
rPAWyLb2IDDnAILzjyAqdxq+EQUijNtgyuwlUssnQoW6rn48/fTTuHDhgizF
y3H8kUcewZdffjlH27OqAV5/stB4BJvLxaPsmKOg/Xr+UXE5w1gtDtuqsEoE
Zu9HkEA7PVqA+BWUewjR+bvgF1spgrsVkcmtyi0zV8I9pb6Z2bXMS6iqoIdz
NY0sdyI2JFhXZ7VYTrifaK9DcVIIqjOiuefMwKwHitkod6gwRGLWSizfKVjp
RpEVvjal38qvCT4ykr0c9yoO9losf3+fhVvQn9lqolh+rX7jp+P2/miekfij
qwJFJGFfaxTum0yRZ8G0uuxFNtzqWCT1JfzxkRZM1kcqrqX3V0kAr8L754rR
7/BWXQRdAKf6KnVm8tLyU2mLpR5H2rk6IAD+/q6YJRK2p4pXa4K5dr5fph5d
8d2tKbdiT4NFvEV2/S0igDem+6I5L45C+HypwjdQnoKk6HBuW3U1YLtzBM9J
ToTZlIKqGj07IvCQUuwfbleYa8m0nlgRppNEdN4gQuVy/zz4Z+5CUN7huYQe
EXADCTsFexGS2Cywk4AgSxXH/sTCrQRrDdVffPEFPv30U5w8efKH5yPmG8I4
iRwBETn0OIYHIz4dk96FW8VP9TS18JME5hx05webL4B9EGH5Ipyn9TF1olsm
qXBC0jNVXJeruPbXcT1XUgMb6gdRnpcvokEIukRKMoNvjzeXs6VCT2my4Ro9
2EeTFmkoMa9md0QqiC+UpEAaSWtKvIXd816d0rInJZadbPDCdLGBq3ws+Pa+
Gh/26JP4Ng08PrbFIjiqHz6lgUedb4v4ercyYUFLieQy64M1uLgtE0cGrBTG
ZZDTho5NNWF4/Xg+90Req2eZVbyhY0ulP7mey1T7w9M5GHZ44nlBschPRqba
x5sDsK9yLb0MF2FRFrt0pS7FA+JYvKrfZO5cG7i7KxjViWvIPFaWQqgJMt/s
hR3thYYorml+efEhKMvJmdUcUl9ShPDgSC4SuvJFD24oTEwqgJfIFVkoLjWi
Kj5nhInJahErA0SsFMhSUOXGqAoVqIqwjwsUrQuilMBkH0LX+Lkf5uAzrS0J
5eUVfSylewXatGdx08i/4Eyh1jos25AIz/h+OnVzlOe4VjzSQQ7f2q/gvEOI
ytkmUJ7PyWV83hghfSbQM+w1DHS1qWCu+oYQH6eKqT05GalxFilkKGR80GlH
erS3YlbUV6p/ANQ0fGywBJ05gejNC2Qy7soqM/HQpgyM5a3lfb1viKxSZi2P
DgvWkqHVgWJ11kK68MW+UGwtW89kgLpsr5GGFd86mKJMFJ1Uwu08VXomu7SH
p2zkbCA+GcFYNPLC8N2fw0tCiKhQ78wNUpMkrZocqQzRp86vlXqN7xhLUOwy
79QpOhV5aNNOv8gwPzicLPMW8TtMFK/DPd3KVmN9XHlfLB4eDOTesZcmDXnJ
eL4ye69nmO7sHdaQ5oN2R7wc1g+pzWJJkUHcJi6X6ZSmqwZYIiORnl4mURZF
+aJqurefCXG5Ixz/XLFdhNC0TqzwSsTamEYVVQfdGGTXGEBGPIFjff5OBJgq
sHRdHGo6dv0Y5UR87q++xZqGl59ZPMuY9CwefPAikpqxzCsZ3rbNRFDmKA/i
rj+IX+ZO8XX1+ebRb/nu8Yt1MrchRj8D9MTWE5OLEBaewE1BIgxooBf3H7N1
arVsNNSU3THe3shRpbfMZvgMRFSfbrCxFHj3cAbuHdOiupZ8BrE74CvbzC7C
Esvk/OmNIUTIr5dKfiR60zq+mX2VHx5L5c20L+9LI7TpAooA3ov7M5Q+rwtG
GyQqSL56oogrnl8+pAymyxinboHBMmUv5R9nYPxgZ4xIOaT+SDfW1x+ZTKCx
U8k8JpUHoQeyb+UV3bSwa7GUSh+p8cS2Yk+K47ICSp0xtDTwcJuVe43kS/BM
b7KI6y6JW47rFFoc1hAU2zMMnwyFo+LMDERF0gc66Eq9KP3s4MplbHonoUlH
FtFfS1a/Ekfj+ijRm6PFc6K+fhk7CGEq2jzoBCCsQJCJ+GZYBHa+/PKLHxPX
3RnmoWFJCLPWI7V8h+E57P1YLp5jQ+KoiOdH5uhQFvHb374by4Kd+MXaFGxI
3kTxXvvPCpk5AJ/IYmwIzUFqCafTc9W/mZCeX76Ru35sKcV8/l2xXSExjowM
pFrMNNpnoIP1uVZWvUXiL2ssR3rz2a3mWLvVQF4oGb27P56TUW0wd5EumZkx
WbiKZQrBZ2Wx+82dJp4BoRmJ3+lit6C9R1N4Du+RSSu1vMyTGnTfU91APj5f
auwhFkklbVRR1gQ3sdoiY51MvPpLg9Q/59SxTn9uR30oHtgcTz9KzkNpLn5f
rZSHKgnyKzvj0aPI30ukUL45/1a28NBpuiIW3i8urSrrahK9XUlMlmBNUWzG
JRKf+VI3y5bGbCRSKG9omBXKexvqEBMagdz8BkljcedQbrZkwS/SQcqgiyds
R6IIfSt9BGWJaTfiiknxAayMasEv1iRjjbnHhSt3jur+WXtR1XmQ6pg/CuUv
v/ImUvL6kFy0VcaiJq2s9EnG6thOZi1uarIgfg8vgexfbcjCT71zsSyyCb9c
ly6+xvFeO4ABFNfztmONf4o4Qw2zDnNmbhO365Rxp8+QDnVqDGiqbuYJrCal
JdLVydLaip7KPORYFO1F5J4GGnN8oAQd2YEYKQo1aoob09CRdivu7Ah2hTZq
Eo7DsdoN2FOxhqKfxtM/2GvCcP5qPD1pIjDpaovI+Q6IHPROpQ1dRvknZwt4
GTb5aH41o1P+03uqeG/QJ/fWGVBOHe8fX6rDQGmgvmPgGul0kNvShDNQ96aQ
oX6gOZR96f6gVXrceH704bEobMx1mUUq0qJSudTmBRZJvGWf0xtdOf4UGrR3
ikZGa2w+bHKo35nunI52l9qQYjbRJIMcespzszmOi89NZisORxPW+1kQzwxh
es5ciQ9Tark+2I5bQ50CLIcMyPJN345f+xXi5xvsWCHOwS8907HG0uvCuxu8
M/aiYuDij5XLxddefBcJjkl+Eu15Uvh5tmJ9UCZWhVcRPVKf51qmJqtjOjiW
/4JiuqUfv07aiBXRrYx4b9sEnU0N8fT4ken9rL7E5YxISaoS3HMqtiA2OgUJ
1rzZwb1uEHlpaTzeOlN5GWuuhj3WV/CCbEl5UT6Uqdo4Hn+6ZzRDVhSmSr35
HhcIl0t6d3cGsA3PB+q61+u1NTDi37dXbMAFARK9bUtRXm7rjuRx5a9c+uJn
5/J47IkGnv90uVoGO7Wub6qNxFu3V/JYyA1Sezstzhopc1mkXyPJL+xWXuyL
3xzKYKezRdLh+gNv1vDTt4obKkHiEOwuX+NKT5W9o89tDmVPy+c2x8qVoN7M
1VrxV7sJaZ6flrPnmrzYefFgn+v9pe6IyvQY5Kelu/qHPLg4T10olqhoZCi1
PUPwIrkjPCIJIXFV3L0kZ4L+0cVY5p+nJqQH58zVkXYY6xOGcZNnGn7m5xBI
68F/J45hdfygQFoa1sb1u/Qa5QSU91/4YWV9enoajz32mPEEFGwxnAAKxn7R
JeK58mc813X8XGvN3fiZOAE/j2rGTeJ5bhLP8gtLHz/fCnEj0Qn1TtlCJ2Gu
znQOsAofYHKyAk8HPqXUcLuk53VyTyO1QtNYlesdpMatJhH+Q9DmrGSmM1c/
DM2C1DehODmC5wGpNirLkIe7c3nJ4cmuJAPdoRNxvDmS9wFQt8bL28wCmS56
S9o7DcFpLfHasfhEMOJjTf443OCPz0+5jgVNbA9HY5eySVMXJO8swP42RZD8
s0uroRHq6RaByePF1NMun4i37qjkdXPaZkv5RHxwrhh9Dm8WZ0gRWiSVsGjy
erx4A57cEsuzuPKJ2FXljdsavSh1lRV3Gl0lD2vxcuX8nTaC1ySuwqnuZG7S
WihxHloAVJocSiFH6xciTp8TF4LqwkL5RBCdJzuo8FCT1pOlf57clJVZAa+A
JCQVTXJ301wpWwxPbMAtXmnwzaCs8JCKvEXMpb2SNuHnAnn/X2iVirwBRt5/
Cd69On5AnIxUrBNfo8BL/Eg9FroK/+ijj9LNgN27d19Nr6T//Mc//lGfgJIO
B/+ig2HO7MOv16fAR1xLrsfzYErmk7oNPxV07Kfh9bgpgZQpcTQE+rXDsTxK
ECPPDHinbnUdDg9OA2Lyd2BdYDqC46qZGMnXRGb5FGJMdlgsdkNsoWuiXlwT
9qQk5NiSpcKUklC1FtpQnBQqc1SWdgQnasn0x3hZhEHaoTmRe4asaE66BVdG
ImfkAEq++4yc77rzDPTd3UFqsTVFEzBJDL+y2YRJkhZnCJjkO32yz4wzw/Fk
NOSu0/gq7O+J48b1b12HghoYX6cpqGrXFJR2KGi32EtHcrGx1OULt0huGDiZ
zeYvb+whD7YU8d9cmg6NA24uWov71ZW5cspL6ubOsg1yIqCsNg9mbyLxNsma
DnGjnhKDnkCdRtTWnhwjLp0qp+Qy5KG368ZFUh9IueHzJL2+oqIH/oFxiErt
5LtCZkmJBZuwwjtZzTIPS9ATgTVrD34p+Pe/B5Tglzr0FuinY5XIkG9amwrP
hCE+HerJ0A6GQD2bHf1DQv6C2QejdAqewdlYGdlMecpcKeUOzD6A//Z14N98
CujIiq/KZ2JUJAuNfCbo7EhngotX9hGmThb7AB09iaGJZMHRK26LaBQWtRpu
C1IMnCU1LP/0KM2ihveefDvJcG6XKjrL1GmL04z6VC/qZ9I+4QfH09BnX4XT
ikeFnBJvnZ0Sf7wvBo+OhLIzjWZ0eo3eeJWCF3cmYrTYBx/dljNL1D+/0YpD
nbHGylUNTgwn4e6tmVS5cp2Jdrx0UtkepbXcXCO1jl3Zkc4j4J/dni+rm7Tl
gfpnfnvYtniG+DNI4o9ysnWnPGKAH4l0YTzvVu3Vz5HTpK3F6zHgYJs4Wfy5
vd+GfIsPtjblGd5fSo4319lhjQqlYSj5oqB8ITIinnQf6aJQ8uLUtFJ4hWS4
qrHufEcEmsqxLLCEuLnMtVeE1eLfBZZ+IQiJuBrENxPW5iuHwDqCVSJtuElc
EZ7i9772A3w1/Ail04h7ugxi0jpwi3cG/Ox7iCXJB5JS83+jNIF+pniWefws
17twH9HACbNyFxzU1VgqbgXFNWBdQLpexZirvvTM8mnExedyn7IkeXIDWV0f
UiwmblAYMTCkFoy31SHH7I/R6gxD0w0xpAOd2SwInRb3/X3yfS8Y0h6nL497
SMCnFRfH6jZgdxknyLIW9Np0DC/TenufsphMVvffO5zKi5LeYz+QAtdlcL4A
D2xJZONWfRmK0vFLBmAnR2wG4NMw1LEiTNEA4CUj8Klx7PzGRBxsCSUKtlhq
KX5sm5VHUD4+liLLQK/uSmAZ6O3tkXLnDRUuOD2mQpa4AW+QGnvJy707YxV2
1SvL0hZKCdb+JosAvh/dsAukmELT/NQPpZMjJUdOMpmRll4uV2so+tPMX2CI
FZEp7YaIR/yYylfLvVM0Zm0gHhTV/3V1In4m8uOb4jWwzdOj/0pzD34lWPkq
yyAlyj8C8/MNmKcpEaLw/+1bwM+gK/k5+/Er7xz8i/gZIsgv4B/vwefwv8VZ
+4VIWFZGNRsurYCcQwjJmsatXvEw2fsNuTH1xyVltXHbpTLP4ooKNMfhsOew
J89oe5v+ptJIcWIEupTBWJnvHO0vRlmCp4BGnJQEKJT2kIBSn/1W+nTl4Ham
2ReTjlvlWu07e8y8Y+MNZQJIRvdvj6aiN3+97sqt1a4I3VemkrG9MWKOtrFS
UB7cPZHKC8x1aLsztGlzgfhWMg+RhfwzQ/E41hHuaodUpggvb4oj2qUbHJAY
RT3HvSJteX939GKpW4jM09pTluLR0SgjpMUtRhuPGm1rcICdZe0GSG+tolUT
ISTiy5CuzbYgOzlZhvRwa7PIfk1Iz6icCWmS79f5xsOSPTzjM96OaAHz//RM
hX/WHinJ9ECwiOVLQ5z4f26NYwhT/JyvYYohPYb/FEnxL70LCc7/P2K4YNfU
o/ZrrwysNvW4MmolhvumT+M/1tjwLwLaxF20Z6DU46eB5fhPrxz4243PTgny
hthGbAjJ1nsbtPhNuA6JyUd8fLar0Ua96BKS4EhPkyFNkkJ6TCD76x5y9ZQR
u9zTno0KEbXP9tkMWRp9altK/dnaVVd6FPq6v9ITh6vXyVGbrvcXJiN5ROM9
1QxXpitvcM/vBnxwagZducuBu0fjsJ9WPcumy9U4M5qEMxtTZ0bt50Wqu63e
NbYt05V7t9iwpzGYfoLcL0luoyP5a/HRUQNdoUpDb8Zygelwri/LdIWtazKX
467OEJo9MiibgznrsKksUqYrpPRQmpsVS2ZiDmkcw40b9wadGUiIijKUVUqy
cxEenuTSNunTG0KUORc+kUUEahdJVajxmuB8LA2tlWLfPJ2w3+Sdi38S+LpJ
wfg8CV4/E/Tg54Kvr0uc+LEQn523UtiOTqMrIBU+aVOzrg/f9Cn8++ok/C/v
PImj04OI9Dq6DT9ba8OGpHFD/ObirX07tw2rjW8GCTUpbwR+AbEqQXel/9RV
VlTUjJjQUHRUz06O+mrLmaBrs8KuT6QcY84k1KVs0OdRafCgK30VzrQEyio+
xbWNgraSsYuI4Hqyui8WDwwEY6JkPeV/crL61LY4bCqnndAzklURaE/2xuK2
QUpWDZ571FJ2aTqLZknlZJW21eirEe41JqtP71O7ba6SrL5z1I4ewZ7eO6DM
kcvJ6geHbRjNW6kb0uvJqsi+txWtxgFlQYIM92P1/mhM9yGMy0J+WaIfestS
ZiarO0WymhRF8pqTbYnmSsImVa3CQ6J50Hpmskpt72t8EmDOGpxFVxLyNnJs
90wYuQr29+IXXtn4J8GPVey7q4yFIPdzUxd+ui4Fa0zd8LEfJJr+I+zur65p
0hHwjanAf/kWqs0PUsimfFjQ8H9eYcH/8snnI+AmHwFB4f+DKExkk1T3Eqwn
7xDWhJXDOzzf1RTtzq+e7D5IrhHpqBYnqsQbFm9KRH5aiuvqdGN3oIaiPBTE
BxpGfduzg7l78vImQ351Z38SmpKW4jFxl78i3eUvq3d5Z8rNvJrzI733RhmF
ONvqi11Ob1nMpzaz+0djOFf8/e0Fchvl3rYonsyQduB8eW8V9nZZ8Oj+Almg
IdWSlmVvcgbj9zNUS+qGpwLYoMMLn6hL42XV8sPTudxm9sKMNjM6kZPFa3nR
lN4Xr4hNNGg4nr+Gmutk1fKhwQhUJKzGnYNps1TL0ZJIlKZE0Ju6QEpG7XGR
qMjNlVkMrUHITklFbGyGfD0T4zQlOLAhNA8pZdvlzpawpFbc5JmpdbBIID/E
AZ5qVv8kovhNSkI6V81HSba8KUFwCAG2WwJKOJZ6Z+6T89HZ28I09f7RRx/9
++q9CL7JRZNY7puJW6PbJcRex8TKS+QP/7zCjP/XVxAoVb3nGyemDf8h+PvN
/kUIyN5nyGfzjojD2IFVPsmw5o8by3fibbBkdrFNSVlZ16xY7yho4J7rnvpa
qSKu3KaVWeImTY/CkYEKQ07aXcDGWsc65CYFZUhnfwNtJL1VN7Ahk+CHB8Mw
aF/KlU1Zraeq/oHaGe2Viih+rjtcsVg3lm0/PZuPydpQPHMgmxuCDQZngrtM
NcfgxZOlvBpePgHkojpaEYTPLsyuZNFYSb84Ae8dt3PuK1eyPj2Xj4lyb67T
GitZ6ThY58vdN5/s00/AR+o+xTbbMipbGGZOtyg8fl9TnJSaKiSH1vASyVHd
fl2a10A5Dy6kWxMM5SxxC7dXVSEoMBpFxe3GYowIYuXlXfD0S0BshnLru+Le
NNaFFeKWoApXEfQGPgtEkn+23o5/FlFUPQuMuZ+bu/Fv4iL4lTgnviyoH5yp
TF65cgW33Xbb90n2jz32GA2yGk6E1bFVav5UYr/JPoBfiTvFy7ZFylyv5ROx
wTaBf1puwr/6FZJ2iv8Qqce/rUzAirA6vqoMyo64KpJGscLLCjNlrTLroe6d
4q0ICElEamqJ1K5HNoYjSIpPQV6qzWXe4M5vdb41Aj0lSRSf5kvxf7w6GTXJ
63HPiKGOu7lwA4vTErt/nRNWf9oKSWF/kWrotaXEE5dHoiRvVsWV5mhbGE53
Rxk6FqiEJAL0VTsWtGGP6nDFLHhGxwJNRg2VBVyliOvEp+qQyEt7Uw1DIkSr
dtb443xPiNxDTyOyF/rCeKJL8HlZlXljKgKdqUvxyLCxLkHJ+phIvbnZclxv
trw0moGSBF8MVKa53lcP3lNBixsTIsPQp1oDywWT1IRkmONyZgb/xORieAZn
EaWXS7dJjk24eUMq1lj6pSB7vQr43fiZZwb+WfwiCvHTyEb8m2DYN4to75ux
Q2/gmVmgElCmBRc4derUD+vw7nj2pXcRktCoNRfIV5O/pQa/8nYVlrWWCgL8
+uRNTHb+RTwRaTNaUUCQIkM5K20blnslIzK5ZUazzk5EJ9ZyX5rSfTlooIXl
ZR3srtyu1j60wMI7tFrqkBHjr7QuGPThcrRmBWmNOtoHSQvompNX4J6eUJnh
K+u/PHHImNBqSWBf1kq2Jf5Uakmj5HFXXSDuHY8ztKTRpl3BQwZK/FWzGSP0
P7m7EoOVofj0/vpZLWm/u7ce/cV++N1dM6AvbosvLjmxrSYYj261Sj7ZSu58
ojNCWe7m6ktjZ75Jk0hfl9Jiba0v7YM90RjMWoHbxQ0ml6MEYbqtOQi1KV4y
wye2M+AIQ2V6tIHh95ZiT2cRbDGhaCwrN3ST8AqAykoEBcUINt9p7LuiclRl
LzYEJCIypcPQukAIoxLVrzzT1MAtNe0Q/jN3Mf7/aZWVWwWogY2qQUFaKFVa
FioHL/1w087sPsy3ERWbDc+QHNcw9jw+kikl27AyMFsk3NWqqH5Qrg9Q++VS
cTuRruM6sR7cAU2BfplvFoJMldKtphbeHBPwDbBogyRS594I0pLtSI+3UFuC
PmomKH1rmbLYiJYayb31B0QI2t+l8BtyhjZMkIxn4Wx3HF/vTxtsB5Scdjxv
BU9I6TmtIuG8MiUQn7Oa5+vkhuOPjqRgU5kvT55+4Rp55U73w1kYqwyetXeV
GPvb50qxscY1sW1owrzciP6SQN15+FopId7dHK6ssXbNvVKcv7TRzA6Cv9PU
d5eOQzI77cn+cEbP8XSJ4DFVPgRzw9yruMwqE9bgXL9x7pUozpE2q3i7fbFH
mb80dOwUJkehyG6X711Si21xVsQn5MmTniTEpWVWYZWvTR3ZNpIIr6hy/JdP
AdNjF+JVDUVE9aVB5dwIaWhyF6j3FdzeVjKBP/7xyx+BdEHsXn6TT+WGgCSE
GDomFbiT78BynzQB9xqO8zrrcmP002MF6aFf6Uamx1zhnwuvsFxBVqZcgykK
fwuOzEZMbNpMb5FiEmxClP4buW17uKUJ2XGh7PA8g79sdCahNpmdc138JQtT
IpbS+nRDP7kZz22OuUrqqlpSjIRivMgTHx62GcdGVGmkL98TvzmcIUnvii3B
U7tSDTVSN0mKefGYA1ubYvDHh6/SVf9AI4YqgvH+WePkyDcP1uBYdyxO90Rd
fbh7LclK8uTIB4eSMZK3Eo/NEG6o/+ZcizdGc9e4fNTc8bK4AFpS12BHnUnu
SCPh5sJwGhwWH4zW2KWBNEWP7yy2ITE2xuCTLvDdXFGOIO6nksekFIPh0MgM
bjpPK9su44naHtcG5+O/fQtFSrtLbrEMZDwdnAV7f3EUlgaWIT6lRJCXP86O
5OJCUP73V3z8wXt4/7cf46/fKV/5yzd/xhdffqV5znzx2Sd455338Mc//2XO
95uGqfNVlX3IzW/Cau94RNja5Glu7lTO34hVgTn4T+98eFNjQd7h+fohoBR4
kluMqPlOsQ+RprNEVh9XjpAQi156cletbCwx8WxbaxxIU2J9loj1tHTZZV5F
zVElqLb58Mqwy5pdqBvaU1cJvh7AhhzaGSBCc093MIazluHdXVGGXkxCy23N
vtzIJVvoCcrw0q5EXlf30enZCv2F0TjsbZtlLPPwTjuvRfz60TZ5SpCi/Zgz
VKH/hilBVaFvCHEp9KpkeczO1pdv7bMy09K5zMlUTJd54myTFz27bIf37Hgo
WpOXzrrdiMYfqPZDY5qv3HtMwzc9BeEosUXINJ6GA2ntsDUyBOW5OdKYtxsj
PyPRhshI2xx5DrakTCSt/olcvXTNJ81XYT+J9eHF+OX6DKxPHFOHqA+Kd9OF
czoONCe72j8DrUPHZ+vxIoTgu29wdv8kqkXyXFVVjcn9Z/CtwPpzl0+gc2ga
ZM39/APnUCP+W1NTPWpbh/Cbjz7/fi8l18Qs3UzZufVYJfBOxWHXTTBfHU/f
Bj9TFX5Jsqm5V8kvROh31wUpwfqtI7h5fTJC4muUCXcJ9TR0m9ICP/9Y9qF0
kRwPTl2zMkt4IxVZgLusI9yJ9MCRFIXmPAtdtYahtqpkVFhVW5oxly3NfeIS
ON4eL1j9shneSmRLY8GeCk8ccK51+ay6cwK4rdSTR65/fzJVdhG7byQaU7Tu
4I58Tb+kAe2DHdG4S7FJko2Ibx9Lwgmlp0DGPllvb64NxytH8wxj4DR+RV8b
KfbGJ2dzjZq96jU/Tl7zm8hr3nUAOI/tD8OESMg/1DYruGHAvhxn1V2V8t33
+Fg0KuJXi7RgBh0Ucf92wXWyYrwEIbMbWje4GlKTCVNoEPP3mZx+oKkB0RFm
JCQWGD5MNhot6YCnXyICLVWuo6CYcHkgLKkZ/+lpw/KIJq7qU7x3l0InRVPu
eIkqQkL+BLo2nrvaUfj49SuoqajFC+/9Ab9//3lUVzjx5Bsf409/+BhvvvMB
W6c2VTtx7uGXxDXwFfZv7sLI7rvko/Dqq68a6rYzPUDIrYOOA6Uhevi/QfVk
2o7o9G7814Z03BJcydm41hKqngysM3di+QYromyt9Mdla574rH74+kUjJ7fO
eBLIn6+8AzGhYagvKTbcA7SYubO6AmnRvrPaxqnEUm3z5d18Um5LcuamIl8u
1gpMaCyAnLd605fisZEgipnXy70pNLphX4nXdiXgd7KsIwB3vC0UxzoiZg+i
5GFjZZBm/THHVcKqEvl3Au7bkU27H2Xbj68eacHWhig8RyLoDNuPT86XYaDQ
W2910NPa84XYWx/IRkyy25i4o+hhe8TrIQnHyPRjeSMUzcpKVhA8PluwAf0F
wfL4rKLkR/Dw4EGtxcaD+T2NusVHRenBSXa5bWMdkyxdamadg8LidqzzsyLY
UsO5rbtq95HGvhojWOGXxWyfK6YSeiiWeiWOYukGm/izTr5CSHIku49Z6r34
ylN3TaFtaB+Hf3FHYPdQG0498KxmQ/+7155AeWk9fvPpX/kbnrrnIDqGj8g2
9Dt27KC/ksTOuWqJ92pGT/asGqzyikN0Wpf0ghbyC6Le0bUhDvxqvV2ZQqQ2
heTNuMW/QKQ7KTBn9vKlIp8DehM2+MUpYmadtuzBjatYZERI+PfQaydtKElP
QGVapEFxo4VG9ZnsnUrbAgy2cSK+3T2Uhsak5bxl4FXZAEfcBPf1hmAwcyne
2RllMMChnOCR4VCMOdbhwyNKTrBQHXqaqvLHgxPxPB/lSnjz8e6JHAyW+OvT
gzOF/a2N0XhJFfaXqAaT21ti8PjuTLa+0Qyevrgkvrc6CI9usxqmDUWie4/I
fcnTVRD/xVJz0Ee0eCRf2cL7iWQiyVMpm0J54vLpccUdWGaD9/aGo0x/1+wz
zPbS2DlhsDJ9hj0Fme2VIM8awUP/coAifbPK4eCmWarbkqpvOA1FbVjrmyDY
QZ0EnhsYPKRqe0c7uXdyLamdAjw+6dNYHlKBFV5JiExuVo1vpjTjG81gm+R7
gu7tt9/O/9y1a9cMK/ouyYr+5CbJin4ePn3jGVRkZ2LzvtN46YUn0Vlbgh0n
H9SOBt0STzzxBBnn4E9/+pOWR8w28lNeYWZWNV8Y0ek90iu8hs8vNTCHJDRw
xfpmvwIs86KzXYVk7sDcbjgWpoxerPe1IDnZIaUHyuR5otWOxOhIfWui7FtR
lZeN1EgfTDbmzGiNFTEsIwRDDkPEo+vhgAjDir2AabEkfe5zrsdeGlF0cSQq
jp5s9uN5VylJ0LLQ/vz13AA8M0fWrfsuGnNkKlx9LGL9kEjXtTlFzfuMVP89
7SY8NJ0me5/x0Hl7FM71R8tLeUhuemV/Gntiaj3R8njKwTofHK6mMpdOlAbt
K0TCE2hwLCSi1Ja2Flud0a43yUMZxXXGINcSyF59rvRLUftpSWB8eBCcBfmz
TkN2Sjo7Pjmr+mf4iAzDUdiqnIaEelYDNawQuyCkR6V24b83pOHXvvlY6pUi
6FAxrHkbDfUgye2M/ycgSu/vl1/+XyTvS4fgrq3oGDkINSvGweF2HHatKfnk
tSfRUleN7dun0drUhO2HzuGLr/8qUyVjWcB4AuRXlWF38gmgyl2abqM9j8+4
2T6AQFMFEvLGNNNhdyknjrY1w8cvRjNglXKrEaTaHDCHh7ORjrzVijhRuTi+
BPrxWiOB5RmJ1lwUW9awk47LENqdBb3+7LXc1/KGixjRoFZfxlJcGWI1Ratz
kTUH7ZsmBi739Dy/PYGZ+8czm+1FbnzPxnjsaY1yOWorqtCrJwuxuV7dxSOb
VT7RjgPdFlwSibDBrLIGFzcnq0sV8hcbWhtyMFTgiefFaf1sRmvDIxuj2Ozq
t3t0nZ9Uz1MNXsoKXI0LuTEXOtkUiOrkDeq+cXdcHE1HUbw+c7JASgiGnekw
i4SgvbpqVkIw1NyE+BgLzJZsw+fHBpGOFqzxiUeotXEGLHZwG0+AqVwpdvJG
AoNJZdfG7zGpFH/8nWfvhrOmFR989R2+++q3aBXwuPz8uxquv/nsHbTV1+Ch
F99nWnTl/Anc/dgrP+QsTM468tIc0uCp7ZQje1o3n85UDbtuHNzTlC+ph3ie
mvxuR1hcFTvL5+YRzx9xvSUjsGeWX0XwVCBdlJGKdEHzadbfZcaq0Pza9GD0
5AbIhnVkJnyyMx4NiUvVNQBmA8253B+Kgcxb8PYOI80hwv+b7dHoyVqJ13cb
/RUocN7VH4mddUEUxuUVhCd6YnF6MF42hyIN55Fdduxm2adVM9CmxoU/05pY
XqGgbUbowJG+eG6l/8ZloE3ZwouHcgyjVrIh2pTTV2TnYa5iriLyv7U3kUX+
l7aEScYi6uKUzWGC+9zCA4cy96FA/+zmGFRbV+EgNyW7fJkp2u+oM7MVLott
0i4ROgDVWfGwxsRou7YNB4CCU0hQtNqiPKLix111SG3iAyCi/dyZ4FHwpN0A
ikqeWLgN9T1X91YQ+Prb119gpyD5zV396OtsRtvIbnz59Xca8AnsD5w5AKez
AVs2j6GkpAL3P/f29wNf3H6vvCmy0QaUlnW57K/nqaOUVVgjsO8XU6atR9C8
4tkhmfLZ3FH4huciKDiO2zGNXvEjyM2tQnRw8Ay7HMHseU1XMjJj/Hgl9CFp
TQiRlu7iBJTHr9O7ruT43Zu1npfnvT5tkQ0dDzjX83j2J64tOtSW+dBQCMYL
Pblk5do1rzS1720MVhwWzjsWqP3vO5si8NB2dl6dI+s758aStJkRNxXkX11p
0c1bF0ou8adGkrmzXg/nCuf5UG3Weee43Zjf3l2I092Rmq3lEr2onILNjjW4
2Onr2iqoTM8PZa1QNxvHuVbQWTBV5oP2LAoQWa4u1gyUJ/qj1WGlTih57cQB
QWmy48KQk5Iyi76QAhHgH6XuutAAPV8FdDPW+ydhQ2guR/A0denBAonGclEz
qgi2tMLvUe49FJX+qz/gkcuXcO+DV/CHP2nSvDvD+PXnn8D583fjpd988ENr
PjKS0hAYZOE8Y+bj0tbAkIh0rPKzITy5VR0p3D5H29FAZ5D8SIiRb/A1Iy4u
m3fDUQ5jAHJOFaIEkGsKHbNc42oL8zhqb2/NZ88E+W3e1pSNQtMaHGlLMMQb
0iqPtcXxTiglTpkNcYoaLnvSb1EWBOyL1ZfefLLPhCP13jhY7zfTR5sKVkOF
G/Ci2hIjazUfUQtOaQDeOKm04BhmY7vj8MCuXFqAYFjV/VCTYCxReGhPLtu3
ynH8zol0nOizkIazcEa6SzYiT0wnGToTlDpWEkby11DTvbwm7VRrIHaWrqbX
KO/5oBLWgH0VFWcXy3Xa4UiUWVYpQ/eGPR92HGiOR0aUlwgnxk+BOjLJLyo+
PBjFSq3WBfZ2cedm5cHfP1qvwMvooR6FaHOuYABWBMfVipxtUloSM09zc0VE
cgv3rYWGJ2N4ePpHdNTP07f+5aVnigw6CnZFT5I391XXDAqCXQWfwESsDcwk
ks0rGDhrmFLJ0w7EiuyTHArDwpP0pVRyFsnsw17BOK4tKpzBPpSdg2nRARio
SGH2ofUMU2OB0xaAYaVnWA8uIhZ3ZXqyV58Ui18Tsfg2kS6OZi/DB7tjbpAa
aN7dFY2BnFXsZirv9f69iMVPTFp4gPWTM7mzXOCf3pWK8aoQvfN9ZgPZWHU4
3rrTyQ1kCoTnKS4gF+vQ54xCf3kYHt2fTw3116go/vbJdhYoyctMHAJ5z/Hh
ziiRN0S5RBiFd//2VA4G8tYy73a1FCtrD17aHofuWUKkSrSmIrkwdaknzLj5
Tfw9W0u9uTB178ZMeeNxb34Y8uKDsI/Szr5SAxGZaMhBXHggSrOzDUvNKBA5
CwoQFBCBZFsxK8p6qX2JvtQsN78RgWEpbPkYYdPjofpPZedAXPYIvEOyREyN
Q7ZAI+2fnJw8/GMGRVywpvukutDB0E5MLprrWqJzPT8QWRBbk0tFSmyFV3gB
4nJGmUFbsofgF5kPH38z0tJKeV8THU8PKShnpJcgmi2Ei7lSpymG7J+fYWWH
AhGE5a2Jg+XJKBPEgpZKkBmfHI+PtFjQlEia2Yy9HOriFbLtuK+H1xrMkSnG
I8MhGM5fq3QRSF0yn51Iw9GWEE07ny8pemcHRBjvNrlGstUs8UQhNtVF4gvV
kW+uunyJtopdOVCI/YIOvX5XAwbKQ/DEIQfFYgEOFdEik3zsQAEPvH5xyakg
2o1Hp67sTKd9fHNulEZXd9UF4KwiqS+R+tg2F67B+XZvmXJQj/+R2vUYkVsK
lFvqmU2xqEpYjR21JsPtRvor9ddV2XxRlRFLM0+GfaKb6rNhCQ1EudIlr4N4
lJe5VyA8JAYmcybv6NDxskgJaNVKLKTGlXVBdqVVmIdCxhEkskkvXxPvqqFt
R8qiV95f+ffBaywIGfcTs92F04mosGjExGZof7EaVa9lG55adRWaKS4fawSQ
NwSlwyfAgrj4bPa41/dvavF8GCk2h0gDQ9FcXkY/wkPKAtud5Yqa15Q7ixiT
wldgWou96rawG9TKf1vaGhys9jYIW69NxeGO1gAM2ZfSBrlFUtF/f40XjggK
KVOIYwqFGHRs4FUGtKVLg+zv7xC4Esz4galU2UWAouXFyVQc7E3AX55oU4jx
Akbi6fFM3LU9H3h9I165vVaE4BA8c7SIkDxfMht44YTRbEAmzGQ9yYT5mEyY
3Xgd7IOb47G5eJ2+Moy6jylL3Ji3nILuEqmHjTbpkO/IzCXmvFyqNQil8Wtx
3pBmKIn12b4UZMd6aWqIAb/jtVmM38r8/BlBWHGCt8bGsf6nLhv1uEq8o/LQ
egETL/84RMeko7BQv7Dlvat/f2hjampKGtq4dtZueXoeutUTzQkICU3Qlolo
++O1S8FR2IKUlBIVqgpp8JBEEFqvZIpNgSUiXJMtPCTeQD/FHife5VyLYd8j
mZ8S88o3rccm2ms7btfXy5MEvbUiFF1py/GC4Lqu7ejutIlDYHYZ+whJ2Zzm
MEctuM+pl7DMIB7fYsZYCakISj1d7sMl/jsk+O9bpwpn8d99gv9e3pVD0JSX
PpKD+7b2BLxyRz2+e3EEeHkEL5ypRp9AMo3iiW+fJ81pvHl7JTY6Xf5KGqP4
QvDiyeogPLwlwcgoCvDu8Sz05oiguivewCiIzNPSVypt0pZHeU0qBeMDVZ5a
aXOJ5BoykL0WffkhrvzPg/Y/Yr/gxemCFxNxODQDx2PVmTALHFP1RnBhHcea
rGe3pSEggGSsBlcMc+GG4J2SWiISrxYtrs252gZtTX+gVQSnT5+WKzVyFH7y
ySfx9ddf698uoVmdG5yvbjduQlZKBj8ZFdJr60YkyLqpjyd9ba7624zMSoQH
RyA7KZGPBSnKHhJ/6mtoRL4tAbkWf2XdhrbTa54iKrc7UBjnhZGiEI4QupmL
Qh9OdyagLkEEmv4wQ6ChMHxe3LQDmUvx7k5lkecNUiy+1BuI8cJ1+PiYbaGk
5R5uDsZxeYudwoVfOpBp8PGSuTA5241UheG9u6olLryQyyofXmrAaJ0Jnz/e
g7++MExr4hnSz5xyCjoRitfPVuCvIs3zkNoVP7pUi4GSAPzurjIDpkmdPt1v
wdF29g5YYpDr/HB3byhdLEukFoLdVV44WiOtc1T7iyfDOTY/OBghcWNFvKR5
jFLLSvFDrIbIQUrdSEmUumJQUepkhsdSdUggaoqKpO3wSmyuyMtDoH8kUtLK
1EA3/L2gEV+jq35y61EDhv/85z+zpdH3jFHM1JVlDLurGOb5NHFzECWOt+Yz
c9B+9DVQ9tRrj1MsjpgpJgWm8HDUFRcqa6m1bkzlOBBlqBHpXFJUMAoTQ0XI
zTPIl9SkT+kbiceUs1Gh/Dqp5HtRELWWlDXY7+RFEnOW6D2jZq6XnW8jJyLT
Ir1lKhZbigWD7g2bWQ78rapAaJtb5kt1kUvj8dg9uy7y4tECTNRH6XWRuRKb
pRW7u3qS8e1zIo0ReCXV6OVRXDlUhkFxdbx1h9OwxF1vsC0PYkc7ivPXST/q
2QPZV1WZiU9cHDEpTmPHUzX7atq/+4S4magu9O5OpW9S3r2xp2IdJsSxfd0V
hUmF2yMOQI3NC5dGM+VZcvLzaswIQIktfKHWYCICMJXZKQDXFRdLPQ/zueeh
qawMYcHRbMlDw3LazXyN1HtIECkq7kBjUx++/fbb2UrbD2GzX9kIp/7gBYyv
lspKWE1Wdsy1Jhbw9jwVnx4GeA5zC3lCQi5HVVo4TwvmdHagIJ3U8SxrHFKi
/NFXlsy7zw4ZNnVXoLsoDk7rOlxQ97fKLGFTSZD4DFaQD77EEpSweldHoMjK
bpkRVpWG0OcnwtErqMJb++T16UoB7+ltcRgtcYkNrjWhBTjUHoU7xxLlBj9a
CHBmNBEnRpIJcW66htCBc1uycG5rLgHTXcLo5f3FGHKG410Rianxw03uh32o
CRurQ/HGiQLum7pOiuAfi0g7WOSD1w+mS2NtCl2gxqjenNXqC5LogiDu1Kwy
nHMrHuwL4H2iWpD9WBxWanynzSGPDEVKS4Rds50NyauxpTKaDfuu1xM4O+4e
TkNxgi/qssxMHK6VcNtfnsK4bSgtlYKsAp/uulpkJtkQHBgFkyVLxu5MAkHI
IvvxkIBwVJeV8u3/owZ23n79NdQVFmqNivwYc/leX4JGcX4SBPFlGCc5tAqH
zrC181NY3AazOZ0Xf2QkJIgEtEwLswZ2UF9chMQIfzjTo7BTUAFx2VwjMd2u
ogRWeo8KzijogP5WMjXoUqjBgwPKPXejWoUdtC/nUXQRXufI6dmxei/sqfGR
lzjTSP2JthAcag0jHjnfQG5zeL7yteMFhkY8skSh2EpJlki25sqNeLR/q8OK
F8/W4ruXSDIn7M7DX4kVvDSCS7scPKnz4cVaYg+yLqxsNorQNxtdLy2o214f
KqBjoYB/o5qtTZR54f4hxU1XFn5PtgRgR+kqIrc3Si4a20vXYFsxrQGOWyIt
eL6rKwxF5tU402tj4VdWy24TX6NErbMoidOPayUy0Ftqg0ngtLZYVohu5HuT
EvuOKicyEpNUrGZLWB02YKS0rBsJ1lwRj6OQbDKhuiAPh3fu+FGK2Idvvozm
vAR+HqejQMOY8kiLmJ9Qi3kDwVZE3yABW0tcDj8SPUqpOCuJibmIDA5Dri2Z
HU74z7SR3OmCaUtZEWwR3nxEieNLAhgJCEMVyQzT4x0KrdKK/yTXtKauVtRc
7Urz0CeAaQj+vV3GKKt5mlGUfVpEZ1nS1ejAsKADTytTYnPkWvLDW5MwURNK
XW9GAUwkIlXh+IO4x110QPV0vlCHsXoTvni8lxis+KoSb//6whDH3D09ibhn
mjdxzdFC9J8E1ne0ROPxXdxsd70kXJwfs2JPQzBJC0sknnJHfzSmlQWPSySF
l5z9aWviy4KxUr15iVRvfn5zKNdxyCqCiP8SqddustgLdSleTLKIeLluNjvX
hDKi1iu2KQMacOfxp9ZfnsoKQ2FGBg+36kBZrGK3hS1V0qw2hARFsYKQV9DE
QCHWSuaGhNW0+Hg0l5eyi9ZgUyMO79j+oxzaPnjjRRzocoi7LB1xYUHITUlV
SuAiNiqgW6gDt7WyAunWZC6F03bSiJBopFgs4jkr9Hg6XyqbkcyVEukjzmiy
9PIVlG5tzILDtJqt7y7rMpdCCDaXBrNP04szCIFW3u9KvQUPiHuQZFo5qJJP
yZbS9eQ/tVDiAg9ujBG00BefnlNooW5Wcns+putDeERFslgmlJ4YSMDpsRSZ
C5BCe98uQbyH0vC3F4fnMzbdgVdGcedUnmpcpX/3N4+3Y3+XWdkn94AKS2VG
8q3Thcq2ohNZ0nI4l5MPjcW8ulMRDLR4SlR1e7knjtZShmW6UXrRB6s92Yv5
1W3mJdL79NKkCW3pa9GTF4z7N9r195eSqwPNcQKWGxiCM2G5pTGXh31T4+I0
rqfA4AaGAIXUzppq5KamIzQkhndyR4bFIiE6Gg0lRfw99NGTNkXM84fgOLNR
n+C4t9PBj7KtJR+Z5jAkxZoEx6ylv3e+RjtVUaqjugpOEalbK8u1NefiL3Eh
sKO6GkkRAegotBpuDkIgOVoUWrywuTzcgEBtyISv80FlI9kSqYFyb+UG9vcT
wXGRFCepBadPfA6PC4CKW32Odq0Tedta4auMt543ZPqvHcrEYKk/Nw4YM/0q
nicfdoYpVa/H2+bIdzs13uzsTsQTx510l8/RsShi5IMHSsQrtdKdriP3qQ4c
G7Di9lErcd5Fat1gS1UQHprgxSdzluh1XAd21wbgdFugq27gwVNZz24Vn0La
LVwBk+OjJtq1pSzFudYgKcFUsnxqW3LGr8JWIqDjRiDua7RwfCSrBRmIVNGl
Hp1iWzQsERForqgg9Wq+lBMTEMkWpqogX+RRJSJmNUp9a24a+NQmgh9QWK9c
uaJXBjQE0uJxeo793cUoTzfDFBbKmyxJRVug/hDtQVywU5Q1Su4oK7JFB6Mp
h8m166WVM7ckNy3alsH5+ljm9VJdhXKkxuRVOFLry2r/Eqk+/uhIBNptv8Zz
qmooh787OvyxSTArgTg5/L2wPZ7jzQcnsw2p0Jfisz7VG4PDXbGyVxM1fN0/
lc670b7WQKTczx9cVO7nPzzGCpOKOw/il3j2tmpxoiz405UWXqempfl/MzYT
3CDdz3eOJSj3853q/ezBdauntifz4k6t4Vc2qdxavBYn69fTy71Rq8+JV35P
lx9abcvwHOnM28z6O0YV2Es9oSgx07apBIkOKfjb02BGeuQGvvyO6OFhPmuB
h/pK0eJIhDmMqlh5HHJcF5zrs3cl9UbQyZLoqVOnvk8Sfeqpp/DNN99cFX4H
1GWGZCjYXWLjfKzIbhfvrfjhi9SEbFiNuJ0iQpIJZKIId3X2WM7CdWY4j7ky
lZYcIuKRTqTQFfsNettVFgbz/cRNspKu3Bs11Vm8gROOVThUtc6lEClnXiv1
PzIeze2zehpzQkljaKvMl+cLDGkMeXaJePeyOitn3CVcicmGKDx+0MGaj3zt
3r8rHwcG+dpdoEJO/BZvXWxWt5s1zVKWHtiVg/0dVPl33mC4ex2GTYGLpQ4a
ctYbLVyPh1Q/JTn40W6E7jRtM2HMnBt1qSwGIyIr31PJytqNKu5IJjpUF4jS
+HW4vV+ZwZBxt6vezHFvuCqDr6WF6odEE0k0mZcQEQx7UjKbvYvP9gYJXkd2
7jDASyTU1BxOqvs/olZK8BL/pqCd0v3N9TlIigpFukiPO2tqtCimQE0Jbdco
UFPhRt702cmJAm6BPGJLewipgf5aaTBhuCoduTHknRk+lyKbfMNuLgtFc9Iy
cT1ESTesQqkvdAWjO/Vm/IabLWIMQe6enkAe3vnYFeRkYZL3G93pMGQiD4lM
hOYUvrjXKWciLx3NF08doXtVyBnF9i4rnjzBt+sCPQkZxmdXujBUFYHPHmjg
pkDtUtYceKcbI3RzaS3SkffdpLhqH9jM3jFyJkJ7iKcrvCiy3ShV6CeK1uJM
0wY5yNHYEq0LbLSqKo8uMdA7Zsa4Yz0a03zo/eWBWRlsO+tMDLbR6ky+ZBWw
XcNRhaxLc+IjYI2JRotg94PNzXx5jciX2lx1Mo7SiwPitpSu1H9MhpRB587/
fy3/9N2CgRXZYmAJDxE5dwwK0tPZeYAGYNTQynFWPCw9ED2Ysgutnuv1+alJ
LNdUZcSIdCJHfBrivtY1rIEK8emmIS92jW65vUhaILu9KhL14t18eMiY0mmt
IH2Zy3G+zduw4OH93TEYzV+N+xS1Q75kH92slSaNOQZVTPY0R+DCpuSZdfU7
xpNxbCiJajJuesG8DW+fr8ZGcS99+QQlwUNzFkhJ8FdP9WOsLpqXH8mkkC7n
18+WY1NVsL7pQr9pxX1+10areBOCXJmwMlrztpptvL47gYfw5Yj32KYYbk98
W/XFk1OOfZXr1F4R8416ymHhKm+TbTWGisL4pl0kgXBHbSzftMz09Ih3Lc/Z
0NRZTXa8gEEwkmKikGNLQX1JCa805c9dg8ECHQZcyxMooWkFR1oqnIW5FAX/
j1Hpwf//P9WLX2kn7Cm1oSQlhp0CrNERrBzSKBARu/7GRg6TVILKFz+fNpkl
RYchJSoA5amR2NqUyykMyeDX6gXGUt4BTVlXPqGyLpbemkVS4ruvIQZ18Tfj
vt5QZn7aNUydDMcbfDCcvUz251ZMaQMxJLElucBIQ7W39UbLBUZlt0kOBor9
9e22HmpCsKM1dtb1S3fpnVuzcGLcTjTvGgmMXz8ziG2tFi6V/+XxdleGQrXI
e2pFZhOgr6KT7+B3blPyX83Pf4kkze+o9se5zmA6XT9RpzLHC1bjjlZ2g5hz
o1RZpPVKrbZb9JzjRukCeXiIukhXYrc4R5TaKe/xPJ3zZYrkt7OYbfPoY1E+
of/BdTSi/MTPByrSUJFhRnJ0KFu+UT5MC6Up4+wXyOuuq2NtvDAzE2lxFvHp
R8AmPv28uEAcGG/Hd99993+MRqUkvYTRSLyNYuVekYZsrLWjITcedks44iNC
xWGJRHJUEOzmEFRmmNBTkoyJ+izxvUX8Ogh9C3X0lbE/Zn1WDBzmdfw23Ddu
l0Pi0bY41Ar03dURLKFPeUdJ+WpL/jUeHwmSuhXEp+NYjfO9YbTFas5CSU17
c18ievPW4w11bnC+5D5PSxsm68JU3Cmg+OD2Ut5r+LE6L2joU35Y8LxmC169
s4Gl7GvVdPfb54exp9uKJxTMzp1VJQzGe2eVKuEi6TLeWk2jktJl7MFP9ejW
RO7x+0idCtaCIRW3H9kYzRuM3tkhB0MPHpwkUy2tp5CKLzIHPNcWzKI2dTO7
IqKCQ9IGs2K9UJ9tUYl+2UL5c+9XPncaTCS/u5aCRORaI8XnHoJE/tyDkR4T
hNLUaFY3xmszsbuzEPu6CnHPwfEf08ppBJ8emfuVh6BDQeFwrDqDFxNph4SA
KQB2vYQvWphQnR4BZ+J63uUiQLVI6reid6EqfgVONvrz6u8bJbmKjAvIfv1k
w3rDpDOFuHv7AkVCuJpWaxtCHBVMTneGikDKGu/8GVx+U2UgHpo2DnpQcvuA
+NqudgtBS7t3v32iDS+frhAoi8Ofnxnge1dD2Xcvj+DIcAruZb9uQwM9NXFu
rgnHy0eUfEZTVjiF3mqjFVZqR6bL82NEMLUHRyM5w9BI3yc0CVK4RstsfyLt
HtxRugbDOavpPfqJlNRSDZqSC6qWyMCiasnpHhuK4n1RnBzOiBAf1g38IV3P
V5v6uXGQG6vJwLbmXL6B9c+0v4yvqr3ij17Yv/HHtFUagXSDmmTwnaqUKcTH
6hJ4CC3OtHAaUORBdSkakShyh8ib6hJX4kC1NxO0n0ia0o5ST0wUrKD3SUoQ
1Lx0ZzQGRV56eSRSykuVLIHWag8UrMdz1P14XotKyvl/eDIR485gvVFRvxVb
YvDwnrxZt+JtmzLVuvDIQkkLPjuZjXPjBi2YNsxvb9b9ABbJlhh3kSWGD/dT
UAFlibS5/p5REzYVrZvzE4lmPiuOSkfKzXiZhuL2yRdiLAuQ3WnLNEd39XjN
Y0ZLOcK2Um9UJHgqFnjaez2PTyb57DRlBCLLFMA9qeLcL77Kp+ehfk2Fxg9r
a9u2bZO0NSM8/kMSnWlArDgxiOso1KkpPR5B4eJIOtrT17LZCYlgukBBhyEO
t7cFsv/bq1sj+E1ZrN5NrIm1+2FcvInXq2yCYsal4WhMVPjNWXCVgPGwYTJM
NTU8U4KRqnD87r76WXcTbZseF1fp2/c0Uw/hnIWSFEstArSqRbBzDQp/UcsC
9yrDwIul7Rcn+sw43BpGuFwyo/RLQwmPbY4xxAy6lvbX+mBnCY/W/ERKFMko
q8F6My52se+pCwgCPIKyc8Wq0urJyoT+Ts/XOocEfY9AWpSXuI7T1fJi6Zz/
+yqf/D8ocD399NP4y1/+ohf0CQO7Owo4xog49K+S5La/pxTtjniRs3myoQEJ
oVQLdmEhk5+yL8eHhSrK00TicaMUGWh+qD35Zjw0EMA1d8N2JpHK9NpXircy
VlLoXaNVVMm8wparrsjAPge7UjFWEahb1MuXyd0TKSKQJVDm5rpM2vHM8VJs
a7fim+cGqeI+R+lucmPZivryFBatdpO8NIJnTlddXT19qgMnhxK1acPF0kjt
b04WCBrthfdPZBtoNMUyKnFu5UKYKiwoO1nIN5NKnLwLQlyyGmio5fTONl+m
MTSq5qIxKmjErwnx5tC9fscAg2axxmRE9CA1KzN6A5rz43mSSAT3//l38CLA
wD12/5BiNQ8fvfUSOvJjUSquM4oTdEcJsPyLihv61x1tDpQmBaFKPN9RknkF
vaJSrIwbmisdLQrS/VFfdeGGXuKLgn9QK9yxWnaTmCMvdz9cu0GTaa6Twshd
/REcRq6RaidHuqJxejDOpadr4qYTk43ReOpwoeEeoXz/6HAqLu5yUMRwU4MH
ZVcUUL4mKvL8kIekeb55semqmifN7F2csuNwt4moyBKp+3lbbQjuGTNTF5wc
WEgCpd5kdS+5HlhoH+0dXcEYzV5ObsL/Q3ojiIwMZUtkZB7n/WwvK2hNVZJ+
kS9WYwqZ8JGDfGGcD/LjQzBeZ2dWS5nJT77/NvnHljF99vZzONMVj47cUKRH
eTOTpqKeyloVDC5QFfRyJtmkSBaY1ovUJJBvlftdSKbAcs9oJvrz/LmW8Kjg
DyJ6arTjNfFCd5R5YvP/bu07wKo6s7X9bdEkmsSSMjO5c//5771z73MnmST2
XqOxIUZUVFQUCzbEQhUp0nuRJghKswB2sQv2Lnax94q9RrG8/1prl/Md1Ezy
zPg8HD3bfQ7n7O/d73pX/Zx/4KY3kR2fqtG5Wd0QNLyRmblVyYW3SWTZIcN0
FXKRpjMe9jCpI84uUUt7tAjlmSXjEef+dq2EzI/eqlX73tweyMCg/7N45s8O
RSKV7oNz633EYaqhhDDv7g6WurX7Vu9p3Vz0uGTaZwrH7coYKhvc3FvtXE8h
mRWhPem2aK2SjFGoFj2qsV71ZCEZU6IM+x7z3TSJYoUikikJzi2ltHJd9FAL
imqJqmWJwjO8nGzaYLJDL3FJdCr47J8DUm3cvXgExSTxuWSjcKY9PBw6YXjv
dgieOEj8Lw5Y1dOjpgaackNcEDl5CCb074yJfVtijruNcAzzjgGpLQna5545
oh1ChzXCQeJWBVJceVPk3wlBQ1i7dBftYvDODYJYIvnWKwMlGPKx4vKsCumG
dPfOHFqspXjVHOVO9fyJfq1l5binnUOMakkjrfOeBeO1stvjZtktp/AubPKD
z/juWBQ7lJ7HWMHmfZFHZrDTK1xlj11RzmQfP1UELg+Jjp7QDkfnDRbof6bE
oTgAFDaymYhaCwPVkC+5Z5aNJFyMTbRVfXNY1zfFIT8pnoGGopOz+yBuTAuZ
bs4dgAqK2AowH60Kc0DAyK5wtGmL6aP7Som5Inh/FUWkI+XPq+e4cOYUTp+9
iOcvtdk7vzx5iFu370E/AdcunsOJslO48+Bplff3t38s4NsUPVhC9wx0riPO
nt4XLv3bYlTfHyVHpIjwOiYOXU0cMk55+8jJ/VsjbVovGfHF8e7PTInvhLVk
t/2GthBnklvPTyvUxvGdaC2+szuqs9y1nynZ1Q3BHWWbN2OGskptVxcMIhXS
SnZc4/WtpcS95/l0w8akQWoLpPSQb5qCRK8e2l4oijFkVssNG4idC1zZqaqm
FywyMlfPGYv8xNFI8uuHKyV+RHix1StHIv1o8TkSecA6ElmudzjcWK1Fgj5T
JPeyiL5Y4CfVliaxrR+HZcE9kDm1LWOyvpJ7Tp/UQs+D2jZQrs4a/3bwG/gd
Dkr852drNJJtjR7dAt4ObYUIiBC0Nakt3MDLXUB84+7QESNs2yPEhfgmzMUo
6qtSR4dkbvAEFM+f9R5I1gVe/4KCOXFw9/SCl4c74ubk48VrY3ZsBl6+fonV
OSmY5uaJoMAAuHmSQj1349em5SiopMvJjx9KjLA41hGJU3pjlG0bCTDEe4zQ
biAzcqDvNfiRJvD1INK8wAkS9x5r1wFT7dtgrpetZKANpaYilceWzhjcBKnj
m6OMbmdSaqbvrxeA8VyYZXqkSK3TO0UsGkJGeG+irfj+Km0uD/yJaLMTG91a
irgvy3OU/RturHZR+sZqCH0enj9KmnWfWLUuaOkW7sCJo29xf69WC6HZ3w+E
LB+XhiPJ3x7l+6KwJnOCFiY4FacTKok1DlyG2mP//LEqNbM/menfG1tSpR7t
M0Xs82Rl3gPX6JRUszjcZBbq1BSHZ/exolHGLBc1Bjp8i9K4LhK2NYDLVeMZ
Lpqa441pT1UCLlePRY1sQQ55W3HICan1TBRobiLH1seRrBtj96NkdebOHC8c
pYPX0iKohZXmBjijICWU4+LvxPDNkzvgMXU6zt75Bc/unIXX1GnYd/oGnj+5
j5vldwmQR+E5ZRqOXGaKfY1VOfEISVmqYvjUqVPvxXA1efxKOjXYCDAdRo7v
hjE/E8v26YApQ22EaSth2YjMmtEx3iMg1n24lIiP7tNOwJxOlFtsgJl+Sz0z
GOckWyN72TdCxqSWopNPGWCuqVfp8c4MjWktmoiLejO7tzHIjjcxWzi9o3iq
HyshIJlEQJzCc/E4ZFRLkWQ8Qk93P2ooHsDcAFsUz3HikQgWGRCAVUmOWJoo
eZ1qOnrZF92/xA1ZUY7AlTk4uMoHuXQ/WqpxtdBGQSw5LWnD2T9V3V6OxCeT
EnioKwGDbJ9tc5MsqD50oZ4iWzgHnjyxJSHTEm5fHfQjOWPfaU05pvGvKWkf
HugZMfwHzJ7Y0ogFGRf0tA7ceFKRUwa0xLKQgRxpry/L0VCWgzV/4mQbMa4j
iXFdBndHOPESN/4p4DUrbwz0cmSCSyS8nWwxqndrBE1xxPPnL96OftHHKF03
GzPNkXxvkBszE0u2HbbMpTwIT3cvnLn9XE7YkJ+M4MQl6shW3qmNt63as2dP
1XdguYY8NhCPhe/GTcTJC/37E0F0lztyZCU8M3bZL0rwdJIv4GxHX7xfa0SN
6yq9dyXxjvw+DQzYEoyXBpMKHvADcqe2kbCK6vEJXfRFuksLxI/6Xl8kW1Uw
FPm3F8HAAfu7ix3M4BuXQeoQMINvzGAXidUiJnTApRXjrX0ggs3xRWMk+Hb/
XT7QFvKBvG1xZbMf3pg+UG3pcag4Go254UNwqGgGcDUDdw5EI54o696eEEFz
dSVmu27OGCyPH2zhYmM+ji/ipnXF0QUjJSb4meKacQmTFn1Rk5iW3bCiSINu
i1HD/Jp3dHvxEKRMaEZCorlYrwaKruU+9kCHvyNnmhbxbiBX/AO52mz55k5p
J9mkHJ++wmEWrqkvvMZYYBW5NMge8S49Mdm+nexr4kJ2OtzVALmrcFjS9FHw
d7bDuH6dML5vK4SM6oxcbxvszw+uYoxf5cAew5BHr/LflcavrvvV8atvKp6h
ICMSU7x8ER8dDpdpXthzWtMcVZmueYLLixfc02a+hQ7y2vKdPpHvw9+TudXE
d992GPVzB0wY0EVwPNGuNbFhZ8z37as59ARenoPeUAFzIWHcrd/3WOgh15XB
3ECRvgvd28nG11LYmqt0VumuRqBjIxwVF2WoScSy/WUs3WIT2shyc12lKn3z
/LsjP1RCMQYRc5v4HF8bFKdbE7FeVp5DLPv6eEw1HcKses9zfdvMQXhxMgm4
kIY3F9OwMI6+X/YEJmEdwtWEtPfmM0n1r1JDneGUNBR5M2UTh3rK5G5u4Vkb
1dtCxFr+Y2dSP0Rylm2hlmVTZcSZrP6yg9quyM7WqM3jWXxcyPUNFpOtYp/e
RC3xBNctLPftAtefG8sGypqDPKKKRjWfSu5BkEx6grOD8ZMsqB1PKzzWrqOg
039ERymXWB89RNMfcUNwsDDUACpjiLFk/BBYFaCWrk/DzJj55ojUvOgA5G8p
tUzUe4H5adEInZWJTWtXwtvTE/vO/mZx/JEe6VLBuojAmu5mI8DbFDeMbij6
38/NMPtwMUVchM+oLPBqr6GSrloDMzLQFyv9OiJw8N+k96RcgSWPWbo2z0bm
QuvRAROW5pCwcW30SLaFaHlD+UPzHBAxkcTuGha706zE7o45jjINTPp09/pX
eSvRSXdYWZGXBJQ0lH4oKF2aMgYbcyYDl+cA52fT3+ko2+CPVP9+WvhbCTKc
iMWZtd5I87XFk92+StWdvh3gJg9ETuqEKysniB0wCZc+3tEFoxA1vg1u6vuJ
Gw4bJ2+SJ7czhsfXV2TE5shuZgyBpEMV1Ws7QIKYYwj65mjEF5p146z9toju
cB/QFEHEiRvJthJqtcWrIwA2AMtbLvAqLySnlDspGcy8sPx/TMObYhwYpO/Y
7aMmrh4vwVQ3H1x8WIGKB5fg4+6BXWXXTUy+uAN/jynYePC8QDbB1w35mw/+
VkwaI2O020sjxC91+PExDnPOcukC935/xwrfTtJ/Vxl+6wha/gP/V3IFaukN
X7zlM9pKDeatAi0goLpZ68J7yD7VtC5qfu3OytGYNbUztvDGwNut82vsevGs
6BPLJinR8RpCjQcWTcDsmf0JSdGSSNFM/EcSC7+zK5iuzADcLY0BiBsFfRdm
48XpZKQEDNDmephorcH0its7A0VOGCHV6sqU3tzgvlgbP4A/Xz2lXiTVvYvM
Z1f4kkuVjmU7IGREY5zL0Xawqa8UXmVOaSWtj7dyezdUcizbwjtJgQ2XpPMe
8wbo+PY/mNAbIY7NJHDFiTlijC+VkZi8jFu1iRTV9dXV8fWeqIBmgRckhcHb
dyYCfL0RlrIIzyqUSaKvULxkLtw8fRAZEgRvv1CcvfHgt2LsvxUDy06Oz5CW
CBn2A3ZF9WD8NFSgxHPRfO3/hq1hHSUnV69SrjLI8QccT7cT51VlssvzB0ps
6dBc69iStjOcnXgaZFVrKLatMNwOhVEDeTlNJbfXX9Jqc2b2w96CydIwU810
ZWKxKWsiCpPHAJfSqzGE+DBRWUneZOTHDLM4R1oc9Am59nEevWQsKMdBqyve
z4klLohXNvL9TGHXA9nDta1gVmqja9RBXbnTO2uzQS30xUb3Uq49IeI7aW7g
O7Chcgeundme6MvSkNiwkgvE2ToegsY9oNtmOf35N0FGG9n5uuIpjpbux/5D
xwgur6tYRna+xNmyI9i1cx9u3Xvy/pCmNU7+y+AdzfRjjrutmL15k1tJ0Y/l
02tqbFtkd+Kdb/TRQBbeYbN3PcsGCWObYoVu9uoYKSu6WssCf0KGpxIV17T4
7ZVjEDe5Ew7mOr2V179WpPEOm50XSgu/JHALXcjicTVQhFQDVVUw8OhAOBL9
7XF5RxijpooGmxrCQexmxHlrbsbr49ZpGaNg8uQyLfBZ3Qqh/kjy7IZ98xyt
fA3ZDpR+Eqfy2FcpWjNLilho8siAEKem5qxw1f7tSbCRYhGewcLCVgXQEu/W
snsEd2edNpfgAwlK8DLkuZPy6tOUVNfPesRnRJWvfwuKfq3MTMXFfyq6aXXk
UAQ5tcN0+++wIaiLFHVwRErFBVe9B3C5p19bSdCqueulPro9ylftkVaIeC57
AEJHt8TJXDVcUkMqbLYm9icSkcYAKflRq7M3pgwhR8nOUrxhzSR78t9mkl35
U5ETMwJvLqTVMJjkYjryE5ywc/4k6wRvDHLC7GVkJVm5GqovkOGEDJ/uHLep
p/DHnsyhiCP+uFOJPzhevTykB9JcW3HvgBpFWejd3lIklW3In5oSZyLDJFXc
QUN/kI20CAKf6xRyVuoWu2KyXWNpHtXCf05/+r2L/2vRuP/3Di3sMaAREsc2
k4/DpPC5Yhz3xPaUMOay6a3FaphhMg3MW8M7kxFphLJMzYjUUaL28zw7yUQF
MiC1lPvmYsEIRIxvh0N5zAuWkAOHy26vn4w4t26SoLDSIzovJL+TFzjtGoHU
oEE4tXEmq+AqJgiuzMHeZZ5YTN/2zYmYakocbUn8MGycLXG0Gorevrd1OqJd
fzQLC63YoGSqsEGx1k9qKTDUdw7gYiEj8lBfsaZcIB4zqrFsm2QxJzWNsCPm
uDRFKF1Ejq+TGflcJwSuNtwb1wsRI1vIvG5OTW3RCeEP78fEb6sHUzHxF4UQ
ODUQP7ELPPp9h2U+HSUExUUplTER5PB3+j4tNUzk9FaFxcH4rggc+j32zNL2
V6uj3BmbonoimnwJ7oh8qHju94qcke71E5ZGWoVQ+f7jZg3e+fAtOvAXOti9
iOkgzooODiwhXztsKF6fT61pWIhL6ThTEoisMPLqj0QpCNLgUDxvHBbHDOSs
ak3FkV9DjnxuQC+LI69xAo87sjR2jq2ievN7UvsjzKkJLufZK+q0hkwslaQA
efPbIzpVQoIt7yuBtAlNEEZI0O9EAwnaIvyMpTN+xNR+jRE4spMoVdIXVb56
BxKMRd61axeWLl36vvqwI0eO4NWrV+/ExL8r4oF5guWMj0NLSabvjhahWeUL
BRM8UYwxsdjrnZjgAsHQ4T9ggwy2smBCdvyglwePaimDMVW3mQXahri+Vq01
qoi4WjRRZlXxdMCKSiLiyOJJmog49DZZPD8SifTgwTi6xlfIomYlETFrRl88
PRiu5JaqS6Dn0DI3ZAXasSGqqbjOtzd7I8q1s5SRMGOoAZ+5Pt1k/jx9rfpK
jIr7vHM92/G3N8wG1/7siO0hqoH3XSjPtUYHCzDmiWAyG6VaMcMXBk+Q2Sgl
HyZ2TCu49G2GOR62Rt0VLdH70UFLLxVhSszm18YBMTpKuFRDHO0RX5u5/REy
CnFF6GDpbnUnhZkztY2UdZkfsqZMlOJM90wSElxsXQkjWn7BBnFjmsge6xZb
otXPcflwLCnrddF9rDCiNdWMlDwi75XHAq6Gmfpwl2kSBWoZhkVLpAcY5GHx
U5k8Di13R1qIA16dTWGfVteYtSx+rvi6aXh+MgnxPnbkCwex3qyqlIpxonyW
Z0+zVKymUgGyPHYgFgbbcoS9nhLVZvSEObfSduhba2EUM804spn0QrBtUaXG
Yt+OiHb6TvojSFpUYpTeyJ7cHAEO32MfGRJzOWqJt8K3LcsN3rLDz7G9rKDE
LBJGNPzn9GZNPLh8BPPce2KWa0+pW9Tf9ys9Bb9Z96v5v2IndJHehJTxLbCX
WERCIiRBDeSwJN1OrknA4G+w3Ke1FXKqiryqjjmTmiNjalu+LmxxTRVCyMn1
6ij7rZCRrq14sUVRP2Oub0++S2so2d7LKzVOubbhbcekNH8ikv3stNrAd3FK
yGAcWT1DcUyqCWiu7gzDi1PJHNy/wKLk1dlUzCH+4WHT5OOq/s3D/WGIceth
7t2lDkG9sclTYM5DTJ4qPMP2qCDYhpRcVzxeO7a+Yop4km/4SG2LeYspqimi
hNtd011bIH1CYxEhdElV7LBMXejWEr7kqXBPF63Clzp2mG/YQKVMaEN80xQx
E7oZ3RCsT+r/s9h5duUg1s9kXdmMdE8zRIzrimUhg6paMmUfmkkXJqOicAck
uXaHe/9GiB3VlIyrDCNmjH9pKms7Cf2we8tujI6h+oqSXTajLaKdmxHTDJQq
wjpKvmF7nI3Zu/5Q8WSs2EfvrFely9KoASiIeot9uKgh1Y9Ie8lURbpUZp9U
gUtVAdKHwjgvTiVhdsgQXN4eKnpGmOiS5uZsy53IOkbFJLfTpMwgp3LlFLGP
HygsVEi8uDjsZ/6U9RT4XyOTGj6WyFtvilCNFsv5XI/2qtFiL4+vF0/H4wmk
bLQ+V8QtX+bFXq1lgBOHdmkZtBX5ULxdmRFBgiCajL+rXVORNZwV3axrDlrk
T/85MH2EJ5dLcTqlp8gn3psxjowk/y6/4R2xwK+/XvslFk0vWv5Q+/3EThys
XUjn8LAQDwJXpFNjaZBnOUYA+0px29bO7AI/+79pO8pobpIJLr1VNGh4Y5zM
7CfJLNW08TTi8DEtsTfNXkybNUv1wTy/XlYsxcpnlcZSV9a5vaV89swfp4dt
o5SwrREGYeUzqBJLfSSJg8MbguHpYodNuWpCgesQpmNeyEAeFKq8n9YCmBVi
b7StWgaPB+DKejcp/DJaAOspzv3q2P56LZDJVIaJ437oyiaORTPvRBs09Dsz
LqUijC/uSt+20k/OUKK1kGWpKo5sLYnSpZDzzhs1Tx/aVnwodnG2SfXTiLoK
vA4t/t3wqivwKkvqIZU8zDlnxT2yQebk9sRHTaSkNoP0GJfVGoFksoO15Pd+
rE0vnTXCyJ3KXPTUqb3gPbgFpg/8AXNdW9NX6G6Y66+U+hX+bctndBTU6RO5
1Iw279MVxEHheBvSlkPooyrNyWQSc2f8xHd47cpMNrGDOSNEZbIVMfYy2+jl
gYBqlbNTPn1wdIW7VbxfmGyZO9JDh1Riso8lT1BxOhlzE1yxfEEQMsId8fKM
aC1huafHZyGBdNWNbQEsplQnrTB2qFH7ou7MsCDMDiui+6lkZkxs5gLY/ena
jVXfKuVqJ5bxynzNSVMlVWmyrdZwq+dfTLyRlWRLsS6gPbz6f4MNQV2Z0f4g
i/KxsAJTwcFZZIvdO8BrYBNpi0idaoMird/HXGeS57WUFIMQUNxQ7F0Y/Lua
RxX00eXlxz8KVPhjcDCh0LsTfAY3w9QBzYXw0t1t2ZIKsZlpjQStdMEoaeHj
nHpbHDQQyVN6YMbQNvC0b4yw4U3oTutE72sr78+hyz8odWv8eznNzyOSeA/s
23l9VDyeS++JyJGNsWKmFkCuq5jXzTG9yOq0wtUlThIUqK1YnUWBPZEdYKMy
IMvlK0QtkcSAl9eqDFhNm1kzdzQyg+2rVK9kC2cHEfOxz2cyH4HxUhrObotC
WsI0PLu1DrMjRuParjCJHgoD0t8L40Zg54JJFvOqgXFjpjOWxg7izRlqKeTH
3ihnUnmHhScm+dUQ8ts22wHxLpZQopVP6NUJ2e7iEzY0ro2+8QJ32PGgWTav
X+hgvCOzlzpKUmsl706vVVdoK/KFrAivEsf5l/v+CP+hLTBtQDNMH9KWNHkP
qbXnsliFk0wUGLlLfs5o4Uyr75CWiPYYhhcV/2AO8smTJ98L0ermh9PqEzhd
UkI2OcetA4Idm8sH9BzUSgbt582ww2ryDhiqzIo5xAjhYzrDY2ALqeqLGdUM
S6Z3kCEip3UtwV7DH410DD0/ltxHSll5yjTn8lWKJInLLlOCcxPkeXUQDqir
xCS4NSd4ZHM9AWYYZY07NiXYyX4HUreq1ExxiJB5JyugN1YkOFgFMDlQUL7Z
G7GedCts8FHI0ghgT0FmxDCOWFU3oHlxNi7tS0JGkgde3t+EhWluOLpmhkX5
XZmDdVkuWK1NSLaKVCylTxEkPW21lSzr/FA7KbxWeVLCmtNkIt2ayN5WPPlQ
b1WKGNMCxeSA8W1rQJOnIvL2YnxpuXhKhSZzJJerzLD/BllT2optJHbQVqah
WZ3Cf/PqMUlFkQJ0698M7vYtRAXy4JyVYQ4oJgiyj8gqLHp8V3gNbqWTUVPx
TvZnT1HLWt8d+pw9ezYXVL0Tkh/I42ea7Z6j+cSlZDJXB3RB6sTW8B7YlEi8
OcGyJf3ipvB3aIxMMsqcKzmcaGO+jkOkXyt6cG9sL4Q5klM7tpFsUkXXx0gC
MBtyN86ssfRWU9vwOCV2xlRJeGUBdy21pLvRjtMMVugrzRiE0HHtcWaxsxLt
0MKSG5MHS9X0wx0+hDjLXCXmpcUkcZbOGlE5q8oe5yx/e5zfEizEaKLvfCoe
nclGQtQkPL6xBjlJrji9aaYFfSQNdy/xRCGtUKXox9XN/hL9eKRHP2pVkoYR
uhNrCZZp9YanCp0RSvL8PA+uMJ0P7VvzcCkW0CcytGRbA8WT5VzDAq/2sjnv
pUwuOLFVochKiEea8qRO7hQgpvhaX3XDVvIqHiV7uTH4Jyn183VoDjchI1r1
gc0wY3AjJI9vJe3OpQkSSMGZVBuyQjPMBpN/EGs9duwY7zv6ThB+KI8faaSt
A4rLkznqsY7s5e6YHtJdY3xU5rl/s6r1shMxEjDoG+R7tJBIMs+JfAtxzoS4
aW204pHFQyyS0EEkYTbvh2yRhFxpwXuPh41vh73zhnMviRXYSnOcZOehS+yE
KFRH9hCnV05FLN3Dkos9EWOhOiKndZnjpaOETKsWea2HirNzMDtuMi4dX4SU
iDG4cyDKEm4jwJ3cEIDssMFa4F7r4dWq/Q+EIdadVrxYJmmb+88dCMQyEqzz
A6UCr76iY5n+MjlXvWZsAwVeXAodO7Y5bugVeCq8OKbEA2C5/fBalg27HV8o
F5arorme32/Qd+Rgsgdi92dZm7qmTNcI4mfpkGZHkdeVG6gt69lXSLIsuRcu
rfAxEEVwkRjtb4rPWiOqrj4DXfdUhZ7+r5FM0PtGF3py98w3KAnpoLmsOZZE
pXQMZvREwphGmOfWFuUqYjTRtjGiB3FUa2mCtmRyNKvBybD4yZ2wNsHeiqOe
6f0UEa5dZMOql+Y0aX1yOm8LFNAPO0RtWdyJN8Qq3G0ZP6M/yvczNoyofT28
OZeGBRne2LE+EclhI/G8bJa4EZp6S8ONPRFI9usnKUJWghp2OGKndy5pJQG1
lUgbR/QjXTvjZP5oJiaTpJ6yU7Fecyp2pfa3MpZcaUX/kj1a9N1mdWNZU0Jt
3JuZNL6ptBvoszM0FH2gZ4VtsYr9WPIrlvh0NGChL1o1janSNdVdQz+mw+X3
Oq3WSPleyeGwtE8a24yM+reyNxKxZ0MFEIx0ppB5bu0IEA6VAcGeZsioFrJF
Ks8BVHlk0UxS8f42ErywpPuNPsUpSPLsgY1pI6y2HGEpvz3bWWvmP6I281cX
M5MfPwLr5k1iS/SBYbQuZ2J1QShy0ryQHe2EV4Zbed5wLROElh5qU1SrKu+W
Fz4Iu7JH829VN2PdMNsRGdO5JmBafYP7SMhzrUjkuNZ6GtNiqljIG6GMrTE9
FMlUU4JlV8mwc4ojy7WpEXT9UhH0+8jj9B/0LdImthSCoNX+73/lYn+nhEN3
EAlxM1f6hCa4kCEKRc1LX5trg5TxTTCHFYr13c/6+DDRFevjfeRbP1lnvdhH
s4ZIqavsE1MpjrAucbA+3ta/uiILOLYd49m7kjTWBjSUrfZGYsBAiQvQDV3l
AyO3fzUTO9fGIGjGCBQkO5ulQHp4IYU8nKub/ZQObe44iMeKpBFYnTiUV/fD
StNEYqd2Ias2QijLMBrPtrojx68HKdCf+Ms1UGpYDtB3D3JsLG1B3CpkLDRf
H07+xoxujJwpzSScriw0axJOh8aMbITw4Y2wP86GLcFf/5UL/a3C9WwDuNSU
uX6FT2uNbnKsqxAkuzipmYxw53medy2rLfVw9G0inVuSHO9rtdpc+MCj3ROn
dUYJj3JXuN4yWuwnnCuayrLALPlkrufxYavTRlfWo5x4TQ0ciMPcPHKZt742
8vdpOLk1Gl5THVCU6SKJW6No8PW5VMwLH4ITRZ6MGNVJZ799YfgAXuKPlNpW
3nc5yU3vlqKPq8bA2XEPH9cGpfpGy5V1QoyZJHCwWnIuGowa1QjzpzUXujSX
/ANhd77EXPfDkz5YrBlprv/8V6773/QUjeZh20nWL3lcMwQ7fKv3F/dh3lFZ
ncXivMnNeVSPZFmVpWe7xff+bHLiFvh3E1H+oXEDrB+HdTE/I9mjq9TFWOY2
aXHeZdH95cJX7J+pEvquHGcp4uGmds6hWZeDTjCKuGorq162JRreUwdjR747
+7x6HFG70ZngM0IG48w6bT6d4YZwdfzpNd5Im2Epd1fvdt5ePsmjm+y5yTXR
KrdvT3NAzIS2uLV8lNR2aNxeUyw8h0nnenTEXFLRkkQrdFAhwFNeIp1+kF7l
ShAw0iHrA9vDZ8A3SHdpZYTSqvzH74VAWVnZeyHwv/qtzwadbUi+ZweyKd8g
e3Iz6bO5reRo+DOypM2c1Bzx41toecIl+vpruqW8cDCy6S24sFzf0cn0RsnY
sQwKm9ABp8kbZalkOgi8/diiMYic0tUcXKCOWr1OhB/tQcS33prwWeld2xqA
OFJ6t/ZGikuqUUBNWfBnp1KQHueC6IDh2FHgxoBQ8cBx5PSgQZodORFrVTt6
d3cQYridbZs2QfgjtVyExGj0pM4oXztJMhYG+zOJZU7vhvwALmMb10DJWHA3
RMy4Vlge8KNUtHxu4ICk0PlsDQdMBbqZ/0rHAZt5HiGWREKKXQbugjBqrP7y
e3DwjyIb/1Mp08/pT45IcHcn6wwGI63/54orwyO2U8Y1QbJLS1Eq91QokCNE
VMB+Ig8kubFspJWvyPWwi4YjbFw7HM4baW0ItmqqOd6tG3ZkjbaExapKs0s1
sgT9xRKohRiGUs+JdkRx3hSh+9oGDsgjXL9wOhbnBWH72lgsTxvH/2+WkJPQ
mx87HPu54tw6I8o+QNJ0W4nRMiI/VrYM4RaEpeESHKv/DmuwV+/paWB+43Ey
YZMrgDdH9eBtl6wgcCF3AKLJGrDSu54tFleDQC2xBnznbQruQC77t4gf00zi
RUzY5M7/+ztQ8BsLujjIwOGwd+Hhr5VMAxcR8VS3mYO/QZFvG2lsKs+zjpwk
d0MUwTjLvZ2hCj5RNGC+b2fMctWwwMliVRWwK8Tdbssi7dgcmKNR+IZaGPoz
5of15xVQa/hZ63N5BZuG16Zp0HJJu/OnYHawg/SXWGpyPtaCBJys5B+OTLHj
R39f2x2FuLDxeHitCGX7s5AbO5IcxVQTIlcyUDRnnLaJ8qk4o5svSyaaymAA
HRr65gsbPUTQniocIyRnsRYe4gmEO+vjAegiqGrhcOZAEYg8wr4yRTDVxpMn
kDGxiVAwAeIPCkVwzDCLrDJnzhd5tpdQOWc0//wr4KCVl5gBccXvjRf8lw4O
FiVsNBZ7d5CS70zyVrmGndGgxD346e7oHxHs+D2Wz/xRi8JZwPGAwwORPaRk
gr0hizdY0wTHwoDuPDMHdzdOZr9aDSxtTx8ms5fuEVFXthuX1rohyt0GFzfN
eMtu3NoRiLjpdriklUfoIKkrgLi+OxwL40diJ9mMKztC8KxsFl6fT0NWoiu2
ryfH4sUO3DxTiNQgB1ScYpClVVVaUvYu9UJ+nKOlo07rXl6SMAzrUx3Zl6ij
TgNLc5SNt0kXNVDmkhaE9EaGRycySiQlTJyQjtqWaCf9TFwdynGDzxVJwTzM
04hmj2ss8zWJPP6g9OTfFq9R27OJS4g3h3WT1PfX/0pV+R8GOtK1OBXbrNBh
34spYVUppiTXVo1ncECJ8/VcB7xVSqKHSGDtEyVgwMMAg0e1wPHsIUrhgyG0
nFEQ2FNqP8vXuVq6eTWInC50RsSkLji9YrJVmoVb07mLcra/HTZb9epWR8Wx
KOREDsXGHFeLufhEk5HkQGQlTkJh1gwszaWXh49EethwpIYOR2ayu6Rf8GQL
nt1ai1kzh+Dh4ThGiEWKEsIubA1GeuCAStVhmheyNXci8nkc5b6AOupuf/RJ
E927YVvaECHJ+kqcibcgjnPtiI0xWsFjQ4VXVob2kiZ/Hm52V8ELu2rc+M9b
tcSP/l5yrrQ2f1RMPLP8Mp82smdlAjky2yO6C17++K/Ey18UvHB8ej25O5FO
jRA27O/YSPqX+Y5MzZeKA8pYXu7bFsHkEB8kArq/dOinClT2J9tKuGGnttG3
6YWw7lgR1ls2bb65ZpJStqAFG88vHYdokqDb5o5iv8+qZ6woyVEbxXUsWokt
WexN2lv25hMplNmzwhdzktzx+lEJCIx4eW8Trp0uxP4tKbhxZjFo+YDHJXj1
oBjzElxwriRQ6MiiU9Pw+Gg8UZW5YbSV33JytRfS/WxlL0gWSHXMzxyI44sn
attWrXFRdKp2T5wqcEaI0m7TQLmdWLDmendGKik7Lfrv8LkSluJDuR5tJZvH
6RJyEzXQ1NZuZo5YpPaQvdF5R+kE5+YSwz6jByS/+i3g0ZofP8Kbl89x5coV
/FLxmo/g8YPbuHDputYj/uYlzpUdw979B3Dr/uMqv9bppgJOk0y1zWA6+9dR
BLiQIdy11EYyvboJq2JBnVYgu8C9FSJGNpHSrEqo48Gv70Id93PvTR8kmoBr
nJ5VCnFzcULM1J9QkuFkFdFk1B1b4iKRrts7g5TMiGLAfPpVMmAfisq5fzge
8SFjcOnEIkLYVoDR93izhrZn24ieNlflg3zNiKmWzvPC7sXubLys/KPTKZgd
OBAX9Zm5qn/EPb2af6T159ZVMscLQu3IfEnRcgOlnmHDLHskKFv9GpBjuPHE
7WTXtsgj9cIYI1f5c8WvZK5ark8P1stF/qRDjrUxrxYLj7xpLYSrZhHkdiiQ
++K3Q+5jPC8/h4CAmThzS5v7cmTbEoTEZuA1gW7r8my4unsjPNgP7v4RuHTn
6a/14TLqTiT24PScrsVqiWDaGNwV0U6Npel9Fanpixk9tWCL4flp5UOcnuRd
WOPHNZdgAaHsUzPvOwQHU7Sqg93kVT9RAMfX8zy5VxHj22N/thagNBQ1Z+/n
BdjKnomvSs1gizT1F3silrB2cJmbWELTuTqmuULzwhywkcv9zLwv2cqzKZif
7IK1SyMZZ7UMTP2yHZuWhWJF2lj2uVVfKzdqGPYVTGHerOxrJXj1xvkizdeq
qwjqWyVeiCR3+3Cek5Wg5moE+hfSvbqigAQiY6qhElrnXFOUc0upPGA18YXi
nXLlVnFEN6ncKglpz5f7ayP8pgCKQ3DMYUljCVBRPX4/kIICg3FWB9KLZ09w
/8EjVDy4Cj8PD2w/dg3crJsR7oM5q3arQLIO2ZDLea0Uq7ybY1toZ5xK6fUn
4rKejKWknjiT2gvLvFohwP6vWOHdjK5xF9zO7o4b87r9Edfn8u4Nt7K6Y0Ng
W8x2aYaruXa4WzCAwFK+qD+9/50Ce9zOHyDPqstjDXLdByDXqz02xdrg8ZpR
uL/Kic6/t9KJi49Wj5Qhtcsj6P7eMplwVczTw56UuOLKynHIDrLFva0eeL7H
h4lsNz9W7JuBnTljUJQ6Am+ORxJqjoZz8v94JC6V+CI/YQSJaBLV55P5+55L
4it6PhnlhxKxcC6xS/ka4q5iDuw82sQoeroZJ/fPRcl8Vz5Pe8lHRHup2Dp/
EkoLJxFZRfIvoV8RhVdHI1BxJBwrE4fgzLIJ/GnINGuf7OV+X+yY64hVUX3w
tMS1gf5lnm6ejCvLnZHu1h6X84fgQZETAYu/P2kI+v5n8wYhmVTTmUxb3Mkf
wGJJruXdwoG4vUi7lp/o1/J+oT12x3ZFnNM3OJ38Iy9LlX+TdSHpO6+7LFVZ
YmdkT2oEvwH/gy0hssL0+3iJq+FEQldcWeH9a0irpiLNrIu/faYUAf5BuPVM
wIe967IRZsxhs4yvWr58uTq+6syJgygtKcSDiwfpHnx69XBVPLt2pA7/iz7w
k6uH8eLGEfrfA7h/YT9e3zqMihuH8Pz6IVqBF/RYDRU3j9Cxw/KsjjzW0I7d
PIyn1w7jlxtH8bL8GJ1fcesonX+oJB/Xj2+WZ9WNx/LjpJ6P4+WdMv6piZe3
yzjFffcUXvEhelZVHqvLKRV3TuEl/9/90x/g1b3T7N0+PEeS6hzePDpH1+L1
w3Nc7ffoAv1cxJsnl+iFbx5fZDA9u0I/V/HqyWVirat84CpfZzr85vlNvK64
C3pbukvv8cmv7nE+6c0D/pfl56X+9/Nr8sqrHHr65Srh9BL/EvnZQ5f00c3j
/KsJyxf44xFBaB+Ln9O/9GfaYwV/m3un5ctYvtJZ/oby7EN5pHvo/hlafLoC
t0/g9b1TfDUIefrV0p7rz4wrWIZnN4/hxc2j9L7aAmxbmUHLWyrP6sojUTK9
X8WtY/T3cTy+Yr3AL28d4X8pC2x9rJo80srQgr8igNw5t4/h9IFgiFyv60f5
X/LsY3msjSdXDqJkZa7EJPgPB64YmIR0+VsJYBlAfXX34jsgfwBBwcG49YsG
+QNbChCSUGhAnon1+PHj8lvox3ingwcP4eTps/Ksrn7s7PlLuHajXJ7VMW6N
cxdw49Ztq/NOnTmPm7fvWp1XRu9Vfue+1bEdu/bg6vWbVq89XnYKd+8/sjrv
6PEyPHj42OrYkWMn8ODRE6vXHj5yDI/0Y8Z5RavXinlRz9u7vxRPn72wOm/v
vlL88vyl9TE675cXr6xeu4fOe/HytdV5K4vW4rl+zHLeAWiHLOfxa7VD1sf0
0U7ma3fTa1HpvOUriwwg8G365s1bsy9eoHjtGhzYvRVBQdYI4Jl9/v6BuPFE
+027i+YiJGmxYVp5iwNGABSNxvt6GrrN+Fj79u0zS1PNj79nj7mLinEe4/Tc
uXPWS71jBy5cuGB1bPv27eLFqK/dunUrrl27ZnVeSUkJbt68aXWsuLgY5eXl
Vq/dtGkT7ty5Y738RUW4f/++1Xl8/zx48MDqPB4Q9/jx47eOPXliDTHeKYL3
q7Rafno/3sdSPY8DzDzLSz2Pj+nz4qyOcX2S+tolS5ZIDNJq+el+tyw/2ODR
IXX5X6N4ZR4mOo/G2Gn+uHjbsvwvH11DgIcb1u0jqnzxGLP8PJG3odRY6vPn
z2P+/PkS8qymPy5YsACLFi2SZ3WVY/n5+fKsjn6MX2ccq6scKygosDovLy8P
hYWF7z1mvDY3NxeLFy+2Oo+P8SV53zHjtTk5OXI51fOysrLMY3WVY7yM6nlE
pe88ptOt+Vr1mHoeQ6Xyecaxd52nHmP4VH6tcexd51XTbnxZfnq/ShbgzL6N
mOYegHPlv+iufw3sL15Gxzzh7+uNgPBE3Hz0oooQwWDttQYSbly/jmcvXltA
RsduXrmA4ydO4tGzCu01Dtpr5N9jdfz9n/8PAyycfA==\
\>"]]
}, Open  ]]
}, Open  ]],

Cell["Value differentiation demo", "Text",
 CellChangeTimes->{{3.699383757107028*^9, 3.699383769153255*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"prms", ",", 
      RowBox[{"n", "=", "21"}], ",", "dims", ",", "out", ",", "h", ",", 
      "valueVariation"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"dims", "=", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"2", "n"}], ",", "n"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"h", "=", 
      RowBox[{"1", "/", "n"}]}], ";", "\[IndentingNewLine]", 
     "\[IndentingNewLine]", 
     RowBox[{"prms", "=", 
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{
        "Setup", " ", "a", " ", "very", " ", "basic", " ", "fast", " ", 
         "marching", " ", 
         RowBox[{"algorithm", ".", " ", "The"}], " ", "methods", " ", 
         "illustrated", " ", "here", " ", "apply", " ", "to", " ", "any", " ",
          "model", " ", "that", " ", "inputs", " ", "a", " ", 
         RowBox[{"\"\<speed\>\"", "."}]}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"\"\<dims\>\"", "\[Rule]", "dims"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<gridScale\>\"", "\[Rule]", "h"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<seeds\>\"", "\[Rule]", " ", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0.1", ",", "0.1"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0.1", ",", "0.9"}], "}"}]}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<exportValues\>\"", "\[Rule]", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<progressReportLandmarks\>\"", "\[Rule]", 
         RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{"\"\<speed\>\"", "\[Rule]", "1"}], ",", "\[IndentingNewLine]",
         "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"Forward", " ", 
          RowBox[{"differentation", ".", " ", "Compute"}], " ", "the", " ", 
          "first", " ", "variation", " ", "of", " ", "the", "  ", "values", 
          " ", "with", " ", "respect", " ", "to", " ", "a", " ", "variation", 
          " ", "in", " ", "the", " ", "speed", " ", 
          RowBox[{"function", "."}]}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{
         "Here", " ", "we", " ", "test", " ", "two", " ", "two", " ", 
          "possible", " ", "speed", " ", "variations"}], "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"\"\<costVariation\>\"", "\[Rule]", 
         RowBox[{"Array", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"{", 
             RowBox[{"1", ",", " ", 
              RowBox[{"(*", 
               RowBox[{"First", " ", 
                RowBox[{"variation", ":", " ", 
                 RowBox[{"increase", " ", "speed", " ", "uniformly"}]}]}], 
               "*)"}], "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"2", 
                   RowBox[{"n", "/", "3"}]}], "\[LessEqual]", "#1"}], " ", "&&",
                  " ", 
                 RowBox[{"#1", "<=", 
                  RowBox[{"4", 
                   RowBox[{"n", "/", "3"}]}]}]}], ",", "1", ",", "0"}], 
               "]"}]}], " ", 
             RowBox[{"(*", 
              RowBox[{
               RowBox[{"Second", " ", "variation"}], ",", " ", 
               RowBox[{
               "increase", " ", "speed", " ", "in", " ", "a", " ", "central", 
                " ", "band", " ", "only"}]}], "*)"}], "\[IndentingNewLine]", 
             "}"}], "&"}], ",", "dims"}], "]"}]}], ",", "\[IndentingNewLine]",
         "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"Backward", " ", 
          RowBox[{"differentiation", ".", " ", "Compute"}], " ", "the", " ", 
          "sensitivity", " ", "of", " ", "the", " ", "values", " ", "a", " ", 
          "certain", " ", "points", " ", 
          RowBox[{"w", ".", "r", ".", "t"}], " ", "variations", " ", "of", 
          " ", "the", " ", "speed", " ", 
          RowBox[{"function", "."}]}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"\"\<inspectSensitivity\>\"", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"1.5", ",", "0.7"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"1.6", ",", "0.5"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"1.2", ",", "0.2"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"1.3", ",", "0.9"}], "}"}]}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"optionally", ",", " ", 
          RowBox[{"group", " ", "some", " ", "points", " ", "together"}]}], 
         "*)"}], "\[IndentingNewLine]", 
        RowBox[{"\"\<inspectSensitivityLengths\>\"", "->", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "2"}], "}"}]}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"optionally", ",", " ", 
          RowBox[{
          "define", " ", "weights", " ", "for", " ", "the", " ", "different", 
           " ", "points"}]}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"\"\<inspectSensitivityWeights\>\"", "\[Rule]", 
         RowBox[{"{", 
          RowBox[{"1", ",", "1", ",", "0.3", ",", "0.7"}], "}"}]}]}], 
       "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"out", "=", 
      RowBox[{"RunExec", "[", 
       RowBox[{"\"\<FileHFM_Isotropic2\>\"", ",", "prms"}], "]"}]}], ";", 
     "  ", 
     RowBox[{"(*", 
      RowBox[{"FileHFM_Isotropic", " ", "also", " ", "fits"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"\"\<log\>\"", "/.", "out"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"\"\<values\>\"", "/.", "out"}], ",", 
        RowBox[{"{", 
         RowBox[{"PlotLabel", "\[Rule]", "\"\<values\>\""}], "}"}]}], "]"}], 
      "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"valueVariation", "=", 
      RowBox[{"\"\<valueVariation\>\"", "/.", "out"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"valueVariation", "[", 
         RowBox[{"[", 
          RowBox[{";;", ",", ";;", ",", "1"}], "]"}], "]"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
         "PlotLabel", "\[Rule]", 
          "\"\<Modifying cost uniformly scales the values uniformly\>\""}], 
         "}"}]}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"valueVariation", "[", 
         RowBox[{"[", 
          RowBox[{";;", ",", ";;", ",", "2"}], "]"}], "]"}], ",", " ", 
        RowBox[{"{", 
         RowBox[{
         "PlotLabel", "\[Rule]", 
          "\"\<Modifying speed in a central band only\>\""}], "}"}]}], "]"}], 
      "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"\"\<costSensitivity_0\>\"", "/.", "out"}], ",", 
        RowBox[{"{", 
         RowBox[{"PlotLabel", "\[Rule]", "\"\<A simple geodesic\>\""}], 
         "}"}]}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"\"\<costSensitivity_1\>\"", "/.", "out"}], ",", 
        RowBox[{"{", 
         RowBox[{"PlotLabel", "\[Rule]", "\"\<A double geodesic\>\""}], 
         "}"}]}], "]"}], "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Print", "[", 
      RowBox[{"TablePlot", "[", 
       RowBox[{
        RowBox[{"\"\<costSensitivity_2\>\"", "/.", "out"}], ",", 
        RowBox[{"{", 
         RowBox[{"PlotLabel", "\[Rule]", "\"\<Two geodesics\>\""}], "}"}]}], 
       "]"}], "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]], "Input",
 CellChangeTimes->{{3.6993838257473793`*^9, 3.699384017378907*^9}, {
  3.699384071207568*^9, 3.699384074188665*^9}, {3.699385676480459*^9, 
  3.699385860237097*^9}, {3.6993860567907476`*^9, 3.699386658198286*^9}, {
  3.6994630512679367`*^9, 3.6994630569696493`*^9}, {3.699463169334442*^9, 
  3.699463249095566*^9}, {3.69981969111069*^9, 3.6998196940115128`*^9}, {
  3.720187344772634*^9, 3.720187346120771*^9}, {3.720187397065546*^9, 
  3.720187397375668*^9}, {3.729540862717638*^9, 3.729540870233613*^9}, {
  3.7295409081987457`*^9, 3.729540939922349*^9}, {3.7295410067300653`*^9, 
  3.729541023125242*^9}}],

Cell[CellGroupData[{

Cell[BoxData["\<\"Field verbosity defaults to 1\\nField arrayOrdering \
defaults to RowMajor\\nField origin defaults to {0,0}\\nField sndOrder \
defaults to 0\\nField showProgress defaults to 0\\nFast marching solver \
completed in 0.000201 s.\\nField exportActiveNeighs defaults to 0\\nField \
exportGeodesicFlow defaults to 0\"\>"], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024508893*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztWOk71H0fdUuhCcmUFllKm7qlvUEdpTKVSkIqZSlapHpaNLQp2Z4kiltE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   "], {{0, 0}, {42, 21}}, {0, 1.914492423555992}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"values\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024557914*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztWPs3lHsfnReF5Balixilknqle5NqK8VUKhVSKZciHamOStQpJOQkpTgu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   "], {{0, 0}, {42, 21}}, {0, 1.9144924235559917`}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox[
   "\"Modifying cost uniformly scales the values uniformly\"", 
    TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024614373*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztmPs31Pkfx0cXthbfVC756tsU0V2+FG14TsmtFKmW0DbVV4uEqM1XaFIq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   "], {{0, 0}, {42, 21}}, {
   0, 0.7600929453715038}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox[
   "\"Modifying speed in a central band only\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024709017*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt1f831XccB/BbJFa6lRTSbawMybdOJTPPu2Jzs62UFCqTL9Muk8saIUm+
xErWF+W6937ul8/nfU7iminUVgmbRPkyW46StntPTaGSltKxzw/7E1b98nmf
8zqPX1/v53mf59tmR/yGqMk8Hs+cHUd2KDVMtiYQcHJycnJycnJycnJycnJy
cnJy/h82T14kXMvqdzvN3IlV+qH8qn4XgbGdWXIIq6AhIy80nsAocPdmOo7g
mrDpZM9XBKJzd8xTogmil/K6b4URhExY/OAcRJARfGhnuB/BuGTYxX45QWj8
9rpBK4I668KbLf8wqH44nU67ymC8oHNZyCEG/NwxnUjIIIl5KDrXR8MkAmR/
JI3He6ebT7RqEGeTLbw1W4OU8vc7BK5q/P3yfuJsgQpV/ZLDXt0UtHlDHrIS
OfR2QwmuL0pwtnrv6cfVJ/Ak/KC3qroIPPuzEe31uaCm2c64lJb02vP0qDio
OcLm9byfSfiGzUsSPtXFaCeBeWUsPyqcQHw5ZZ/nJoISg0/DR1YT6DMrDNrs
CdzGVF06Q4J1HRef+HczkBnGFQpOMrBWZ9XK/RmEFQ1JNgzRyCkz6VuUQUPl
UbYq/5UG2plJDLNTg2tdF9K/bVSDdz/JdoKvxlNBi0viOhUyvrccjM1VYjQx
8vH8ixS8K1oaR58p8NRBumf6UTlM+3a7DcTIYKXTKu8dL0XMwtSF9V6l4JOy
tWY5pbg944RvLyWDcM2rrS+uKCCtdc/pO6bEkZKLtLup5rXneUe1MEAQTJDl
dKHcnX1Xl/gRdxsXE2wPVmoDRxjM+aM9orOCgfGpWWkjQQwML4t//fIBjY8C
7KKGY2ns22NKf3Fbg3cqpgiV0OCAjVXNsWI10qyKBZMGVCh5sj5A7KWCPjUz
6AF7L/vUUcXYGIXNoStOL5dQ8MhOl9gZUvi47b5oQKzAidhbWh8rBao6nD6I
tVZgpuf1Lb8nK2A7d8nmTZYUfPmTcrM0FCKv50UuCFXCevFfN9NWquBjEVTQ
wL7fqM/mpcd6avCA9N/tWUNDSEkC7qxk95/1TOw3zrz2PJfUL3jhlsmg8F5R
2gQY6Ge3DmwspvHb12F1NR0aODJT/MxequFftigkWKBGTF96jN5XhUety8TO
qUqojPjZ6b9Q8Dc/nFjrTMHpvZ4/0yUKZEifHq/skqNZayHali1H/g3PobI8
ORraKyXne+VobHtk2JGiwJ4pS5d3LaVQt//Grp4LFF5Gfr7EL1yJftuyhB/n
qRDdYL7D/aYKM7JFWZRUjf7R5w66EA2Uplqyik/DusRhalYVjYKzIptSdn8e
kfVWVTFIeZ4arTYhSJ7oCdziQ+Cqlomdo95cj/L+O/tWePXnvwqBMD26qJOJ
Q+2gm/3R+cnQlMd3al0z4VFZn2VRnQMfn169R8p3GM+UXTFzK0KN0iVwm8Mx
DMefO19oUYy4SKOOd0dOoancwNFYW4r2xKbhrjUKPNoYvS2Y7T3jrrlzPIeV
cN1Ofr48qIJh9Bb1vRtqtFUeiI88roFBsM5S6k0jrmW97+FWGtOu/LS6je1Z
u64iXZiMQdOCyZ9U6hjEdJeckVgS1DQ75hd4E2wNoHpG2B4/ubiiWcrmJ1Ge
6fZne8zv4TLxgV1v/7/i5HyT/gvZjIJQ
   "], {{0, 0}, {42, 21}}, {
   0, 0.045786739472368554`}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"A simple geodesic\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024732415*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJztl3k0lfvbxgmnbA1KMlZExsyZQreOITIk0mQoyZB5psxDkpOSJMq2nwGp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   "], {{0, 0}, {42, 21}}, {
   0, 0.038073261046674876`}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"A double geodesic\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024751433*^9}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzt0m0sFHAcB/CjjmgSaVnclV1HO1mTXbjOvuTUCq2dVo1qw4iyWCSaRld5
KPWC83DP/8uLoskmpkakeSwxlJCiDGfFPC1GLOqd5V0bbf/v9ttn+7367buf
TVCkOESfwWBsXRzekmZL0YJK/Z+Mqe8tLmnLRIUHa1KxU4oFR9+eYIkU4hoy
5zojRfVPZiT/RhYO2w1Vj1plozLV0eJxXTYy+g2LohJzsMf2bdgWUS62N7Yy
3cxl4DTf8YoeksHFb/6ydbUc04Uz4xKlAhbnLC06Y5XQb08N3e+rgnH+qCzP
Wo0Y/oiV+Vc1/A1E0UFyDbbxb/P32RBMfRJsjLhIIHy/19+5gGCTs3nFwEeC
DRE72K8WyKr3ttywxsBqsY5A6mVgOtdF4OH0rie8jMDz4DevjrsEH6Szg+dP
E7iHdSQGsAnm8q/HCys1KN08wst00aAZR8wFRWok6XGSxrlqXHsjSysmKkR4
dHo+4qiwK1xyhvdUiYYu8vyZnxKxpbVfpo2UGHRwWCjsVmD4nu7Ey9cKGDjr
bBx7FciZnS9nsZQQyGOnK9KUmIjnl5/lqaBlFlaFrFejLbYv6jhLg6OChR8n
3dZen+nJSYGHTLX4PDE2UW+ixeCg2bE8Yy10paIneUwtqlqS2a6Lf9AaFSCo
mVy8f1jOlQ0QpLOmbPXaCRhN9t0HygkexjVxk1QECXYpzPk4AtNoiVO3D0HF
1bIyF0uC3JvC2tASDRqE3pcqORqILpjez0lQ40p4laHhCxVSdteZWPYrcWos
Qdv0XQFFQRaT3SnHmBmPO/9AhvZ1RuNi71xk+GSWMFqyUWPrHtxnn7Xq/VGp
VCqVSqVSqVQqlUqlUv/I+J1U4d+9tcz0FfYrKf3HrpzV7pFKXUv+AmFgFyY=

   "], {{0, 0}, {42, 21}}, {0, 0.03333333333333336}],
  Frame->Automatic,
  FrameLabel->{None, None},
  FrameTicks->{{None, None}, {None, None}},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultPlotStyle" -> Automatic},
  PlotLabel->FormBox["\"Two geodesics\"", TraditionalForm]]], "Print",
 CellChangeTimes->{{3.6993865151440363`*^9, 3.699386545132873*^9}, {
   3.699386577166647*^9, 3.699386659746543*^9}, 3.699386719203549*^9, 
   3.6994630629170628`*^9, 3.699819694765341*^9, {3.7201873209373198`*^9, 
   3.720187346808228*^9}, 3.7295408741360283`*^9, 3.729540927373258*^9, 
   3.729541024772751*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{740, 755},
WindowMargins->{{Automatic, 30}, {Automatic, 0}},
FrontEndVersion->"11.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (September \
21, 2016)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 131, 1, 148, "Title"],
Cell[713, 25, 106, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[844, 30, 4842, 111, 663, "Input"],
Cell[CellGroupData[{
Cell[5711, 145, 655, 9, 158, "Print"],
Cell[6369, 156, 434, 6, 44, "Print"],
Cell[6806, 164, 137368, 2308, 199, 60096, 1041, "CachedBoxData", "BoxData", \
"Print"]
}, Open  ]]
}, Open  ]],
Cell[144201, 2476, 108, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[144334, 2481, 8625, 194, 999, "Input"],
Cell[CellGroupData[{
Cell[152984, 2679, 657, 9, 158, "Print"],
Cell[153644, 2690, 4379, 76, 210, "Print"],
Cell[158026, 2768, 4448, 78, 212, "Print"],
Cell[162477, 2848, 3488, 62, 212, "Print"],
Cell[165968, 2912, 2417, 44, 212, "Print"],
Cell[168388, 2958, 6028, 103, 212, "Print"],
Cell[174419, 3063, 1643, 31, 212, "Print"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

